@@ -2,7 +2,7 @@
  * JFreeChart : a free chart library for the Java(tm) platform
  * ===========================================================
  *
- * (C) Copyright 2000-2006, by Object Refinery Limited and Contributors.
+ * (C) Copyright 2000-2007, by Object Refinery Limited and Contributors.
  *
  * Project Info:  http://www.jfree.org/jfreechart/index.html
  *
@@ -27,7 +27,7 @@
  * -----------------
  * ChartFactory.java
  * -----------------
- * (C) Copyright 2001-2006, by Object Refinery Limited and Contributors.
+ * (C) Copyright 2001-2007, by Object Refinery Limited and Contributors.
  *
  * Original Author:  David Gilbert (for Object Refinery Limited);
  * Contributor(s):   Serge V. Grachov;
@@ -40,8 +40,6 @@
  *                       Science);
  *                   Benoit Xhenseval;
  *
- * $Id: ChartFactory.java,v 1.22.2.7 2006/08/23 08:38:47 mungady Exp $
- *
  * Changes
  * -------
  * 19-Oct-2001 : Version 1, most methods transferred from JFreeChart.java (DG);
@@ -105,11 +103,18 @@
  * 17-Mar-2005 : Added createRingPlot() method (DG);
  * 21-Apr-2005 : Replaced Insets with RectangleInsets (DG);
  * 29-Nov-2005 : Removed signal chart (DG);
- * ------------- JFREECHART 1.0.0 ---------------------------------------------
+ * ------------- JFREECHART 1.0.x ---------------------------------------------
  * 26-Jan-2006 : Corrected API docs for createScatterPlot() (DG);
  * 23-Aug-2006 : Modified createStackedXYAreaChart() to use 
  *               StackedXYAreaRenderer2, because StackedXYAreaRenderer doesn't
  *               handle negative values (DG);
+ * 27-Sep-2006 : Update createPieChart() method for deprecated code (DG);
+ * 29-Nov-2006 : Update createXYBarChart() to use a time based tool tip 
+ *               generator is a DateAxis is requested (DG);
+ * 17-Jan-2007 : Added createBoxAndWhiskerChart() method from patch 1603937
+ *               submitted by Darren Jung (DG);
+ * 10-Jul-2007 : Added new methods to create pie charts with locale for
+ *               section label and tool tip formatting (DG);
  *
  */
 
@@ -121,6 +126,7 @@ import java.text.DateFormat;
 import java.text.NumberFormat;
 import java.util.Iterator;
 import java.util.List;
+import java.util.Locale;
 
 import org.jfree.chart.axis.CategoryAxis;
 import org.jfree.chart.axis.CategoryAxis3D;
@@ -129,6 +135,7 @@ import org.jfree.chart.axis.NumberAxis;
 import org.jfree.chart.axis.NumberAxis3D;
 import org.jfree.chart.axis.Timeline;
 import org.jfree.chart.axis.ValueAxis;
+import org.jfree.chart.labels.BoxAndWhiskerToolTipGenerator;
 import org.jfree.chart.labels.HighLowItemLabelGenerator;
 import org.jfree.chart.labels.IntervalCategoryToolTipGenerator;
 import org.jfree.chart.labels.ItemLabelAnchor;
@@ -156,6 +163,7 @@ import org.jfree.chart.renderer.WaferMapRenderer;
 import org.jfree.chart.renderer.category.AreaRenderer;
 import org.jfree.chart.renderer.category.BarRenderer;
 import org.jfree.chart.renderer.category.BarRenderer3D;
+import org.jfree.chart.renderer.category.BoxAndWhiskerRenderer;
 import org.jfree.chart.renderer.category.CategoryItemRenderer;
 import org.jfree.chart.renderer.category.GanttRenderer;
 import org.jfree.chart.renderer.category.LineAndShapeRenderer;
@@ -188,6 +196,7 @@ import org.jfree.data.category.IntervalCategoryDataset;
 import org.jfree.data.general.DefaultPieDataset;
 import org.jfree.data.general.PieDataset;
 import org.jfree.data.general.WaferMapDataset;
+import org.jfree.data.statistics.BoxAndWhiskerCategoryDataset;
 import org.jfree.data.statistics.BoxAndWhiskerXYDataset;
 import org.jfree.data.xy.IntervalXYDataset;
 import org.jfree.data.xy.OHLCDataset;
@@ -208,6 +217,36 @@ import org.jfree.util.TableOrder;
  */
 public abstract class ChartFactory {
 
+    /**
+     * Creates a pie chart with default settings.
+     * <P>
+     * The chart object returned by this method uses a {@link PiePlot} instance 
+     * as the plot.
+     *
+     * @param title  the chart title (<code>null</code> permitted).
+     * @param dataset  the dataset for the chart (<code>null</code> permitted).
+     * @param legend  a flag specifying whether or not a legend is required.
+     * @param tooltips  configure chart to generate tool tips?
+     * @param locale  the locale (<code>null</code> not permitted).
+     *
+     * @return A pie chart.
+     * 
+     * @since 1.0.7
+     */
+    public static JFreeChart createPieChart(String title, PieDataset dataset,
+            boolean legend, boolean tooltips, Locale locale) {
+
+        PiePlot plot = new PiePlot(dataset);
+        plot.setLabelGenerator(new StandardPieSectionLabelGenerator(locale));
+        plot.setInsets(new RectangleInsets(0.0, 5.0, 5.0, 5.0));
+        if (tooltips) {
+            plot.setToolTipGenerator(new StandardPieToolTipGenerator(locale));
+        }
+        return new JFreeChart(title, JFreeChart.DEFAULT_TITLE_FONT, plot, 
+                legend);
+
+    }
+
     /**
      * Creates a pie chart with default settings.
      * <P>
@@ -232,8 +271,7 @@ public abstract class ChartFactory {
         plot.setLabelGenerator(new StandardPieSectionLabelGenerator());
         plot.setInsets(new RectangleInsets(0.0, 5.0, 5.0, 5.0));
         if (tooltips) {
-            plot.setToolTipGenerator(new StandardPieToolTipGenerator(
-                    StandardPieToolTipGenerator.DEFAULT_SECTION_LABEL_FORMAT));
+            plot.setToolTipGenerator(new StandardPieToolTipGenerator());
         }
         if (urls) {
             plot.setURLGenerator(new StandardPieURLGenerator());
@@ -243,6 +281,125 @@ public abstract class ChartFactory {
 
     }
 
+    /**
+     * Creates a pie chart with default settings that compares 2 datasets.  
+     * The colour of each section will be determined by the move from the value
+     * for the same key in <code>previousDataset</code>. ie if value1 > value2 
+     * then the section will be in green (unless <code>greenForIncrease</code> 
+     * is <code>false</code>, in which case it would be <code>red</code>).  
+     * Each section can have a shade of red or green as the difference can be 
+     * tailored between 0% (black) and percentDiffForMaxScale% (bright 
+     * red/green).
+     * <p>
+     * For instance if <code>percentDiffForMaxScale</code> is 10 (10%), a 
+     * difference of 5% will have a half shade of red/green, a difference of 
+     * 10% or more will have a maximum shade/brightness of red/green.
+     * <P>
+     * The chart object returned by this method uses a {@link PiePlot} instance
+     * as the plot.
+     * <p>
+     * Written by <a href="mailto:opensource@objectlab.co.uk">Benoit 
+     * Xhenseval</a>.
+     *
+     * @param title  the chart title (<code>null</code> permitted).
+     * @param dataset  the dataset for the chart (<code>null</code> permitted).
+     * @param previousDataset  the dataset for the last run, this will be used 
+     *                         to compare each key in the dataset
+     * @param percentDiffForMaxScale scale goes from bright red/green to black,
+     *                               percentDiffForMaxScale indicate the change 
+     *                               required to reach top scale.
+     * @param greenForIncrease  an increase since previousDataset will be 
+     *                          displayed in green (decrease red) if true.
+     * @param legend  a flag specifying whether or not a legend is required.
+     * @param tooltips  configure chart to generate tool tips?
+     * @param locale  the locale (<code>null</code> not permitted).
+     * @param subTitle displays a subtitle with colour scheme if true
+     * @param showDifference  create a new dataset that will show the % 
+     *                        difference between the two datasets. 
+     *
+     * @return A pie chart.
+     * 
+     * @since 1.0.7
+     */
+    public static JFreeChart createPieChart(String title, PieDataset dataset,
+            PieDataset previousDataset, int percentDiffForMaxScale,
+            boolean greenForIncrease, boolean legend, boolean tooltips,
+            Locale locale, boolean subTitle, boolean showDifference) {
+
+        PiePlot plot = new PiePlot(dataset);
+        plot.setLabelGenerator(new StandardPieSectionLabelGenerator(locale));
+        plot.setInsets(new RectangleInsets(0.0, 5.0, 5.0, 5.0));
+
+        if (tooltips) {
+            plot.setToolTipGenerator(new StandardPieToolTipGenerator(locale));
+        }
+
+        List keys = dataset.getKeys();
+        DefaultPieDataset series = null;
+        if (showDifference) {
+            series = new DefaultPieDataset();
+        }
+
+        double colorPerPercent = 255.0 / percentDiffForMaxScale;
+        for (Iterator it = keys.iterator(); it.hasNext();) {
+            Comparable key = (Comparable) it.next();
+            Number newValue = dataset.getValue(key);
+            Number oldValue = previousDataset.getValue(key);
+
+            if (oldValue == null) {
+                if (greenForIncrease) {
+                    plot.setSectionPaint(key, Color.green);
+                } 
+                else {
+                    plot.setSectionPaint(key, Color.red);
+                }
+                if (showDifference) {
+                    series.setValue(key + " (+100%)", newValue);
+                }
+            }
+            else {
+                double percentChange = (newValue.doubleValue() 
+                        / oldValue.doubleValue() - 1.0) * 100.0;
+                double shade
+                    = (Math.abs(percentChange) >= percentDiffForMaxScale ? 255
+                    : Math.abs(percentChange) * colorPerPercent);
+                if (greenForIncrease 
+                        && newValue.doubleValue() > oldValue.doubleValue()
+                        || !greenForIncrease && newValue.doubleValue() 
+                        < oldValue.doubleValue()) {
+                    plot.setSectionPaint(key, new Color(0, (int) shade, 0));
+                }
+                else {
+                    plot.setSectionPaint(key, new Color((int) shade, 0, 0));
+                }
+                if (showDifference) {
+                    series.setValue(key + " (" + (percentChange >= 0 ? "+" : "")
+                            + NumberFormat.getPercentInstance().format(
+                            percentChange / 100.0) + ")", newValue);
+                }
+            }
+        }
+
+        if (showDifference) {
+            plot.setDataset(series);
+        }
+
+        JFreeChart chart =  new JFreeChart(title, 
+                JFreeChart.DEFAULT_TITLE_FONT, plot, legend);
+
+        if (subTitle) {
+            TextTitle subtitle = null;
+            subtitle = new TextTitle("Bright " + (greenForIncrease ? "red" 
+                    : "green") + "=change >=-" + percentDiffForMaxScale 
+                    + "%, Bright " + (!greenForIncrease ? "red" : "green") 
+                    + "=change >=+" + percentDiffForMaxScale + "%", 
+                    new Font("SansSerif", Font.PLAIN, 10));
+            chart.addSubtitle(subtitle);
+        }
+
+        return chart;
+    }
+
     /**
      * Creates a pie chart with default settings that compares 2 datasets.  
      * The colour of each section will be determined by the move from the value
@@ -297,8 +454,7 @@ public abstract class ChartFactory {
         plot.setInsets(new RectangleInsets(0.0, 5.0, 5.0, 5.0));
 
         if (tooltips) {
-            plot.setToolTipGenerator(new StandardPieToolTipGenerator(
-                    StandardPieToolTipGenerator.DEFAULT_SECTION_LABEL_FORMAT));
+            plot.setToolTipGenerator(new StandardPieToolTipGenerator());
         }
         if (urls) {
             plot.setURLGenerator(new StandardPieURLGenerator());
@@ -315,14 +471,13 @@ public abstract class ChartFactory {
             Comparable key = (Comparable) it.next();
             Number newValue = dataset.getValue(key);
             Number oldValue = previousDataset.getValue(key);
-            int section = dataset.getIndex(key);
 
             if (oldValue == null) {
                 if (greenForIncrease) {
-                    plot.setSectionPaint(section, Color.green);
+                    plot.setSectionPaint(key, Color.green);
                 } 
                 else {
-                    plot.setSectionPaint(section, Color.red);
+                    plot.setSectionPaint(key, Color.red);
                 }
                 if (showDifference) {
                     series.setValue(key + " (+100%)", newValue);
@@ -338,13 +493,13 @@ public abstract class ChartFactory {
                         && newValue.doubleValue() > oldValue.doubleValue()
                         || !greenForIncrease && newValue.doubleValue() 
                         < oldValue.doubleValue()) {
-                    plot.setSectionPaint(section, new Color(0, (int) shade, 0));
+                    plot.setSectionPaint(key, new Color(0, (int) shade, 0));
                 }
                 else {
-                    plot.setSectionPaint(section, new Color((int) shade, 0, 0));
+                    plot.setSectionPaint(key, new Color((int) shade, 0, 0));
                 }
                 if (showDifference) {
-                    series.setValue(key + " (" + (percentChange >= 0 ? "+" : "") 
+                    series.setValue(key + " (" + (percentChange >= 0 ? "+" : "")
                             + NumberFormat.getPercentInstance().format(
                             percentChange / 100.0) + ")", newValue);
                 }
@@ -355,9 +510,8 @@ public abstract class ChartFactory {
             plot.setDataset(series);
         }
 
-        JFreeChart chart =  new JFreeChart(
-            title, JFreeChart.DEFAULT_TITLE_FONT, plot, legend
-        );
+        JFreeChart chart =  new JFreeChart(title, 
+                JFreeChart.DEFAULT_TITLE_FONT, plot, legend);
 
         if (subTitle) {
             TextTitle subtitle = null;
@@ -372,6 +526,36 @@ public abstract class ChartFactory {
         return chart;
     }
 
+    /**
+     * Creates a ring chart with default settings.
+     * <P>
+     * The chart object returned by this method uses a {@link RingPlot} 
+     * instance as the plot.
+     *
+     * @param title  the chart title (<code>null</code> permitted).
+     * @param dataset  the dataset for the chart (<code>null</code> permitted).
+     * @param legend  a flag specifying whether or not a legend is required.
+     * @param tooltips  configure chart to generate tool tips?
+     * @param locale  the locale (<code>null</code> not permitted).
+     *
+     * @return A ring chart.
+     * 
+     * @since 1.0.7
+     */
+    public static JFreeChart createRingChart(String title, PieDataset dataset,
+            boolean legend, boolean tooltips, Locale locale) {
+
+        RingPlot plot = new RingPlot(dataset);
+        plot.setLabelGenerator(new StandardPieSectionLabelGenerator(locale));
+        plot.setInsets(new RectangleInsets(0.0, 5.0, 5.0, 5.0));
+        if (tooltips) {
+            plot.setToolTipGenerator(new StandardPieToolTipGenerator(locale));
+        }
+        return new JFreeChart(title, JFreeChart.DEFAULT_TITLE_FONT, plot, 
+                legend);
+
+    }
+
     /**
      * Creates a ring chart with default settings.
      * <P>
@@ -384,7 +568,7 @@ public abstract class ChartFactory {
      * @param tooltips  configure chart to generate tool tips?
      * @param urls  configure chart to generate URLs?
      *
-     * @return A pie chart.
+     * @return A ring chart.
      */
     public static JFreeChart createRingChart(String title,
                                              PieDataset dataset,
@@ -396,8 +580,7 @@ public abstract class ChartFactory {
         plot.setLabelGenerator(new StandardPieSectionLabelGenerator());
         plot.setInsets(new RectangleInsets(0.0, 5.0, 5.0, 5.0));
         if (tooltips) {
-            plot.setToolTipGenerator(new StandardPieToolTipGenerator(
-                    StandardPieToolTipGenerator.DEFAULT_SECTION_LABEL_FORMAT));
+            plot.setToolTipGenerator(new StandardPieToolTipGenerator());
         }
         if (urls) {
             plot.setURLGenerator(new StandardPieURLGenerator());
@@ -457,6 +640,34 @@ public abstract class ChartFactory {
 
     }
 
+    /**
+     * Creates a 3D pie chart using the specified dataset.  The chart object 
+     * returned by this method uses a {@link PiePlot3D} instance as the
+     * plot.
+     *
+     * @param title  the chart title (<code>null</code> permitted).
+     * @param dataset  the dataset for the chart (<code>null</code> permitted).
+     * @param legend  a flag specifying whether or not a legend is required.
+     * @param tooltips  configure chart to generate tool tips?
+     * @param locale  the locale (<code>null</code> not permitted).
+     *
+     * @return A pie chart.
+     * 
+     * @since 1.0.7
+     */
+    public static JFreeChart createPieChart3D(String title, PieDataset dataset,
+            boolean legend, boolean tooltips, Locale locale) {
+
+        PiePlot3D plot = new PiePlot3D(dataset);
+        plot.setInsets(new RectangleInsets(0.0, 5.0, 5.0, 5.0));
+        if (tooltips) {
+            plot.setToolTipGenerator(new StandardPieToolTipGenerator(locale));
+        }
+        return new JFreeChart(title, JFreeChart.DEFAULT_TITLE_FONT, plot, 
+                legend);
+
+    }
+
     /**
      * Creates a 3D pie chart using the specified dataset.  The chart object 
      * returned by this method uses a {@link PiePlot3D} instance as the
@@ -587,18 +798,18 @@ public abstract class ChartFactory {
         if (orientation == PlotOrientation.HORIZONTAL) {
             ItemLabelPosition position1 = new ItemLabelPosition(
                     ItemLabelAnchor.OUTSIDE3, TextAnchor.CENTER_LEFT);
-            renderer.setPositiveItemLabelPosition(position1);
+            renderer.setBasePositiveItemLabelPosition(position1);
             ItemLabelPosition position2 = new ItemLabelPosition(
                     ItemLabelAnchor.OUTSIDE9, TextAnchor.CENTER_RIGHT);
-            renderer.setNegativeItemLabelPosition(position2);
+            renderer.setBaseNegativeItemLabelPosition(position2);
          }
         else if (orientation == PlotOrientation.VERTICAL) {
             ItemLabelPosition position1 = new ItemLabelPosition(
                     ItemLabelAnchor.OUTSIDE12, TextAnchor.BOTTOM_CENTER);
-            renderer.setPositiveItemLabelPosition(position1);
+            renderer.setBasePositiveItemLabelPosition(position1);
             ItemLabelPosition position2 = new ItemLabelPosition(
                     ItemLabelAnchor.OUTSIDE6, TextAnchor.TOP_CENTER);
-            renderer.setNegativeItemLabelPosition(position2);
+            renderer.setBaseNegativeItemLabelPosition(position2);
         }
         if (tooltips) {
             renderer.setBaseToolTipGenerator(
@@ -1118,15 +1329,15 @@ public abstract class ChartFactory {
             ItemLabelPosition position = new ItemLabelPosition(
                     ItemLabelAnchor.CENTER, TextAnchor.CENTER, 
                     TextAnchor.CENTER, Math.PI / 2.0);
-            renderer.setPositiveItemLabelPosition(position);
-            renderer.setNegativeItemLabelPosition(position);
+            renderer.setBasePositiveItemLabelPosition(position);
+            renderer.setBaseNegativeItemLabelPosition(position);
          }
         else if (orientation == PlotOrientation.VERTICAL) {
             ItemLabelPosition position = new ItemLabelPosition(
                     ItemLabelAnchor.CENTER, TextAnchor.CENTER, 
                     TextAnchor.CENTER, 0.0);
-            renderer.setPositiveItemLabelPosition(position);
-            renderer.setNegativeItemLabelPosition(position);
+            renderer.setBasePositiveItemLabelPosition(position);
+            renderer.setBaseNegativeItemLabelPosition(position);
         }
         if (tooltips) {
             StandardCategoryToolTipGenerator generator 
@@ -1287,7 +1498,14 @@ public abstract class ChartFactory {
 
         XYBarRenderer renderer = new XYBarRenderer();
         if (tooltips) {
-            renderer.setBaseToolTipGenerator(new StandardXYToolTipGenerator());
+            XYToolTipGenerator tt;
+            if (dateAxis) {
+                tt = StandardXYToolTipGenerator.getTimeSeriesInstance();
+            }
+            else {
+                tt = new StandardXYToolTipGenerator();
+            }
+            renderer.setBaseToolTipGenerator(tt);
         }
         if (urls) {
             renderer.setURLGenerator(new StandardXYURLGenerator());
@@ -1835,6 +2053,39 @@ public abstract class ChartFactory {
 
     }
 
+    /**
+     * Creates and returns a default instance of a box and whisker chart
+     * based on data from a {@link BoxAndWhiskerCategoryDataset}.
+     *
+     * @param title  the chart title (<code>null</code> permitted).
+     * @param categoryAxisLabel  a label for the category axis 
+     *     (<code>null</code> permitted).
+     * @param valueAxisLabel  a label for the value axis (<code>null</code> 
+     *     permitted).
+     * @param dataset  the dataset for the chart (<code>null</code> permitted).
+     * @param legend  a flag specifying whether or not a legend is required.
+     *
+     * @return A box and whisker chart.
+     * 
+     * @since 1.0.4
+     */
+    public static JFreeChart createBoxAndWhiskerChart(String title,
+            String categoryAxisLabel, String valueAxisLabel,
+            BoxAndWhiskerCategoryDataset dataset, boolean legend) {
+        
+        CategoryAxis categoryAxis = new CategoryAxis(categoryAxisLabel);
+        NumberAxis valueAxis = new NumberAxis(valueAxisLabel);
+        valueAxis.setAutoRangeIncludesZero(false);
+        
+        BoxAndWhiskerRenderer renderer = new BoxAndWhiskerRenderer();
+        renderer.setBaseToolTipGenerator(new BoxAndWhiskerToolTipGenerator());
+           
+        CategoryPlot plot = new CategoryPlot(dataset, categoryAxis, valueAxis, 
+                renderer);
+        return new JFreeChart(title, JFreeChart.DEFAULT_TITLE_FONT, plot, 
+                legend);
+    } 
+
     /**
      * Creates and returns a default instance of a box and whisker chart.
      *
