@@ -2,7 +2,7 @@
  * JFreeChart : a free chart library for the Java(tm) platform
  * ===========================================================
  *
- * (C) Copyright 2000-2007, by Object Refinery Limited and Contributors.
+ * (C) Copyright 2000-2008, by Object Refinery Limited and Contributors.
  *
  * Project Info:  http://www.jfree.org/jfreechart/index.html
  *
@@ -27,7 +27,7 @@
  * -------------------
  * ChartUtilities.java
  * -------------------
- * (C) Copyright 2001-2007, by Object Refinery Limited and Contributors.
+ * (C) Copyright 2001-2008, by Object Refinery Limited and Contributors.
  *
  * Original Author:  David Gilbert (for Object Refinery Limited);
  * Contributor(s):   Wolfgang Irler;
@@ -70,6 +70,8 @@
  * 06-Feb-2006 : API doc update (DG);
  * 19-Mar-2007 : Use try-finally to close output stream in saveChartAsXXX() 
  *               methods (DG);
+ * 10-Jan-2008 : Fix bug 1868251 - don't create image with transparency when
+ *               saving to JPEG format (DG);
  *
  */
 
@@ -408,7 +410,8 @@ public abstract class ChartUtilities {
         if (chart == null) {
             throw new IllegalArgumentException("Null 'chart' argument.");
         }
-        BufferedImage image = chart.createBufferedImage(width, height, info);
+        BufferedImage image = chart.createBufferedImage(width, height, 
+                BufferedImage.TYPE_INT_RGB, info);
         EncoderUtil.writeBufferedImage(image, ImageFormat.JPEG, out);
 
     }
@@ -435,7 +438,8 @@ public abstract class ChartUtilities {
         if (chart == null) {
             throw new IllegalArgumentException("Null 'chart' argument.");
         }
-        BufferedImage image = chart.createBufferedImage(width, height, info);
+        BufferedImage image = chart.createBufferedImage(width, height, 
+                BufferedImage.TYPE_INT_RGB, info);
         EncoderUtil.writeBufferedImage(image, ImageFormat.JPEG, out, quality);
 
     }
