@@ -2,32 +2,32 @@
  * JFreeChart : a free chart library for the Java(tm) platform
  * ===========================================================
  *
- * (C) Copyright 2000-2006, by Object Refinery Limited and Contributors.
+ * (C) Copyright 2000-2008, by Object Refinery Limited and Contributors.
  *
  * Project Info:  http://www.jfree.org/jfreechart/index.html
  *
- * This library is free software; you can redistribute it and/or modify it 
- * under the terms of the GNU Lesser General Public License as published by 
- * the Free Software Foundation; either version 2.1 of the License, or 
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful, but 
- * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
- * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+ * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
  * License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, 
- * USA.  
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
+ * USA.
  *
- * [Java is a trademark or registered trademark of Sun Microsystems, Inc. 
+ * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
  * in the United States and other countries.]
  *
  * ---------------
  * LegendItem.java
  * ---------------
- * (C) Copyright 2000-2006, by Object Refinery Limited and Contributors.
+ * (C) Copyright 2000-2008, by Object Refinery Limited and Contributors.
  *
  * Original Author:  David Gilbert (for Object Refinery Limited);
  * Contributor(s):   Andrzej Porebski;
@@ -35,8 +35,6 @@
  *                   Wolfgang Irler;
  *                   Luke Quinane;
  *
- * $Id: LegendItem.java,v 1.9.2.5 2006/07/20 16:21:58 mungady Exp $
- *
  * Changes (from 2-Oct-2002)
  * -------------------------
  * 02-Oct-2002 : Fixed errors reported by Checkstyle (DG);
@@ -46,13 +44,17 @@
  * 21-Jan-2004 : Added the shapeFilled flag (DG);
  * 04-Jun-2004 : Added equals() method, implemented Serializable (DG);
  * 25-Nov-2004 : Changes required by new LegendTitle implementation (DG);
- * 11-Jan-2005 : Removed deprecated code in preparation for the 1.0.0 
+ * 11-Jan-2005 : Removed deprecated code in preparation for the 1.0.0
  *               release (DG);
  * 20-Apr-2005 : Added tooltip and URL text (DG);
  * 28-Nov-2005 : Separated constructors for AttributedString labels (DG);
  * 10-Dec-2005 : Fixed serialization bug (1377239) (DG);
- * ------------- JFREECHART 1.0.0 ---------------------------------------------
+ * ------------- JFREECHART 1.0.x ---------------------------------------------
  * 20-Jul-2006 : Added dataset and series index fields (DG);
+ * 13-Dec-2006 : Added fillPaintTransformer attribute (DG);
+ * 18-May-2007 : Added dataset and seriesKey fields (DG);
+ * 03-Aug-2007 : Fixed null pointer exception (DG);
+ * 23-Apr-2008 : Added new constructor and implemented Cloneable (DG);
  *
  */
 
@@ -64,6 +66,7 @@ import java.awt.Paint;
 import java.awt.Shape;
 import java.awt.Stroke;
 import java.awt.geom.Line2D;
+import java.awt.geom.Rectangle2D;
 import java.io.IOException;
 import java.io.ObjectInputStream;
 import java.io.ObjectOutputStream;
@@ -71,76 +74,98 @@ import java.io.Serializable;
 import java.text.AttributedString;
 import java.text.CharacterIterator;
 
+import org.jfree.data.general.Dataset;
 import org.jfree.io.SerialUtilities;
+import org.jfree.ui.GradientPaintTransformer;
+import org.jfree.ui.StandardGradientPaintTransformer;
 import org.jfree.util.AttributedStringUtilities;
 import org.jfree.util.ObjectUtilities;
+import org.jfree.util.PublicCloneable;
 import org.jfree.util.ShapeUtilities;
 
 /**
- * A storage object for recording the properties of a legend item, without any 
- * consideration for layout issues.  Instances of this class are immutable.
+ * A temporary storage object for recording the properties of a legend item,
+ * without any consideration for layout issues.
  */
-public class LegendItem implements Serializable {
-
-    // TODO:  keeping this class immutable is becoming a lot of overhead, need 
-    // to look at the consequences of dropping immutability
+public class LegendItem implements Cloneable, Serializable {
 
     /** For serialization. */
     private static final long serialVersionUID = -797214582948827144L;
-    
+
+    /**
+     * The dataset.
+     *
+     * @since 1.0.6
+     */
+    private Dataset dataset;
+
+    /**
+     * The series key.
+     *
+     * @since 1.0.6
+     */
+    private Comparable seriesKey;
+
     /** The dataset index. */
     private int datasetIndex;
-    
+
     /** The series index. */
     private int series;
-    
+
     /** The label. */
     private String label;
-    
+
     /** The attributed label (if null, fall back to the regular label). */
     private transient AttributedString attributedLabel;
 
-    /** 
-     * The description (not currently used - could be displayed as a tool tip). 
+    /**
+     * The description (not currently used - could be displayed as a tool tip).
      */
     private String description;
-    
+
     /** The tool tip text. */
     private String toolTipText;
-    
+
     /** The url text. */
     private String urlText;
 
     /** A flag that controls whether or not the shape is visible. */
     private boolean shapeVisible;
-    
+
     /** The shape. */
     private transient Shape shape;
-    
+
     /** A flag that controls whether or not the shape is filled. */
     private boolean shapeFilled;
 
     /** The paint. */
     private transient Paint fillPaint;
-    
+
+    /**
+     * A gradient paint transformer.
+     *
+     * @since 1.0.4
+     */
+    private GradientPaintTransformer fillPaintTransformer;
+
     /** A flag that controls whether or not the shape outline is visible. */
     private boolean shapeOutlineVisible;
-    
+
     /** The outline paint. */
     private transient Paint outlinePaint;
-    
+
     /** The outline stroke. */
     private transient Stroke outlineStroke;
 
     /** A flag that controls whether or not the line is visible. */
     private boolean lineVisible;
-    
+
     /** The line. */
     private transient Shape line;
-    
+
     /** The stroke. */
     private transient Stroke lineStroke;
-    
+
     /** The line paint. */
     private transient Paint linePaint;
 
@@ -149,17 +174,30 @@ public class LegendItem implements Serializable {
      * use this.
      */
     private static final Shape UNUSED_SHAPE = new Line2D.Float();
-    
+
     /**
      * The stroke must be non-null for a LegendItem - if no stroke is required,
      * use this.
      */
     private static final Stroke UNUSED_STROKE = new BasicStroke(0.0f);
-    
+
+    /**
+     * Creates a legend item with the specified label.  The remaining
+     * attributes take default values.
+     *
+     * @param label  the label (<code>null</code> not permitted).
+     *
+     * @since 1.0.10
+     */
+    public LegendItem(String label) {
+        this(label, null, null, null, new Rectangle2D.Double(-4.0, -4.0, 8.0,
+                8.0), Color.black);
+    }
+
     /**
      * Creates a legend item with a filled shape.  The shape is not outlined,
      * and no line is visible.
-     * 
+     *
      * @param label  the label (<code>null</code> not permitted).
      * @param description  the description (<code>null</code> permitted).
      * @param toolTipText  the tool tip text (<code>null</code> permitted).
@@ -168,22 +206,22 @@ public class LegendItem implements Serializable {
      * @param fillPaint  the paint used to fill the shape (<code>null</code>
      *                   not permitted).
      */
-    public LegendItem(String label, String description, 
-                      String toolTipText, String urlText, 
+    public LegendItem(String label, String description,
+                      String toolTipText, String urlText,
                       Shape shape, Paint fillPaint) {
-        
-        this(label, description, toolTipText, urlText, 
-                /* shape visible = */ true, shape, 
-                /* shape filled = */ true, fillPaint, 
+
+        this(label, description, toolTipText, urlText,
+                /* shape visible = */ true, shape,
+                /* shape filled = */ true, fillPaint,
                 /* shape outlined */ false, Color.black, UNUSED_STROKE,
                 /* line visible */ false, UNUSED_SHAPE, UNUSED_STROKE,
                 Color.black);
 
     }
-    
+
     /**
      * Creates a legend item with a filled and outlined shape.
-     * 
+     *
      * @param label  the label (<code>null</code> not permitted).
      * @param description  the description (<code>null</code> permitted).
      * @param toolTipText  the tool tip text (<code>null</code> permitted).
@@ -191,28 +229,28 @@ public class LegendItem implements Serializable {
      * @param shape  the shape (<code>null</code> not permitted).
      * @param fillPaint  the paint used to fill the shape (<code>null</code>
      *                   not permitted).
-     * @param outlineStroke  the outline stroke (<code>null</code> not 
+     * @param outlineStroke  the outline stroke (<code>null</code> not
      *                       permitted).
-     * @param outlinePaint  the outline paint (<code>null</code> not 
+     * @param outlinePaint  the outline paint (<code>null</code> not
      *                      permitted).
      */
-    public LegendItem(String label, String description, 
-                      String toolTipText, String urlText, 
-                      Shape shape, Paint fillPaint, 
+    public LegendItem(String label, String description,
+                      String toolTipText, String urlText,
+                      Shape shape, Paint fillPaint,
                       Stroke outlineStroke, Paint outlinePaint) {
-        
+
         this(label, description, toolTipText, urlText,
-                /* shape visible = */ true, shape, 
-                /* shape filled = */ true, fillPaint, 
+                /* shape visible = */ true, shape,
+                /* shape filled = */ true, fillPaint,
                 /* shape outlined = */ true, outlinePaint, outlineStroke,
                 /* line visible */ false, UNUSED_SHAPE, UNUSED_STROKE,
                 Color.black);
 
     }
-    
+
     /**
      * Creates a legend item using a line.
-     * 
+     *
      * @param label  the label (<code>null</code> not permitted).
      * @param description  the description (<code>null</code> permitted).
      * @param toolTipText  the tool tip text (<code>null</code> permitted).
@@ -221,37 +259,37 @@ public class LegendItem implements Serializable {
      * @param lineStroke  the line stroke (<code>null</code> not permitted).
      * @param linePaint  the line paint (<code>null</code> not permitted).
      */
-    public LegendItem(String label, String description, 
-                      String toolTipText, String urlText, 
+    public LegendItem(String label, String description,
+                      String toolTipText, String urlText,
                       Shape line, Stroke lineStroke, Paint linePaint) {
-        
+
         this(label, description, toolTipText, urlText,
                 /* shape visible = */ false, UNUSED_SHAPE,
                 /* shape filled = */ false, Color.black,
                 /* shape outlined = */ false, Color.black, UNUSED_STROKE,
                 /* line visible = */ true, line, lineStroke, linePaint);
     }
-    
+
     /**
      * Creates a new legend item.
      *
      * @param label  the label (<code>null</code> not permitted).
-     * @param description  the description (not currently used, 
+     * @param description  the description (not currently used,
      *        <code>null</code> permitted).
      * @param toolTipText  the tool tip text (<code>null</code> permitted).
      * @param urlText  the URL text (<code>null</code> permitted).
-     * @param shapeVisible  a flag that controls whether or not the shape is 
+     * @param shapeVisible  a flag that controls whether or not the shape is
      *                      displayed.
      * @param shape  the shape (<code>null</code> permitted).
-     * @param shapeFilled  a flag that controls whether or not the shape is 
+     * @param shapeFilled  a flag that controls whether or not the shape is
      *                     filled.
      * @param fillPaint  the fill paint (<code>null</code> not permitted).
-     * @param shapeOutlineVisible  a flag that controls whether or not the 
+     * @param shapeOutlineVisible  a flag that controls whether or not the
      *                             shape is outlined.
      * @param outlinePaint  the outline paint (<code>null</code> not permitted).
-     * @param outlineStroke  the outline stroke (<code>null</code> not 
+     * @param outlineStroke  the outline stroke (<code>null</code> not
      *                       permitted).
-     * @param lineVisible  a flag that controls whether or not the line is 
+     * @param lineVisible  a flag that controls whether or not the line is
      *                     visible.
      * @param line  the line.
      * @param lineStroke  the stroke (<code>null</code> not permitted).
@@ -260,17 +298,17 @@ public class LegendItem implements Serializable {
     public LegendItem(String label, String description,
                       String toolTipText, String urlText,
                       boolean shapeVisible, Shape shape,
-                      boolean shapeFilled, Paint fillPaint, 
+                      boolean shapeFilled, Paint fillPaint,
                       boolean shapeOutlineVisible, Paint outlinePaint,
                       Stroke outlineStroke,
                       boolean lineVisible, Shape line,
                       Stroke lineStroke, Paint linePaint) {
-        
+
         if (label == null) {
-            throw new IllegalArgumentException("Null 'label' argument.");   
+            throw new IllegalArgumentException("Null 'label' argument.");
         }
         if (fillPaint == null) {
-            throw new IllegalArgumentException("Null 'fillPaint' argument.");   
+            throw new IllegalArgumentException("Null 'fillPaint' argument.");
         }
         if (lineStroke == null) {
             throw new IllegalArgumentException("Null 'lineStroke' argument.");
@@ -280,7 +318,7 @@ public class LegendItem implements Serializable {
         }
         if (outlineStroke == null) {
             throw new IllegalArgumentException(
-                    "Null 'outlineStroke' argument.");   
+                    "Null 'outlineStroke' argument.");
         }
         this.label = label;
         this.attributedLabel = null;
@@ -289,6 +327,7 @@ public class LegendItem implements Serializable {
         this.shape = shape;
         this.shapeFilled = shapeFilled;
         this.fillPaint = fillPaint;
+        this.fillPaintTransformer = new StandardGradientPaintTransformer();
         this.shapeOutlineVisible = shapeOutlineVisible;
         this.outlinePaint = outlinePaint;
         this.outlineStroke = outlineStroke;
@@ -299,11 +338,11 @@ public class LegendItem implements Serializable {
         this.toolTipText = toolTipText;
         this.urlText = urlText;
     }
-    
+
     /**
      * Creates a legend item with a filled shape.  The shape is not outlined,
      * and no line is visible.
-     * 
+     *
      * @param label  the label (<code>null</code> not permitted).
      * @param description  the description (<code>null</code> permitted).
      * @param toolTipText  the tool tip text (<code>null</code> permitted).
@@ -312,22 +351,22 @@ public class LegendItem implements Serializable {
      * @param fillPaint  the paint used to fill the shape (<code>null</code>
      *                   not permitted).
      */
-    public LegendItem(AttributedString label, String description, 
-                      String toolTipText, String urlText, 
+    public LegendItem(AttributedString label, String description,
+                      String toolTipText, String urlText,
                       Shape shape, Paint fillPaint) {
-        
-        this(label, description, toolTipText, urlText, 
+
+        this(label, description, toolTipText, urlText,
                 /* shape visible = */ true, shape,
                 /* shape filled = */ true, fillPaint,
                 /* shape outlined = */ false, Color.black, UNUSED_STROKE,
                 /* line visible = */ false, UNUSED_SHAPE, UNUSED_STROKE,
                 Color.black);
-        
+
     }
-    
+
     /**
      * Creates a legend item with a filled and outlined shape.
-     * 
+     *
      * @param label  the label (<code>null</code> not permitted).
      * @param description  the description (<code>null</code> permitted).
      * @param toolTipText  the tool tip text (<code>null</code> permitted).
@@ -335,16 +374,16 @@ public class LegendItem implements Serializable {
      * @param shape  the shape (<code>null</code> not permitted).
      * @param fillPaint  the paint used to fill the shape (<code>null</code>
      *                   not permitted).
-     * @param outlineStroke  the outline stroke (<code>null</code> not 
+     * @param outlineStroke  the outline stroke (<code>null</code> not
      *                       permitted).
-     * @param outlinePaint  the outline paint (<code>null</code> not 
+     * @param outlinePaint  the outline paint (<code>null</code> not
      *                      permitted).
      */
-    public LegendItem(AttributedString label, String description, 
-                      String toolTipText, String urlText, 
-                      Shape shape, Paint fillPaint, 
+    public LegendItem(AttributedString label, String description,
+                      String toolTipText, String urlText,
+                      Shape shape, Paint fillPaint,
                       Stroke outlineStroke, Paint outlinePaint) {
-        
+
         this(label, description, toolTipText, urlText,
                 /* shape visible = */ true, shape,
                 /* shape filled = */ true, fillPaint,
@@ -352,10 +391,10 @@ public class LegendItem implements Serializable {
                 /* line visible = */ false, UNUSED_SHAPE, UNUSED_STROKE,
                 Color.black);
     }
-    
+
     /**
      * Creates a legend item using a line.
-     * 
+     *
      * @param label  the label (<code>null</code> not permitted).
      * @param description  the description (<code>null</code> permitted).
      * @param toolTipText  the tool tip text (<code>null</code> permitted).
@@ -364,10 +403,10 @@ public class LegendItem implements Serializable {
      * @param lineStroke  the line stroke (<code>null</code> not permitted).
      * @param linePaint  the line paint (<code>null</code> not permitted).
      */
-    public LegendItem(AttributedString label, String description, 
-                      String toolTipText, String urlText, 
+    public LegendItem(AttributedString label, String description,
+                      String toolTipText, String urlText,
                       Shape line, Stroke lineStroke, Paint linePaint) {
-        
+
         this(label, description, toolTipText, urlText,
                 /* shape visible = */ false, UNUSED_SHAPE,
                 /* shape filled = */ false, Color.black,
@@ -375,56 +414,62 @@ public class LegendItem implements Serializable {
                 /* line visible = */ true, line, lineStroke, linePaint
         );
     }
-    
+
     /**
      * Creates a new legend item.
      *
      * @param label  the label (<code>null</code> not permitted).
-     * @param description  the description (not currently used, 
+     * @param description  the description (not currently used,
      *        <code>null</code> permitted).
      * @param toolTipText  the tool tip text (<code>null</code> permitted).
      * @param urlText  the URL text (<code>null</code> permitted).
-     * @param shapeVisible  a flag that controls whether or not the shape is 
+     * @param shapeVisible  a flag that controls whether or not the shape is
      *                      displayed.
      * @param shape  the shape (<code>null</code> permitted).
-     * @param shapeFilled  a flag that controls whether or not the shape is 
+     * @param shapeFilled  a flag that controls whether or not the shape is
      *                     filled.
      * @param fillPaint  the fill paint (<code>null</code> not permitted).
-     * @param shapeOutlineVisible  a flag that controls whether or not the 
+     * @param shapeOutlineVisible  a flag that controls whether or not the
      *                             shape is outlined.
      * @param outlinePaint  the outline paint (<code>null</code> not permitted).
-     * @param outlineStroke  the outline stroke (<code>null</code> not 
+     * @param outlineStroke  the outline stroke (<code>null</code> not
      *                       permitted).
-     * @param lineVisible  a flag that controls whether or not the line is 
+     * @param lineVisible  a flag that controls whether or not the line is
      *                     visible.
-     * @param line  the line.
+     * @param line  the line (<code>null</code> not permitted).
      * @param lineStroke  the stroke (<code>null</code> not permitted).
      * @param linePaint  the line paint (<code>null</code> not permitted).
      */
     public LegendItem(AttributedString label, String description,
                       String toolTipText, String urlText,
                       boolean shapeVisible, Shape shape,
-                      boolean shapeFilled, Paint fillPaint, 
+                      boolean shapeFilled, Paint fillPaint,
                       boolean shapeOutlineVisible, Paint outlinePaint,
                       Stroke outlineStroke,
                       boolean lineVisible, Shape line, Stroke lineStroke,
                       Paint linePaint) {
-        
+
         if (label == null) {
-            throw new IllegalArgumentException("Null 'label' argument.");   
+            throw new IllegalArgumentException("Null 'label' argument.");
         }
         if (fillPaint == null) {
-            throw new IllegalArgumentException("Null 'fillPaint' argument.");   
+            throw new IllegalArgumentException("Null 'fillPaint' argument.");
         }
         if (lineStroke == null) {
             throw new IllegalArgumentException("Null 'lineStroke' argument.");
         }
+        if (line == null) {
+            throw new IllegalArgumentException("Null 'line' argument.");
+        }
+        if (linePaint == null) {
+            throw new IllegalArgumentException("Null 'linePaint' argument.");
+        }
         if (outlinePaint == null) {
             throw new IllegalArgumentException("Null 'outlinePaint' argument.");
         }
         if (outlineStroke == null) {
             throw new IllegalArgumentException(
-                "Null 'outlineStroke' argument.");   
+                "Null 'outlineStroke' argument.");
         }
         this.label = characterIteratorToString(label.getIterator());
         this.attributedLabel = label;
@@ -433,6 +478,7 @@ public class LegendItem implements Serializable {
         this.shape = shape;
         this.shapeFilled = shapeFilled;
         this.fillPaint = fillPaint;
+        this.fillPaintTransformer = new StandardGradientPaintTransformer();
         this.shapeOutlineVisible = shapeOutlineVisible;
         this.outlinePaint = outlinePaint;
         this.outlineStroke = outlineStroke;
@@ -446,9 +492,9 @@ public class LegendItem implements Serializable {
 
     /**
      * Returns a string containing the characters from the given iterator.
-     * 
+     *
      * @param iterator  the iterator (<code>null</code> not permitted).
-     * 
+     *
      * @return A string.
      */
     private String characterIteratorToString(CharacterIterator iterator) {
@@ -468,51 +514,104 @@ public class LegendItem implements Serializable {
         }
         return new String(chars);
     }
-    
+
+    /**
+     * Returns the dataset.
+     *
+     * @return The dataset.
+     *
+     * @since 1.0.6
+     *
+     * @see #setDatasetIndex(int)
+     */
+    public Dataset getDataset() {
+        return this.dataset;
+    }
+
+    /**
+     * Sets the dataset.
+     *
+     * @param dataset  the dataset.
+     *
+     * @since 1.0.6
+     */
+    public void setDataset(Dataset dataset) {
+        this.dataset = dataset;
+    }
+
     /**
      * Returns the dataset index for this legend item.
-     * 
+     *
      * @return The dataset index.
-     * 
+     *
      * @since 1.0.2
+     *
+     * @see #setDatasetIndex(int)
+     * @see #getDataset()
      */
     public int getDatasetIndex() {
         return this.datasetIndex;
     }
-    
+
     /**
      * Sets the dataset index for this legend item.
-     * 
+     *
      * @param index  the index.
-     * 
+     *
      * @since 1.0.2
+     *
+     * @see #getDatasetIndex()
      */
     public void setDatasetIndex(int index) {
         this.datasetIndex = index;
     }
-    
+
+    /**
+     * Returns the series key.
+     *
+     * @return The series key.
+     *
+     * @since 1.0.6
+     *
+     * @see #setSeriesKey(Comparable)
+     */
+    public Comparable getSeriesKey() {
+        return this.seriesKey;
+    }
+
+    /**
+     * Sets the series key.
+     *
+     * @param key  the series key.
+     *
+     * @since 1.0.6
+     */
+    public void setSeriesKey(Comparable key) {
+        this.seriesKey = key;
+    }
+
     /**
      * Returns the series index for this legend item.
-     * 
+     *
      * @return The series index.
-     * 
+     *
      * @since 1.0.2
      */
     public int getSeriesIndex() {
         return this.series;
     }
-    
+
     /**
      * Sets the series index for this legend item.
-     * 
+     *
      * @param index  the index.
-     * 
+     *
      * @since 1.0.2
      */
     public void setSeriesIndex(int index) {
         this.series = index;
     }
-    
+
     /**
      * Returns the label.
      *
@@ -533,42 +632,42 @@ public class LegendItem implements Serializable {
 
     /**
      * Returns the description for the legend item.
-     * 
+     *
      * @return The description.
      */
     public String getDescription() {
-        return this.description;   
+        return this.description;
     }
-    
+
     /**
      * Returns the tool tip text.
-     * 
+     *
      * @return The tool tip text (possibly <code>null</code>).
      */
     public String getToolTipText() {
-        return this.toolTipText;   
+        return this.toolTipText;
     }
-    
+
     /**
      * Returns the URL text.
-     * 
+     *
      * @return The URL text (possibly <code>null</code>).
      */
     public String getURLText() {
-        return this.urlText; 
+        return this.urlText;
     }
-    
+
     /**
      * Returns a flag that indicates whether or not the shape is visible.
-     * 
+     *
      * @return A boolean.
      */
     public boolean isShapeVisible() {
         return this.shapeVisible;
     }
-    
+
     /**
-     * Returns the shape used to label the series represented by this legend 
+     * Returns the shape used to label the series represented by this legend
      * item.
      *
      * @return The shape (never <code>null</code>).
@@ -576,10 +675,10 @@ public class LegendItem implements Serializable {
     public Shape getShape() {
         return this.shape;
     }
-    
+
     /**
      * Returns a flag that controls whether or not the shape is filled.
-     * 
+     *
      * @return A boolean.
      */
     public boolean isShapeFilled() {
@@ -598,13 +697,13 @@ public class LegendItem implements Serializable {
     /**
      * Returns the flag that controls whether or not the shape outline
      * is visible.
-     * 
+     *
      * @return A boolean.
      */
     public boolean isShapeOutlineVisible() {
         return this.shapeOutlineVisible;
     }
-    
+
     /**
      * Returns the line stroke for the series.
      *
@@ -613,16 +712,16 @@ public class LegendItem implements Serializable {
     public Stroke getLineStroke() {
         return this.lineStroke;
     }
-    
+
     /**
      * Returns the paint used for lines.
-     * 
+     *
      * @return The paint.
      */
     public Paint getLinePaint() {
         return this.linePaint;
     }
-    
+
     /**
      * Returns the outline paint.
      *
@@ -640,35 +739,66 @@ public class LegendItem implements Serializable {
     public Stroke getOutlineStroke() {
         return this.outlineStroke;
     }
-    
+
     /**
      * Returns a flag that indicates whether or not the line is visible.
-     * 
+     *
      * @return A boolean.
      */
     public boolean isLineVisible() {
         return this.lineVisible;
     }
-    
+
     /**
      * Returns the line.
-     * 
-     * @return The line.
+     *
+     * @return The line (never <code>null</code>).
      */
     public Shape getLine() {
         return this.line;
     }
-    
+
+    /**
+     * Returns the transformer used when the fill paint is an instance of
+     * <code>GradientPaint</code>.
+     *
+     * @return The transformer (never <code>null</code>).
+     *
+     * @since 1.0.4
+     *
+     * @see #setFillPaintTransformer(GradientPaintTransformer)
+     */
+    public GradientPaintTransformer getFillPaintTransformer() {
+        return this.fillPaintTransformer;
+    }
+
+    /**
+     * Sets the transformer used when the fill paint is an instance of
+     * <code>GradientPaint</code>.
+     *
+     * @param transformer  the transformer (<code>null</code> not permitted).
+     *
+     * @since 1.0.4
+     *
+     * @see #getFillPaintTransformer()
+     */
+    public void setFillPaintTransformer(GradientPaintTransformer transformer) {
+        if (transformer == null) {
+            throw new IllegalArgumentException("Null 'transformer' attribute.");
+        }
+        this.fillPaintTransformer = transformer;
+    }
+
     /**
      * Tests this item for equality with an arbitrary object.
-     * 
+     *
      * @param obj  the object (<code>null</code> permitted).
-     * 
+     *
      * @return A boolean.
      */
     public boolean equals(Object obj) {
         if (obj == this) {
-            return true;   
+            return true;
         }
         if (!(obj instanceof LegendItem)) {
                 return false;
@@ -683,7 +813,7 @@ public class LegendItem implements Serializable {
         if (!this.label.equals(that.label)) {
             return false;
         }
-        if (!AttributedStringUtilities.equal(this.attributedLabel, 
+        if (!AttributedStringUtilities.equal(this.attributedLabel,
                 that.attributedLabel)) {
             return false;
         }
@@ -700,16 +830,20 @@ public class LegendItem implements Serializable {
             return false;
         }
         if (!this.fillPaint.equals(that.fillPaint)) {
-            return false;   
+            return false;
+        }
+        if (!ObjectUtilities.equal(this.fillPaintTransformer,
+                that.fillPaintTransformer)) {
+            return false;
         }
         if (this.shapeOutlineVisible != that.shapeOutlineVisible) {
             return false;
         }
         if (!this.outlineStroke.equals(that.outlineStroke)) {
-            return false;   
+            return false;
         }
         if (!this.outlinePaint.equals(that.outlinePaint)) {
-            return false;   
+            return false;
         }
         if (!this.lineVisible == that.lineVisible) {
             return false;
@@ -718,14 +852,40 @@ public class LegendItem implements Serializable {
             return false;
         }
         if (!this.lineStroke.equals(that.lineStroke)) {
-            return false;   
+            return false;
         }
         if (!this.linePaint.equals(that.linePaint)) {
             return false;
         }
         return true;
     }
-    
+
+    /**
+     * Returns an independent copy of this object (except that the clone will
+     * still reference the same dataset as the original
+     * <code>LegendItem</code>).
+     *
+     * @return A clone.
+     *
+     * @since 1.0.10
+     */
+    public Object clone() throws CloneNotSupportedException {
+        LegendItem clone = (LegendItem) super.clone();
+        if (this.seriesKey instanceof PublicCloneable) {
+            PublicCloneable pc = (PublicCloneable) this.seriesKey;
+            clone.seriesKey = (Comparable) pc.clone();
+        }
+        // FIXME: Clone the attributed string if it is not null
+        clone.shape = ShapeUtilities.clone(this.shape);
+        if (this.fillPaintTransformer instanceof PublicCloneable) {
+            PublicCloneable pc = (PublicCloneable) this.fillPaintTransformer;
+            clone.fillPaintTransformer = (GradientPaintTransformer) pc.clone();
+
+        }
+        clone.line = ShapeUtilities.clone(this.line);
+        return clone;
+    }
+
     /**
      * Provides serialization support.
      *
@@ -753,7 +913,7 @@ public class LegendItem implements Serializable {
      * @throws IOException  if there is an I/O error.
      * @throws ClassNotFoundException  if there is a classpath problem.
      */
-    private void readObject(ObjectInputStream stream) 
+    private void readObject(ObjectInputStream stream)
         throws IOException, ClassNotFoundException {
         stream.defaultReadObject();
         this.attributedLabel = SerialUtilities.readAttributedString(stream);
@@ -765,5 +925,5 @@ public class LegendItem implements Serializable {
         this.lineStroke = SerialUtilities.readStroke(stream);
         this.linePaint = SerialUtilities.readPaint(stream);
     }
-    
+
 }
