@@ -2,7 +2,7 @@
  * JFreeChart : a free chart library for the Java(tm) platform
  * ===========================================================
  *
- * (C) Copyright 2000-2007, by Object Refinery Limited and Contributors.
+ * (C) Copyright 2000-2008, by Object Refinery Limited and Contributors.
  *
  * Project Info:  http://www.jfree.org/jfreechart/index.html
  *
@@ -27,16 +27,25 @@
  * ------------
  * LogAxis.java
  * ------------
- * (C) Copyright 2006, 2007, by Object Refinery Limited and Contributors.
+ * (C) Copyright 2006-2008, by Object Refinery Limited and Contributors.
  *
  * Original Author:  David Gilbert (for Object Refinery Limited);
- * Contributor(s):   -;
+ * Contributor(s):   Andrew Mickish (patch 1868745);
  *
  * Changes
  * -------
  * 24-Aug-2006 : Version 1 (DG);
  * 22-Mar-2007 : Use defaultAutoArrange attribute (DG);
  * 02-Aug-2007 : Fixed zooming bug, added support for margins (DG);
+ * 14-Feb-2008 : Changed default minorTickCount to 9 - see bug report 
+ *               1892419 (DG);
+ * 15-Feb-2008 : Applied a variation of patch 1868745 by Andrew Mickish to
+ *               fix a labelling bug when the axis appears at the top or
+ *               right of the chart (DG);
+ * 19-Mar-2008 : Applied patch 1902418 by Andrew Mickish to fix bug in tick
+ *               labels for vertical axis (DG);
+ * 26-Mar-2008 : Changed createTickLabel() method from private to protected -
+ *               see patch 1918209 by Andrew Mickish (DG);
  * 
  */
 
@@ -105,7 +114,7 @@ public class LogAxis extends ValueAxis {
         super(label,  createLogTickUnits(Locale.getDefault()));
         setDefaultAutoRange(new Range(0.01, 1.0));
         this.tickUnit = new NumberTickUnit(1.0, new DecimalFormat("0.#"));
-        this.minorTickCount = 10;
+        this.minorTickCount = 9;
     }
     
     /**
@@ -513,6 +522,13 @@ public class LogAxis extends ValueAxis {
         List ticks = new ArrayList();
         Font tickLabelFont = getTickLabelFont();
         g2.setFont(tickLabelFont);
+        TextAnchor textAnchor;
+        if (edge == RectangleEdge.TOP) {
+            textAnchor = TextAnchor.BOTTOM_CENTER;
+        }
+        else {
+            textAnchor = TextAnchor.TOP_CENTER;
+        }
         
         if (isAutoTickUnitSelection()) {
             selectAutoTickUnit(g2, dataArea, edge);
@@ -524,7 +540,7 @@ public class LogAxis extends ValueAxis {
             double v = calculateValue(current);
             if (range.contains(v)) {
                 ticks.add(new NumberTick(TickType.MAJOR, v, createTickLabel(v), 
-                        TextAnchor.TOP_CENTER, TextAnchor.CENTER, 0.0));
+                        textAnchor, TextAnchor.CENTER, 0.0));
             }
             // add minor ticks (for gridlines)
             double next = Math.pow(this.base, current 
@@ -532,8 +548,8 @@ public class LogAxis extends ValueAxis {
             for (int i = 1; i < this.minorTickCount; i++) {
                 double minorV = v + i * ((next - v) / this.minorTickCount);
                 if (range.contains(minorV)) {
-                    ticks.add(new NumberTick(TickType.MINOR, minorV, 
-                        "", TextAnchor.TOP_CENTER, TextAnchor.CENTER, 0.0));
+                    ticks.add(new NumberTick(TickType.MINOR, minorV, "", 
+                            textAnchor, TextAnchor.CENTER, 0.0));
                 }
             }
             current = current + this.tickUnit.getSize();
@@ -557,6 +573,13 @@ public class LogAxis extends ValueAxis {
         List ticks = new ArrayList();
         Font tickLabelFont = getTickLabelFont();
         g2.setFont(tickLabelFont);
+        TextAnchor textAnchor;
+        if (edge == RectangleEdge.RIGHT) {
+            textAnchor = TextAnchor.CENTER_LEFT;
+        }
+        else {
+            textAnchor = TextAnchor.CENTER_RIGHT;
+        }
         
         if (isAutoTickUnitSelection()) {
             selectAutoTickUnit(g2, dataArea, edge);
@@ -567,8 +590,8 @@ public class LogAxis extends ValueAxis {
         while (current <= end) {
             double v = calculateValue(current);
             if (range.contains(v)) {
-                ticks.add(new NumberTick(TickType.MINOR, v, createTickLabel(v), 
-                        TextAnchor.CENTER_RIGHT, TextAnchor.CENTER, 0.0));
+                ticks.add(new NumberTick(TickType.MAJOR, v, createTickLabel(v), 
+                        textAnchor, TextAnchor.CENTER, 0.0));
             }
             // add minor ticks (for gridlines)
             double next = Math.pow(this.base, current 
@@ -577,7 +600,7 @@ public class LogAxis extends ValueAxis {
                 double minorV = v + i * ((next - v) / this.minorTickCount);
                 if (range.contains(minorV)) {
                     ticks.add(new NumberTick(TickType.MINOR, minorV, "", 
-                            TextAnchor.CENTER_RIGHT, TextAnchor.CENTER, 0.0));
+                            textAnchor, TextAnchor.CENTER, 0.0));
                 }
             }
             current = current + this.tickUnit.getSize();
@@ -810,13 +833,16 @@ public class LogAxis extends ValueAxis {
     }
 
     /**
-     * Creates a tick label for the specified value.
+     * Creates a tick label for the specified value.  Note that this method
+     * was 'private' prior to version 1.0.10.
      * 
      * @param value  the value.
      * 
      * @return The label.
+     *
+     * @since 1.0.10
      */
-    private String createTickLabel(double value) {
+    protected String createTickLabel(double value) {
         if (this.numberFormatOverride != null) {
             return this.numberFormatOverride.format(value);
         }
@@ -903,21 +929,21 @@ public class LogAxis extends ValueAxis {
         units.add(new NumberTickUnit(20000, numberFormat));
         units.add(new NumberTickUnit(50000, numberFormat));
         units.add(new NumberTickUnit(100000, numberFormat));
-        units.add(new NumberTickUnit(200000,         numberFormat));
-        units.add(new NumberTickUnit(500000,         numberFormat));
-        units.add(new NumberTickUnit(1000000,        numberFormat));
-        units.add(new NumberTickUnit(2000000,        numberFormat));
-        units.add(new NumberTickUnit(5000000,        numberFormat));
-        units.add(new NumberTickUnit(10000000,       numberFormat));
-        units.add(new NumberTickUnit(20000000,       numberFormat));
-        units.add(new NumberTickUnit(50000000,       numberFormat));
-        units.add(new NumberTickUnit(100000000,      numberFormat));
-        units.add(new NumberTickUnit(200000000,      numberFormat));
-        units.add(new NumberTickUnit(500000000,      numberFormat));
-        units.add(new NumberTickUnit(1000000000,     numberFormat));
-        units.add(new NumberTickUnit(2000000000,     numberFormat));
-        units.add(new NumberTickUnit(5000000000.0,   numberFormat));
-        units.add(new NumberTickUnit(10000000000.0,  numberFormat));
+        units.add(new NumberTickUnit(200000, numberFormat));
+        units.add(new NumberTickUnit(500000, numberFormat));
+        units.add(new NumberTickUnit(1000000, numberFormat));
+        units.add(new NumberTickUnit(2000000, numberFormat));
+        units.add(new NumberTickUnit(5000000, numberFormat));
+        units.add(new NumberTickUnit(10000000, numberFormat));
+        units.add(new NumberTickUnit(20000000, numberFormat));
+        units.add(new NumberTickUnit(50000000, numberFormat));
+        units.add(new NumberTickUnit(100000000, numberFormat));
+        units.add(new NumberTickUnit(200000000, numberFormat));
+        units.add(new NumberTickUnit(500000000, numberFormat));
+        units.add(new NumberTickUnit(1000000000, numberFormat));
+        units.add(new NumberTickUnit(2000000000, numberFormat));
+        units.add(new NumberTickUnit(5000000000.0, numberFormat));
+        units.add(new NumberTickUnit(10000000000.0, numberFormat));
 
         return units;
 
