@@ -2,59 +2,57 @@
  * JFreeChart : a free chart library for the Java(tm) platform
  * ===========================================================
  *
- * (C) Copyright 2000-2006, by Object Refinery Limited and Contributors.
+ * (C) Copyright 2000-2007, by Object Refinery Limited and Contributors.
  *
  * Project Info:  http://www.jfree.org/jfreechart/index.html
  *
- * This library is free software; you can redistribute it and/or modify it 
- * under the terms of the GNU Lesser General Public License as published by 
- * the Free Software Foundation; either version 2.1 of the License, or 
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful, but 
- * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
- * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+ * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
  * License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, 
- * USA.  
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
+ * USA.
  *
- * [Java is a trademark or registered trademark of Sun Microsystems, Inc. 
+ * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
  * in the United States and other countries.]
  *
  * ---------------
  * NumberAxis.java
  * ---------------
- * (C) Copyright 2000-2006, by Object Refinery Limited and Contributors.
+ * (C) Copyright 2000-2007, by Object Refinery Limited and Contributors.
  *
  * Original Author:  David Gilbert (for Object Refinery Limited);
  * Contributor(s):   Laurence Vanhelsuwe;
  *
- * $Id: NumberAxis.java,v 1.16.2.3 2006/02/20 21:46:52 mungady Exp $
- *
- * Changes (from 18-Sep-2001)
- * --------------------------
+ * Changes
+ * -------
  * 18-Sep-2001 : Added standard header and fixed DOS encoding problem (DG);
- * 22-Sep-2001 : Changed setMinimumAxisValue() and setMaximumAxisValue() so 
+ * 22-Sep-2001 : Changed setMinimumAxisValue() and setMaximumAxisValue() so
  *               that they clear the autoRange flag (DG);
  * 27-Nov-2001 : Removed old, redundant code (DG);
  * 30-Nov-2001 : Added accessor methods for the standard tick units (DG);
  * 08-Jan-2002 : Added setAxisRange() method (since renamed setRange()) (DG);
- * 16-Jan-2002 : Added setTickUnit() method.  Extended ValueAxis to support an 
+ * 16-Jan-2002 : Added setTickUnit() method.  Extended ValueAxis to support an
  *               optional cross-hair (DG);
  * 08-Feb-2002 : Fixes bug to ensure the autorange is recalculated if the
  *               setAutoRangeIncludesZero flag is changed (DG);
- * 25-Feb-2002 : Added a new flag autoRangeStickyZero to provide further 
- *               control over margins in the auto-range mechanism.  Updated 
- *               constructors.  Updated import statements.  Moved the 
+ * 25-Feb-2002 : Added a new flag autoRangeStickyZero to provide further
+ *               control over margins in the auto-range mechanism.  Updated
+ *               constructors.  Updated import statements.  Moved the
  *               createStandardTickUnits() method to the TickUnits class (DG);
  * 19-Apr-2002 : Updated Javadoc comments (DG);
  * 01-May-2002 : Updated for changes to TickUnit class, removed valueToString()
  *               method (DG);
  * 25-Jul-2002 : Moved the lower and upper margin attributes, and the
- *               auto-range minimum size, up one level to the ValueAxis 
+ *               auto-range minimum size, up one level to the ValueAxis
  *               class (DG);
  * 05-Sep-2002 : Updated constructor to match changes in Axis class (DG);
  * 01-Oct-2002 : Fixed errors reported by Checkstyle (DG);
@@ -71,11 +69,11 @@
  * 07-Oct-2003 : Fixed bug (815028) in the auto range calculation (DG);
  * 29-Oct-2003 : Added workaround for font alignment in PDF output (DG);
  * 07-Nov-2003 : Modified to use NumberTick class (DG);
- * 21-Jan-2004 : Renamed translateJava2DToValue --> java2DToValue, and 
- *               translateValueToJava2D --> valueToJava2D (DG); 
+ * 21-Jan-2004 : Renamed translateJava2DToValue --> java2DToValue, and
+ *               translateValueToJava2D --> valueToJava2D (DG);
  * 03-Mar-2004 : Added plotState to draw() method (DG);
  * 07-Apr-2004 : Changed string width calculation (DG);
- * 11-Jan-2005 : Removed deprecated methods in preparation for 1.0.0 
+ * 11-Jan-2005 : Removed deprecated methods in preparation for 1.0.0
  *               release (DG);
  * 28-Mar-2005 : Renamed autoRangeIncludesZero() --> getAutoRangeIncludesZero()
  *               and autoRangeStickyZero() --> getAutoRangeStickyZero() (DG);
@@ -83,10 +81,15 @@
  * 22-Apr-2005 : Renamed refreshHorizontalTicks --> refreshTicksHorizontal
  *               (and likewise the vertical version) for consistency with
  *               other axis classes (DG);
- * ------------- JFREECHART 1.0.0 ---------------------------------------------
+ * ------------- JFREECHART 1.0.x ---------------------------------------------
  * 10-Feb-2006 : Added some API doc comments in respect of bug 821046 (DG);
  * 20-Feb-2006 : Modified equals() method to check rangeType field (fixes bug
  *               1435461) (DG);
+ * 04-Sep-2006 : Fix auto range calculation for the case where all data values
+ *               are constant and large (see bug report 1549218) (DG);
+ * 11-Dec-2006 : Fix bug in auto-tick unit selection with tick format override,
+ *               see bug 1608371 (DG);
+ * 22-Mar-2007 : Use new defaultAutoRange attribute (DG);
  *
  */
 
@@ -120,10 +123,10 @@ import org.jfree.util.ObjectUtilities;
  * <P>
  * If the axis is set up to automatically determine its range to fit the data,
  * you can ensure that the range includes zero (statisticians usually prefer
- * this) by setting the <code>autoRangeIncludesZero</code> flag to 
+ * this) by setting the <code>autoRangeIncludesZero</code> flag to
  * <code>true</code>.
  * <P>
- * The <code>NumberAxis</code> class has a mechanism for automatically 
+ * The <code>NumberAxis</code> class has a mechanism for automatically
  * selecting a tick unit that is appropriate for the current axis range.  This
  * mechanism is an adaptation of code suggested by Laurence Vanhelsuwe.
  */
@@ -131,7 +134,7 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
 
     /** For serialization. */
     private static final long serialVersionUID = 2805933088476185789L;
-    
+
     /** The default value for the autoRangeIncludesZero flag. */
     public static final boolean DEFAULT_AUTO_RANGE_INCLUDES_ZERO = true;
 
@@ -139,18 +142,18 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
     public static final boolean DEFAULT_AUTO_RANGE_STICKY_ZERO = true;
 
     /** The default tick unit. */
-    public static final NumberTickUnit
-        DEFAULT_TICK_UNIT = new NumberTickUnit(1.0, new DecimalFormat("0"));
+    public static final NumberTickUnit DEFAULT_TICK_UNIT = new NumberTickUnit(
+            1.0, new DecimalFormat("0"));
 
     /** The default setting for the vertical tick labels flag. */
     public static final boolean DEFAULT_VERTICAL_TICK_LABELS = false;
 
-    /** 
+    /**
      * The range type (can be used to force the axis to display only positive
-     * values or only negative values.
+     * values or only negative values).
      */
     private RangeType rangeType;
-    
+
     /**
      * A flag that affects the axis range when the range is determined
      * automatically.  If the auto range does NOT include zero and this flag
@@ -178,9 +181,9 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
      * Default constructor.
      */
     public NumberAxis() {
-        this(null);    
+        this(null);
     }
-    
+
     /**
      * Constructs a number axis, using default values where necessary.
      *
@@ -195,29 +198,33 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
         this.numberFormatOverride = null;
         this.markerBand = null;
     }
-    
+
     /**
      * Returns the axis range type.
-     * 
+     *
      * @return The axis range type (never <code>null</code>).
+     *
+     * @see #setRangeType(RangeType)
      */
     public RangeType getRangeType() {
-        return this.rangeType;   
+        return this.rangeType;
     }
-    
+
     /**
      * Sets the axis range type.
-     * 
+     *
      * @param rangeType  the range type (<code>null</code> not permitted).
+     *
+     * @see #getRangeType()
      */
     public void setRangeType(RangeType rangeType) {
         if (rangeType == null) {
-            throw new IllegalArgumentException("Null 'rangeType' argument.");   
+            throw new IllegalArgumentException("Null 'rangeType' argument.");
         }
         this.rangeType = rangeType;
         notifyListeners(new AxisChangeEvent(this));
     }
-    
+
     /**
      * Returns the flag that indicates whether or not the automatic axis range
      * (if indeed it is determined automatically) is forced to include zero.
@@ -229,15 +236,17 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
     }
 
     /**
-     * Sets the flag that indicates whether or not the axis range, if 
+     * Sets the flag that indicates whether or not the axis range, if
      * automatically calculated, is forced to include zero.
      * <p>
-     * If the flag is changed to <code>true</code>, the axis range is 
+     * If the flag is changed to <code>true</code>, the axis range is
      * recalculated.
      * <p>
      * Any change to the flag will trigger an {@link AxisChangeEvent}.
      *
      * @param flag  the new value of the flag.
+     *
+     * @see #getAutoRangeIncludesZero()
      */
     public void setAutoRangeIncludesZero(boolean flag) {
         if (this.autoRangeIncludesZero != flag) {
@@ -254,6 +263,8 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
      * data range but inside the margins defined for the axis.
      *
      * @return The flag.
+     *
+     * @see #setAutoRangeStickyZero(boolean)
      */
     public boolean getAutoRangeStickyZero() {
         return this.autoRangeStickyZero;
@@ -264,6 +275,8 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
      * range but inside the margins defined for the axis.
      *
      * @param flag  the new flag.
+     *
+     * @see #getAutoRangeStickyZero()
      */
     public void setAutoRangeStickyZero(boolean flag) {
         if (this.autoRangeStickyZero != flag) {
@@ -276,15 +289,15 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
     }
 
     /**
-     * Returns the tick unit for the axis.  
+     * Returns the tick unit for the axis.
      * <p>
-     * Note: if the <code>autoTickUnitSelection</code> flag is 
-     * <code>true</code> the tick unit may be changed while the axis is being 
+     * Note: if the <code>autoTickUnitSelection</code> flag is
+     * <code>true</code> the tick unit may be changed while the axis is being
      * drawn, so in that case the return value from this method may be
      * irrelevant if the method is called before the axis has been drawn.
      *
      * @return The tick unit for the axis.
-     * 
+     *
      * @see #setTickUnit(NumberTickUnit)
      * @see ValueAxis#isAutoTickUnitSelection()
      */
@@ -293,14 +306,14 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
     }
 
     /**
-     * Sets the tick unit for the axis and sends an {@link AxisChangeEvent} to 
+     * Sets the tick unit for the axis and sends an {@link AxisChangeEvent} to
      * all registered listeners.  A side effect of calling this method is that
-     * the "auto-select" feature for tick units is switched off (you can 
+     * the "auto-select" feature for tick units is switched off (you can
      * restore it using the {@link ValueAxis#setAutoTickUnitSelection(boolean)}
      * method).
      *
      * @param unit  the new tick unit (<code>null</code> not permitted).
-     * 
+     *
      * @see #getTickUnit()
      * @see #setTickUnit(NumberTickUnit, boolean, boolean)
      */
@@ -310,21 +323,21 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
     }
 
     /**
-     * Sets the tick unit for the axis and, if requested, sends an 
-     * {@link AxisChangeEvent} to all registered listeners.  In addition, an 
-     * option is provided to turn off the "auto-select" feature for tick units 
-     * (you can restore it using the 
+     * Sets the tick unit for the axis and, if requested, sends an
+     * {@link AxisChangeEvent} to all registered listeners.  In addition, an
+     * option is provided to turn off the "auto-select" feature for tick units
+     * (you can restore it using the
      * {@link ValueAxis#setAutoTickUnitSelection(boolean)} method).
      *
      * @param unit  the new tick unit (<code>null</code> not permitted).
      * @param notify  notify listeners?
      * @param turnOffAutoSelect  turn off the auto-tick selection?
      */
-    public void setTickUnit(NumberTickUnit unit, boolean notify, 
+    public void setTickUnit(NumberTickUnit unit, boolean notify,
                             boolean turnOffAutoSelect) {
 
         if (unit == null) {
-            throw new IllegalArgumentException("Null 'unit' argument.");   
+            throw new IllegalArgumentException("Null 'unit' argument.");
         }
         this.tickUnit = unit;
         if (turnOffAutoSelect) {
@@ -337,20 +350,24 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
     }
 
     /**
-     * Returns the number format override.  If this is non-null, then it will 
+     * Returns the number format override.  If this is non-null, then it will
      * be used to format the numbers on the axis.
      *
      * @return The number formatter (possibly <code>null</code>).
+     *
+     * @see #setNumberFormatOverride(NumberFormat)
      */
     public NumberFormat getNumberFormatOverride() {
         return this.numberFormatOverride;
     }
 
     /**
-     * Sets the number format override.  If this is non-null, then it will be 
+     * Sets the number format override.  If this is non-null, then it will be
      * used to format the numbers on the axis.
      *
      * @param formatter  the number formatter (<code>null</code> permitted).
+     *
+     * @see #getNumberFormatOverride()
      */
     public void setNumberFormatOverride(NumberFormat formatter) {
         this.numberFormatOverride = formatter;
@@ -361,6 +378,8 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
      * Returns the (optional) marker band for the axis.
      *
      * @return The marker band (possibly <code>null</code>).
+     *
+     * @see #setMarkerBand(MarkerAxisBand)
      */
     public MarkerAxisBand getMarkerBand() {
         return this.markerBand;
@@ -369,10 +388,12 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
     /**
      * Sets the marker band for the axis.
      * <P>
-     * The marker band is optional, leave it set to <code>null</code> if you 
+     * The marker band is optional, leave it set to <code>null</code> if you
      * don't require it.
      *
      * @param band the new band (<code>null<code> permitted).
+     *
+     * @see #getMarkerBand()
      */
     public void setMarkerBand(MarkerAxisBand band) {
         this.markerBand = band;
@@ -404,9 +425,9 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
 
             Range r = vap.getDataRange(this);
             if (r == null) {
-                r = new Range(DEFAULT_LOWER_BOUND, DEFAULT_UPPER_BOUND);
+                r = getDefaultAutoRange();
             }
-            
+
             double upper = r.getUpperBound();
             double lower = r.getLowerBound();
             if (this.rangeType == RangeType.POSITIVE) {
@@ -415,9 +436,9 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
             }
             else if (this.rangeType == RangeType.NEGATIVE) {
                 lower = Math.min(0.0, lower);
-                upper = Math.min(0.0, upper);                   
+                upper = Math.min(0.0, upper);
             }
-            
+
             if (getAutoRangeIncludesZero()) {
                 lower = Math.min(lower, 0.0);
                 upper = Math.max(upper, 0.0);
@@ -436,6 +457,11 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
                     double expand = (minRange - range) / 2;
                     upper = upper + expand;
                     lower = lower - expand;
+                    if (lower == upper) { // see bug report 1549218
+                        double adjust = Math.abs(lower) / 10.0;
+                        lower = lower - adjust;
+                        upper = upper + adjust;
+                    }
                     if (this.rangeType == RangeType.POSITIVE) {
                         if (lower < 0.0) {
                             upper = upper - lower;
@@ -486,10 +512,12 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
      * @param edge  the axis location.
      *
      * @return The Java2D coordinate.
+     *
+     * @see #java2DToValue(double, Rectangle2D, RectangleEdge)
      */
-    public double valueToJava2D(double value, Rectangle2D area, 
+    public double valueToJava2D(double value, Rectangle2D area,
                                 RectangleEdge edge) {
-        
+
         Range range = getRange();
         double axisMin = range.getLowerBound();
         double axisMax = range.getUpperBound();
@@ -505,11 +533,11 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
             min = area.getMaxY();
         }
         if (isInverted()) {
-            return max 
+            return max
                    - ((value - axisMin) / (axisMax - axisMin)) * (max - min);
         }
         else {
-            return min 
+            return min
                    + ((value - axisMin) / (axisMax - axisMin)) * (max - min);
         }
 
@@ -524,10 +552,12 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
      * @param edge  the location.
      *
      * @return The data value.
+     *
+     * @see #valueToJava2D(double, Rectangle2D, RectangleEdge)
      */
-    public double java2DToValue(double java2DValue, Rectangle2D area, 
+    public double java2DToValue(double java2DValue, Rectangle2D area,
                                 RectangleEdge edge) {
-        
+
         Range range = getRange();
         double axisMin = range.getLowerBound();
         double axisMax = range.getUpperBound();
@@ -543,11 +573,11 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
             max = area.getY();
         }
         if (isInverted()) {
-            return axisMax 
+            return axisMax
                    - (java2DValue - min) / (max - min) * (axisMax - axisMin);
         }
         else {
-            return axisMin 
+            return axisMin
                    + (java2DValue - min) / (max - min) * (axisMax - axisMin);
         }
 
@@ -557,6 +587,8 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
      * Calculates the value of the lowest visible tick on the axis.
      *
      * @return The value of the lowest visible tick on the axis.
+     *
+     * @see #calculateHighestVisibleTickValue()
      */
     protected double calculateLowestVisibleTickValue() {
 
@@ -570,6 +602,8 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
      * Calculates the value of the highest visible tick on the axis.
      *
      * @return The value of the highest visible tick on the axis.
+     *
+     * @see #calculateLowestVisibleTickValue()
      */
     protected double calculateHighestVisibleTickValue() {
 
@@ -594,25 +628,25 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
     }
 
     /**
-     * Draws the axis on a Java 2D graphics device (such as the screen or a 
+     * Draws the axis on a Java 2D graphics device (such as the screen or a
      * printer).
      *
      * @param g2  the graphics device (<code>null</code> not permitted).
      * @param cursor  the cursor location.
      * @param plotArea  the area within which the axes and data should be drawn
      *                  (<code>null</code> not permitted).
-     * @param dataArea  the area within which the data should be drawn 
+     * @param dataArea  the area within which the data should be drawn
      *                  (<code>null</code> not permitted).
      * @param edge  the location of the axis (<code>null</code> not permitted).
-     * @param plotState  collects information about the plot 
+     * @param plotState  collects information about the plot
      *                   (<code>null</code> permitted).
-     * 
+     *
      * @return The axis state (never <code>null</code>).
      */
-    public AxisState draw(Graphics2D g2, 
+    public AxisState draw(Graphics2D g2,
                           double cursor,
-                          Rectangle2D plotArea, 
-                          Rectangle2D dataArea, 
+                          Rectangle2D plotArea,
+                          Rectangle2D dataArea,
                           RectangleEdge edge,
                           PlotRenderingInfo plotState) {
 
@@ -620,9 +654,9 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
         // if the axis is not visible, don't draw it...
         if (!isVisible()) {
             state = new AxisState(cursor);
-            // even though the axis is not visible, we need ticks for the 
+            // even though the axis is not visible, we need ticks for the
             // gridlines...
-            List ticks = refreshTicks(g2, state, dataArea, edge); 
+            List ticks = refreshTicks(g2, state, dataArea, edge);
             state.setTicks(ticks);
             return state;
         }
@@ -637,12 +671,12 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
 //            }
 //            getMarkerBand().draw(g2, plotArea, dataArea, 0, cursor);
 //        }
-        
+
         // draw the axis label...
         state = drawLabel(getLabel(), g2, plotArea, dataArea, edge, state);
 
         return state;
-        
+
     }
 
     /**
@@ -653,6 +687,9 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
      * NumberAxis class.
      *
      * @return The standard tick units.
+     *
+     * @see #setStandardTickUnits(TickUnitSource)
+     * @see #createIntegerTickUnits()
      */
     public static TickUnitSource createStandardTickUnits() {
 
@@ -668,8 +705,8 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
         DecimalFormat df8 = new DecimalFormat("#,##0");
         DecimalFormat df9 = new DecimalFormat("#,###,##0");
         DecimalFormat df10 = new DecimalFormat("#,###,###,##0");
-        
-        // we can add the units in any order, the TickUnits collection will 
+
+        // we can add the units in any order, the TickUnits collection will
         // sort them...
         units.add(new NumberTickUnit(0.0000001, df1));
         units.add(new NumberTickUnit(0.000001, df2));
@@ -690,7 +727,7 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
         units.add(new NumberTickUnit(1000000000, df10));
         units.add(new NumberTickUnit(10000000000.0, df10));
         units.add(new NumberTickUnit(100000000000.0, df10));
-        
+
         units.add(new NumberTickUnit(0.00000025, df0));
         units.add(new NumberTickUnit(0.0000025, df1));
         units.add(new NumberTickUnit(0.000025, df2));
@@ -739,6 +776,9 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
      * Returns a collection of tick units for integer values.
      *
      * @return A collection of tick units for integer values.
+     *
+     * @see #setStandardTickUnits(TickUnitSource)
+     * @see #createStandardTickUnits()
      */
     public static TickUnitSource createIntegerTickUnits() {
 
@@ -782,17 +822,19 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
     }
 
     /**
-     * Creates a collection of standard tick units.  The supplied locale is 
-     * used to create the number formatter (a localised instance of 
+     * Creates a collection of standard tick units.  The supplied locale is
+     * used to create the number formatter (a localised instance of
      * <code>NumberFormat</code>).
      * <P>
-     * If you don't like these defaults, create your own instance of 
-     * {@link TickUnits} and then pass it to the 
+     * If you don't like these defaults, create your own instance of
+     * {@link TickUnits} and then pass it to the
      * <code>setStandardTickUnits()</code> method.
      *
      * @param locale  the locale.
      *
      * @return A tick unit collection.
+     *
+     * @see #setStandardTickUnits(TickUnitSource)
      */
     public static TickUnitSource createStandardTickUnits(Locale locale) {
 
@@ -800,7 +842,7 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
 
         NumberFormat numberFormat = NumberFormat.getNumberInstance(locale);
 
-        // we can add the units in any order, the TickUnits collection will 
+        // we can add the units in any order, the TickUnits collection will
         // sort them...
         units.add(new NumberTickUnit(0.0000001,    numberFormat));
         units.add(new NumberTickUnit(0.000001,     numberFormat));
@@ -870,6 +912,8 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
      * @param locale the locale to use to represent Numbers.
      *
      * @return A collection of tick units for integer values.
+     *
+     * @see #setStandardTickUnits(TickUnitSource)
      */
     public static TickUnitSource createIntegerTickUnits(Locale locale) {
 
@@ -915,29 +959,29 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
 
     /**
      * Estimates the maximum tick label height.
-     * 
+     *
      * @param g2  the graphics device.
-     * 
+     *
      * @return The maximum height.
      */
     protected double estimateMaximumTickLabelHeight(Graphics2D g2) {
 
         RectangleInsets tickLabelInsets = getTickLabelInsets();
         double result = tickLabelInsets.getTop() + tickLabelInsets.getBottom();
-        
+
         Font tickLabelFont = getTickLabelFont();
         FontRenderContext frc = g2.getFontRenderContext();
         result += tickLabelFont.getLineMetrics("123", frc).getHeight();
         return result;
-        
+
     }
 
     /**
-     * Estimates the maximum width of the tick labels, assuming the specified 
+     * Estimates the maximum width of the tick labels, assuming the specified
      * tick unit is used.
      * <P>
-     * Rather than computing the string bounds of every tick on the axis, we 
-     * just look at two values: the lower bound and the upper bound for the 
+     * Rather than computing the string bounds of every tick on the axis, we
+     * just look at two values: the lower bound and the upper bound for the
      * axis.  These two values will usually be representative.
      *
      * @param g2  the graphics device.
@@ -945,14 +989,14 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
      *
      * @return The estimated maximum width of the tick labels.
      */
-    protected double estimateMaximumTickLabelWidth(Graphics2D g2, 
+    protected double estimateMaximumTickLabelWidth(Graphics2D g2,
                                                    TickUnit unit) {
 
         RectangleInsets tickLabelInsets = getTickLabelInsets();
         double result = tickLabelInsets.getLeft() + tickLabelInsets.getRight();
 
         if (isVerticalTickLabels()) {
-            // all tick labels have the same width (equal to the height of the 
+            // all tick labels have the same width (equal to the height of the
             // font)...
             FontRenderContext frc = g2.getFontRenderContext();
             LineMetrics lm = getTickLabelFont().getLineMetrics("0", frc);
@@ -964,8 +1008,17 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
             Range range = getRange();
             double lower = range.getLowerBound();
             double upper = range.getUpperBound();
-            String lowerStr = unit.valueToString(lower);
-            String upperStr = unit.valueToString(upper);
+            String lowerStr = "";
+            String upperStr = "";
+            NumberFormat formatter = getNumberFormatOverride();
+            if (formatter != null) {
+                lowerStr = formatter.format(lower);
+                upperStr = formatter.format(upper);
+            }
+            else {
+                lowerStr = unit.valueToString(lower);
+                upperStr = unit.valueToString(upper);
+            }
             double w1 = fm.stringWidth(lowerStr);
             double w2 = fm.stringWidth(upperStr);
             result += Math.max(w1, w2);
@@ -974,7 +1027,7 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
         return result;
 
     }
-    
+
     /**
      * Selects an appropriate tick value for the axis.  The strategy is to
      * display as many ticks as possible (selected from an array of 'standard'
@@ -1022,7 +1075,7 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
         // then extrapolate...
         double guess = (tickLabelWidth / unit1Width) * unit1.getSize();
 
-        NumberTickUnit unit2 
+        NumberTickUnit unit2
             = (NumberTickUnit) tickUnits.getCeilingTickUnit(guess);
         double unit2Width = lengthToJava2D(unit2.getSize(), dataArea, edge);
 
@@ -1044,8 +1097,8 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
      * @param dataArea  the area in which the plot should be drawn.
      * @param edge  the axis location.
      */
-    protected void selectVerticalAutoTickUnit(Graphics2D g2, 
-                                              Rectangle2D dataArea, 
+    protected void selectVerticalAutoTickUnit(Graphics2D g2,
+                                              Rectangle2D dataArea,
                                               RectangleEdge edge) {
 
         double tickLabelHeight = estimateMaximumTickLabelHeight(g2);
@@ -1057,8 +1110,8 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
 
         // then extrapolate...
         double guess = (tickLabelHeight / unitHeight) * unit1.getSize();
-        
-        NumberTickUnit unit2 
+
+        NumberTickUnit unit2
             = (NumberTickUnit) tickUnits.getCeilingTickUnit(guess);
         double unit2Height = lengthToJava2D(unit2.getSize(), dataArea, edge);
 
@@ -1070,20 +1123,20 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
         setTickUnit(unit2, false, false);
 
     }
-    
+
     /**
-     * Calculates the positions of the tick labels for the axis, storing the 
+     * Calculates the positions of the tick labels for the axis, storing the
      * results in the tick label list (ready for drawing).
      *
      * @param g2  the graphics device.
      * @param state  the axis state.
      * @param dataArea  the area in which the plot should be drawn.
      * @param edge  the location of the axis.
-     * 
+     *
      * @return A list of ticks.
      *
      */
-    public List refreshTicks(Graphics2D g2, 
+    public List refreshTicks(Graphics2D g2,
                              AxisState state,
                              Rectangle2D dataArea,
                              RectangleEdge edge) {
@@ -1100,13 +1153,13 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
     }
 
     /**
-     * Calculates the positions of the tick labels for the axis, storing the 
+     * Calculates the positions of the tick labels for the axis, storing the
      * results in the tick label list (ready for drawing).
      *
      * @param g2  the graphics device.
      * @param dataArea  the area in which the data should be drawn.
      * @param edge  the location of the axis.
-     * 
+     *
      * @return A list of ticks.
      */
     protected List refreshTicksHorizontal(Graphics2D g2,
@@ -1117,7 +1170,7 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
 
         Font tickLabelFont = getTickLabelFont();
         g2.setFont(tickLabelFont);
-        
+
         if (isAutoTickUnitSelection()) {
             selectAutoTickUnit(g2, dataArea, edge);
         }
@@ -1161,10 +1214,8 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
                     }
                 }
 
-                Tick tick = new NumberTick(
-                    new Double(currentTickValue), tickLabel, anchor, 
-                    rotationAnchor, angle
-                );
+                Tick tick = new NumberTick(new Double(currentTickValue),
+                		tickLabel, anchor, rotationAnchor, angle);
                 result.add(tick);
             }
         }
@@ -1173,15 +1224,14 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
     }
 
     /**
-     * Calculates the positions of the tick labels for the axis, storing the 
+     * Calculates the positions of the tick labels for the axis, storing the
      * results in the tick label list (ready for drawing).
      *
      * @param g2  the graphics device.
      * @param dataArea  the area in which the plot should be drawn.
      * @param edge  the location of the axis.
-     * 
-     * @return A list of ticks.
      *
+     * @return A list of ticks.
      */
     protected List refreshTicksVertical(Graphics2D g2,
                                         Rectangle2D dataArea,
@@ -1216,7 +1266,7 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
                 TextAnchor rotationAnchor = null;
                 double angle = 0.0;
                 if (isVerticalTickLabels()) {
-                    if (edge == RectangleEdge.LEFT) { 
+                    if (edge == RectangleEdge.LEFT) {
                         anchor = TextAnchor.BOTTOM_CENTER;
                         rotationAnchor = TextAnchor.BOTTOM_CENTER;
                         angle = -Math.PI / 2.0;
@@ -1238,29 +1288,27 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
                     }
                 }
 
-                Tick tick = new NumberTick(
-                    new Double(currentTickValue), tickLabel, anchor, 
-                    rotationAnchor, angle
-                );
+                Tick tick = new NumberTick(new Double(currentTickValue),
+                		tickLabel, anchor, rotationAnchor, angle);
                 result.add(tick);
             }
         }
         return result;
 
     }
-    
+
     /**
      * Returns a clone of the axis.
-     * 
+     *
      * @return A clone
-     * 
-     * @throws CloneNotSupportedException if some component of the axis does 
+     *
+     * @throws CloneNotSupportedException if some component of the axis does
      *         not support cloning.
      */
     public Object clone() throws CloneNotSupportedException {
         NumberAxis clone = (NumberAxis) super.clone();
         if (this.numberFormatOverride != null) {
-            clone.numberFormatOverride 
+            clone.numberFormatOverride
                 = (NumberFormat) this.numberFormatOverride.clone();
         }
         return clone;
@@ -1268,12 +1316,12 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
 
     /**
      * Tests the axis for equality with an arbitrary object.
-     * 
+     *
      * @param obj  the object (<code>null</code> permitted).
-     * 
+     *
      * @return A boolean.
-     */    
-    public boolean equals(Object obj) {           
+     */
+    public boolean equals(Object obj) {
         if (obj == this) {
             return true;
         }
@@ -1283,7 +1331,7 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
         if (!super.equals(obj)) {
             return false;
         }
-        NumberAxis that = (NumberAxis) obj;        
+        NumberAxis that = (NumberAxis) obj;
         if (this.autoRangeIncludesZero != that.autoRangeIncludesZero) {
             return false;
         }
@@ -1293,19 +1341,19 @@ public class NumberAxis extends ValueAxis implements Cloneable, Serializable {
         if (!ObjectUtilities.equal(this.tickUnit, that.tickUnit)) {
             return false;
         }
-        if (!ObjectUtilities.equal(this.numberFormatOverride, 
+        if (!ObjectUtilities.equal(this.numberFormatOverride,
                 that.numberFormatOverride)) {
             return false;
         }
         if (!this.rangeType.equals(that.rangeType)) {
             return false;
         }
-        return true; 
+        return true;
     }
-    
+
     /**
      * Returns a hash code for this object.
-     * 
+     *
      * @return A hash code.
      */
     public int hashCode() {
