@@ -2,32 +2,32 @@
  * JFreeChart : a free chart library for the Java(tm) platform
  * ===========================================================
  *
- * (C) Copyright 2000-2006, by Object Refinery Limited and Contributors.
+ * (C) Copyright 2000-2008, by Object Refinery Limited and Contributors.
  *
  * Project Info:  http://www.jfree.org/jfreechart/index.html
  *
- * This library is free software; you can redistribute it and/or modify it 
- * under the terms of the GNU Lesser General Public License as published by 
- * the Free Software Foundation; either version 2.1 of the License, or 
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful, but 
- * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
- * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+ * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
  * License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, 
- * USA.  
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
+ * USA.
  *
- * [Java is a trademark or registered trademark of Sun Microsystems, Inc. 
+ * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
  * in the United States and other countries.]
  *
  * -------------------------
  * CombinedDomainXYPlot.java
  * -------------------------
- * (C) Copyright 2001-2006, by Bill Kelemen and Contributors.
+ * (C) Copyright 2001-2008, by Bill Kelemen and Contributors.
  *
  * Original Author:  Bill Kelemen;
  * Contributor(s):   David Gilbert (for Object Refinery Limited);
@@ -35,25 +35,24 @@
  *                   David Basten;
  *                   Kevin Frechette (for ISTI);
  *                   Nicolas Brodu;
- *
- * $Id: CombinedDomainXYPlot.java,v 1.9.2.2 2006/08/23 09:44:17 mungady Exp $
+ *                   Petr Kubanek (bug 1606205);
  *
  * Changes:
  * --------
  * 06-Dec-2001 : Version 1 (BK);
  * 12-Dec-2001 : Removed unnecessary 'throws' clause from constructor (DG);
  * 18-Dec-2001 : Added plotArea attribute and get/set methods (BK);
- * 22-Dec-2001 : Fixed bug in chartChanged with multiple combinations of 
+ * 22-Dec-2001 : Fixed bug in chartChanged with multiple combinations of
  *               CombinedPlots (BK);
  * 08-Jan-2002 : Moved to new package com.jrefinery.chart.combination (DG);
  * 25-Feb-2002 : Updated import statements (DG);
- * 28-Feb-2002 : Readded "this.plotArea = plotArea" that was deleted from 
+ * 28-Feb-2002 : Readded "this.plotArea = plotArea" that was deleted from
  *               draw() method (BK);
  * 26-Mar-2002 : Added an empty zoom method (this method needs to be written so
  *               that combined plots will support zooming (DG);
- * 29-Mar-2002 : Changed the method createCombinedAxis adding the creation of 
+ * 29-Mar-2002 : Changed the method createCombinedAxis adding the creation of
  *               OverlaidSymbolicAxis and CombinedSymbolicAxis(AB);
- * 23-Apr-2002 : Renamed CombinedPlot-->MultiXYPlot, and simplified the    
+ * 23-Apr-2002 : Renamed CombinedPlot-->MultiXYPlot, and simplified the
  *               structure (DG);
  * 23-May-2002 : Renamed (again) MultiXYPlot-->CombinedXYPlot (DG);
  * 19-Jun-2002 : Added get/setGap() methods suggested by David Basten (DG);
@@ -64,7 +63,7 @@
  * 09-Oct-2002 : Added add(XYPlot) method (DG);
  * 26-Mar-2003 : Implemented Serializable (DG);
  * 16-May-2003 : Renamed CombinedXYPlot --> CombinedDomainXYPlot (DG);
- * 04-Aug-2003 : Removed leftover code that was causing domain axis drawing 
+ * 04-Aug-2003 : Removed leftover code that was causing domain axis drawing
  *               problem (DG);
  * 08-Aug-2003 : Adjusted totalWeight in remove() method (DG);
  * 21-Aug-2003 : Implemented Cloneable (DG);
@@ -77,8 +76,19 @@
  * 21-Feb-2005 : The getLegendItems() method now returns the fixed legend
  *               items if set (DG);
  * 05-May-2005 : Removed unused draw() method (DG);
- * ------------- JFREECHART 1.0.0 ---------------------------------------------
+ * ------------- JFREECHART 1.0.x ---------------------------------------------
  * 23-Aug-2006 : Override setFixedRangeAxisSpace() to update subplots (DG);
+ * 06-Feb-2007 : Fixed bug 1606205, draw shared axis after subplots (DG);
+ * 23-Mar-2007 : Reverted previous patch (bug fix 1606205) (DG);
+ * 17-Apr-2007 : Added null argument checks to findSubplot() (DG);
+ * 27-Nov-2007 : Modified setFixedRangeAxisSpaceForSubplots() so as not to
+ *               trigger change event in subplots (DG);
+ * 28-Jan-2008 : Reset fixed range axis space in subplots for each call to
+ *               draw() (DG);
+ * 27-Mar-2008 : Add documentation for getDataRange() method (DG);
+ * 31-Mar-2008 : Updated getSubplots() to return EMPTY_LIST for null
+ *               subplots, as suggested by Richard West (DG);
+ * 28-Apr-2008 : Fixed zooming problem (see bug 1950037) (DG);
  *
  */
 
@@ -87,7 +97,6 @@ package org.jfree.chart.plot;
 import java.awt.Graphics2D;
 import java.awt.geom.Point2D;
 import java.awt.geom.Rectangle2D;
-import java.io.Serializable;
 import java.util.Collections;
 import java.util.Iterator;
 import java.util.List;
@@ -104,20 +113,17 @@ import org.jfree.data.Range;
 import org.jfree.ui.RectangleEdge;
 import org.jfree.ui.RectangleInsets;
 import org.jfree.util.ObjectUtilities;
-import org.jfree.util.PublicCloneable;
 
 /**
- * An extension of {@link XYPlot} that contains multiple subplots that share a 
+ * An extension of {@link XYPlot} that contains multiple subplots that share a
  * common domain axis.
  */
-public class CombinedDomainXYPlot extends XYPlot 
-                                  implements Cloneable, PublicCloneable, 
-                                             Serializable,
-                                             PlotChangeListener {
+public class CombinedDomainXYPlot extends XYPlot
+        implements PlotChangeListener {
 
     /** For serialization. */
     private static final long serialVersionUID = -7765545541261907383L;
-    
+
     /** Storage for the subplot references. */
     private List subplots;
 
@@ -131,16 +137,16 @@ public class CombinedDomainXYPlot extends XYPlot
     private transient Rectangle2D[] subplotAreas;
     // TODO:  the subplot areas needs to be moved out of the plot into the plot
     //        state
-    
+
     /**
      * Default constructor.
      */
     public CombinedDomainXYPlot() {
-        this(new NumberAxis());      
+        this(new NumberAxis());
     }
-    
+
     /**
-     * Creates a new combined plot that shares a domain axis among multiple 
+     * Creates a new combined plot that shares a domain axis among multiple
      * subplots.
      *
      * @param domainAxis  the shared axis.
@@ -152,7 +158,7 @@ public class CombinedDomainXYPlot extends XYPlot
             domainAxis,
             null,        // no range axis
             null         // no rendereer
-        );  
+        );
 
         this.subplots = new java.util.ArrayList();
 
@@ -168,9 +174,9 @@ public class CombinedDomainXYPlot extends XYPlot
     }
 
     /**
-     * Sets the orientation for the plot (also changes the orientation for all 
+     * Sets the orientation for the plot (also changes the orientation for all
      * the subplots to match).
-     * 
+     *
      * @param orientation  the orientation (<code>null</code> not allowed).
      */
     public void setOrientation(PlotOrientation orientation) {
@@ -185,15 +191,19 @@ public class CombinedDomainXYPlot extends XYPlot
     }
 
     /**
-     * Returns the range for the specified axis.  This is the combined range 
-     * of all the subplots.
+     * Returns a range representing the extent of the data values in this plot
+     * (obtained from the subplots) that will be rendered against the specified
+     * axis.  NOTE: This method is intended for internal JFreeChart use, and
+     * is public only so that code in the axis classes can call it.  Since
+     * only the domain axis is shared between subplots, the JFreeChart code
+     * will only call this method for the domain values (although this is not
+     * checked/enforced).
      *
      * @param axis  the axis.
      *
      * @return The range (possibly <code>null</code>).
      */
     public Range getDataRange(ValueAxis axis) {
-
         Range result = null;
         if (this.subplots != null) {
             Iterator iterator = this.subplots.iterator();
@@ -203,7 +213,6 @@ public class CombinedDomainXYPlot extends XYPlot
             }
         }
         return result;
-
     }
 
     /**
@@ -216,21 +225,22 @@ public class CombinedDomainXYPlot extends XYPlot
     }
 
     /**
-     * Sets the amount of space between subplots and sends a 
+     * Sets the amount of space between subplots and sends a
      * {@link PlotChangeEvent} to all registered listeners.
      *
      * @param gap  the gap between subplots (in Java2D units).
      */
     public void setGap(double gap) {
         this.gap = gap;
-        notifyListeners(new PlotChangeEvent(this));
+        fireChangeEvent();
     }
 
     /**
-     * Adds a subplot (with a default 'weight' of 1) and sends a 
+     * Adds a subplot (with a default 'weight' of 1) and sends a
      * {@link PlotChangeEvent} to all registered listeners.
      * <P>
-     * The domain axis for the subplot will be set to <code>null</code>.
+     * The domain axis for the subplot will be set to <code>null</code>.  You
+     * must ensure that the subplot has a non-null range axis.
      *
      * @param subplot  the subplot (<code>null</code> not permitted).
      */
@@ -240,12 +250,13 @@ public class CombinedDomainXYPlot extends XYPlot
     }
 
     /**
-     * Adds a subplot with the specified weight and sends a 
-     * {@link PlotChangeEvent} to all registered listeners.  The weight 
-     * determines how much space is allocated to the subplot relative to all 
+     * Adds a subplot with the specified weight and sends a
+     * {@link PlotChangeEvent} to all registered listeners.  The weight
+     * determines how much space is allocated to the subplot relative to all
      * the other subplots.
      * <P>
-     * The domain axis for the subplot will be set to <code>null</code>.
+     * The domain axis for the subplot will be set to <code>null</code>.  You
+     * must ensure that the subplot has a non-null range axis.
      *
      * @param subplot  the subplot (<code>null</code> not permitted).
      * @param weight  the weight (must be >= 1).
@@ -274,20 +285,18 @@ public class CombinedDomainXYPlot extends XYPlot
         if (axis != null) {
             axis.configure();
         }
-        
-        notifyListeners(new PlotChangeEvent(this));
-
+        fireChangeEvent();
     }
 
     /**
-     * Removes a subplot from the combined chart and sends a 
+     * Removes a subplot from the combined chart and sends a
      * {@link PlotChangeEvent} to all registered listeners.
      *
      * @param subplot  the subplot (<code>null</code> not permitted).
      */
     public void remove(XYPlot subplot) {
         if (subplot == null) {
-            throw new IllegalArgumentException(" Null 'subplot' argument.");   
+            throw new IllegalArgumentException(" Null 'subplot' argument.");
         }
         int position = -1;
         int size = this.subplots.size();
@@ -308,33 +317,39 @@ public class CombinedDomainXYPlot extends XYPlot
             if (domain != null) {
                 domain.configure();
             }
-            notifyListeners(new PlotChangeEvent(this));
+            fireChangeEvent();
         }
     }
 
     /**
-     * Returns the list of subplots.
+     * Returns the list of subplots.  The returned list may be empty, but is
+     * never <code>null</code>.
      *
      * @return An unmodifiable list of subplots.
      */
     public List getSubplots() {
-        return Collections.unmodifiableList(this.subplots);
+        if (this.subplots != null) {
+            return Collections.unmodifiableList(this.subplots);
+        }
+        else {
+            return Collections.EMPTY_LIST;
+        }
     }
 
     /**
      * Calculates the axis space required.
-     * 
+     *
      * @param g2  the graphics device.
      * @param plotArea  the plot area.
-     * 
+     *
      * @return The space.
      */
-    protected AxisSpace calculateAxisSpace(Graphics2D g2, 
+    protected AxisSpace calculateAxisSpace(Graphics2D g2,
                                            Rectangle2D plotArea) {
-        
+
         AxisSpace space = new AxisSpace();
         PlotOrientation orientation = getOrientation();
-        
+
         // work out the space required by the domain axis...
         AxisSpace fixed = getFixedDomainAxisSpace();
         if (fixed != null) {
@@ -344,21 +359,20 @@ public class CombinedDomainXYPlot extends XYPlot
             }
             else if (orientation == PlotOrientation.VERTICAL) {
                 space.setTop(fixed.getTop());
-                space.setBottom(fixed.getBottom());                
+                space.setBottom(fixed.getBottom());
             }
         }
         else {
             ValueAxis xAxis = getDomainAxis();
             RectangleEdge xEdge = Plot.resolveDomainAxisLocation(
-                getDomainAxisLocation(), orientation
-            );
+                    getDomainAxisLocation(), orientation);
             if (xAxis != null) {
                 space = xAxis.reserveSpace(g2, this, plotArea, xEdge, space);
             }
         }
-        
+
         Rectangle2D adjustedPlotArea = space.shrink(plotArea, null);
-        
+
         // work out the maximum height or width of the non-shared axes...
         int n = this.subplots.size();
         this.subplotAreas = new Rectangle2D[n];
@@ -378,39 +392,36 @@ public class CombinedDomainXYPlot extends XYPlot
             // calculate sub-plot area
             if (orientation == PlotOrientation.HORIZONTAL) {
                 double w = usableSize * plot.getWeight() / this.totalWeight;
-                this.subplotAreas[i] = new Rectangle2D.Double(
-                    x, y, w, adjustedPlotArea.getHeight()
-                );
+                this.subplotAreas[i] = new Rectangle2D.Double(x, y, w,
+                        adjustedPlotArea.getHeight());
                 x = x + w + this.gap;
             }
             else if (orientation == PlotOrientation.VERTICAL) {
                 double h = usableSize * plot.getWeight() / this.totalWeight;
-                this.subplotAreas[i] = new Rectangle2D.Double(
-                    x, y, adjustedPlotArea.getWidth(), h
-                );
+                this.subplotAreas[i] = new Rectangle2D.Double(x, y,
+                        adjustedPlotArea.getWidth(), h);
                 y = y + h + this.gap;
             }
 
-            AxisSpace subSpace = plot.calculateRangeAxisSpace(
-                g2, this.subplotAreas[i], null
-            );
+            AxisSpace subSpace = plot.calculateRangeAxisSpace(g2,
+                    this.subplotAreas[i], null);
             space.ensureAtLeast(subSpace);
 
         }
 
         return space;
     }
-    
+
     /**
      * Draws the plot within the specified area on a graphics device.
-     * 
+     *
      * @param g2  the graphics device.
      * @param area  the plot area (in Java2D space).
-     * @param anchor  an anchor point in Java2D space (<code>null</code> 
+     * @param anchor  an anchor point in Java2D space (<code>null</code>
      *                permitted).
-     * @param parentState  the state from the parent plot, if there is one 
+     * @param parentState  the state from the parent plot, if there is one
      *                     (<code>null</code> permitted).
-     * @param info  collects chart drawing information (<code>null</code> 
+     * @param info  collects chart drawing information (<code>null</code>
      *              permitted).
      */
     public void draw(Graphics2D g2,
@@ -418,7 +429,7 @@ public class CombinedDomainXYPlot extends XYPlot
                      Point2D anchor,
                      PlotState parentState,
                      PlotRenderingInfo info) {
-        
+
         // set up info collection...
         if (info != null) {
             info.setPlotArea(area);
@@ -428,6 +439,7 @@ public class CombinedDomainXYPlot extends XYPlot
         RectangleInsets insets = getInsets();
         insets.trim(area);
 
+        setFixedRangeAxisSpaceForSubplots(null);
         AxisSpace space = calculateAxisSpace(g2, area);
         Rectangle2D dataArea = space.shrink(area, null);
 
@@ -443,7 +455,7 @@ public class CombinedDomainXYPlot extends XYPlot
             parentState = new PlotState();
         }
         parentState.getSharedAxisStates().put(axis, axisState);
-        
+
         // draw all the subplots
         for (int i = 0; i < this.subplots.size(); i++) {
             XYPlot plot = (XYPlot) this.subplots.get(i);
@@ -452,15 +464,14 @@ public class CombinedDomainXYPlot extends XYPlot
                 subplotInfo = new PlotRenderingInfo(info.getOwner());
                 info.addSubplotInfo(subplotInfo);
             }
-            plot.draw(
-                g2, this.subplotAreas[i], anchor, parentState, subplotInfo
-            );
+            plot.draw(g2, this.subplotAreas[i], anchor, parentState,
+                    subplotInfo);
         }
 
         if (info != null) {
             info.setDataArea(dataArea);
         }
-        
+
     }
 
     /**
@@ -468,7 +479,7 @@ public class CombinedDomainXYPlot extends XYPlot
      *
      * @return The legend items.
      */
-    public LegendItemCollection getLegendItems() {        
+    public LegendItemCollection getLegendItems() {
         LegendItemCollection result = getFixedLegendItems();
         if (result == null) {
             result = new LegendItemCollection();
@@ -488,14 +499,37 @@ public class CombinedDomainXYPlot extends XYPlot
      * Multiplies the range on the range axis/axes by the specified factor.
      *
      * @param factor  the zoom factor.
-     * @param info  the plot rendering info.
-     * @param source  the source point.
+     * @param info  the plot rendering info (<code>null</code> not permitted).
+     * @param source  the source point (<code>null</code> not permitted).
      */
-    public void zoomRangeAxes(double factor, PlotRenderingInfo info, 
+    public void zoomRangeAxes(double factor, PlotRenderingInfo info,
                               Point2D source) {
-        XYPlot subplot = findSubplot(info, source);
+        zoomRangeAxes(factor, info, source, false);
+    }
+
+    /**
+     * Multiplies the range on the range axis/axes by the specified factor.
+     *
+     * @param factor  the zoom factor.
+     * @param state  the plot state.
+     * @param source  the source point (in Java2D coordinates).
+     * @param useAnchor  use source point as zoom anchor?
+     */
+    public void zoomRangeAxes(double factor, PlotRenderingInfo state,
+                              Point2D source, boolean useAnchor) {
+        // delegate 'state' and 'source' argument checks...
+        XYPlot subplot = findSubplot(state, source);
         if (subplot != null) {
-            subplot.zoomRangeAxes(factor, info, source);
+            subplot.zoomRangeAxes(factor, state, source, useAnchor);
+        }
+        else {
+            // if the source point doesn't fall within a subplot, we do the
+            // zoom on all subplots...
+            Iterator iterator = getSubplots().iterator();
+            while (iterator.hasNext()) {
+                subplot = (XYPlot) iterator.next();
+                subplot.zoomRangeAxes(factor, state, source, useAnchor);
+            }
         }
     }
 
@@ -504,27 +538,43 @@ public class CombinedDomainXYPlot extends XYPlot
      *
      * @param lowerPercent  the lower bound.
      * @param upperPercent  the upper bound.
-     * @param info  the plot rendering info.
-     * @param source  the source point.
+     * @param info  the plot rendering info (<code>null</code> not permitted).
+     * @param source  the source point (<code>null</code> not permitted).
      */
-    public void zoomRangeAxes(double lowerPercent, double upperPercent, 
+    public void zoomRangeAxes(double lowerPercent, double upperPercent,
                               PlotRenderingInfo info, Point2D source) {
+        // delegate 'info' and 'source' argument checks...
         XYPlot subplot = findSubplot(info, source);
         if (subplot != null) {
             subplot.zoomRangeAxes(lowerPercent, upperPercent, info, source);
         }
+        else {
+            // if the source point doesn't fall within a subplot, we do the
+            // zoom on all subplots...
+            Iterator iterator = getSubplots().iterator();
+            while (iterator.hasNext()) {
+                subplot = (XYPlot) iterator.next();
+                subplot.zoomRangeAxes(lowerPercent, upperPercent, info, source);
+            }
+        }
     }
 
     /**
-     * Returns the subplot (if any) that contains the (x, y) point (specified 
+     * Returns the subplot (if any) that contains the (x, y) point (specified
      * in Java2D space).
-     * 
-     * @param info  the chart rendering info.
-     * @param source  the source point.
-     * 
+     *
+     * @param info  the chart rendering info (<code>null</code> not permitted).
+     * @param source  the source point (<code>null</code> not permitted).
+     *
      * @return A subplot (possibly <code>null</code>).
      */
     public XYPlot findSubplot(PlotRenderingInfo info, Point2D source) {
+        if (info == null) {
+            throw new IllegalArgumentException("Null 'info' argument.");
+        }
+        if (source == null) {
+            throw new IllegalArgumentException("Null 'source' argument.");
+        }
         XYPlot result = null;
         int subplotIndex = info.getSubplotIndex(source);
         if (subplotIndex >= 0) {
@@ -532,19 +582,19 @@ public class CombinedDomainXYPlot extends XYPlot
         }
         return result;
     }
-    
+
     /**
-     * Sets the item renderer FOR ALL SUBPLOTS.  Registered listeners are 
+     * Sets the item renderer FOR ALL SUBPLOTS.  Registered listeners are
      * notified that the plot has been modified.
      * <P>
-     * Note: usually you will want to set the renderer independently for each 
+     * Note: usually you will want to set the renderer independently for each
      * subplot, which is NOT what this method does.
      *
      * @param renderer the new renderer.
      */
     public void setRenderer(XYItemRenderer renderer) {
 
-        super.setRenderer(renderer);  // not strictly necessary, since the 
+        super.setRenderer(renderer);  // not strictly necessary, since the
                                       // renderer set for the
                                       // parent plot is not used
 
@@ -557,30 +607,29 @@ public class CombinedDomainXYPlot extends XYPlot
     }
 
     /**
-     * Sets the fixed range axis space.
+     * Sets the fixed range axis space and sends a {@link PlotChangeEvent} to
+     * all registered listeners.
      *
      * @param space  the space (<code>null</code> permitted).
      */
     public void setFixedRangeAxisSpace(AxisSpace space) {
         super.setFixedRangeAxisSpace(space);
         setFixedRangeAxisSpaceForSubplots(space);
-        this.notifyListeners(new PlotChangeEvent(this));
+        fireChangeEvent();
     }
-    
+
     /**
-     * Sets the size (width or height, depending on the orientation of the 
+     * Sets the size (width or height, depending on the orientation of the
      * plot) for the domain axis of each subplot.
      *
      * @param space  the space.
      */
     protected void setFixedRangeAxisSpaceForSubplots(AxisSpace space) {
-
         Iterator iterator = this.subplots.iterator();
         while (iterator.hasNext()) {
             XYPlot plot = (XYPlot) iterator.next();
-            plot.setFixedRangeAxisSpace(space);
+            plot.setFixedRangeAxisSpace(space, false);
         }
-
     }
 
     /**
@@ -600,11 +649,11 @@ public class CombinedDomainXYPlot extends XYPlot
             }
         }
     }
-    
+
     /**
-     * Receives a {@link PlotChangeEvent} and responds by notifying all 
+     * Receives a {@link PlotChangeEvent} and responds by notifying all
      * listeners.
-     * 
+     *
      * @param event  the event.
      */
     public void plotChanged(PlotChangeEvent event) {
@@ -648,33 +697,33 @@ public class CombinedDomainXYPlot extends XYPlot
 
         return true;
     }
-    
+
     /**
      * Returns a clone of the annotation.
-     * 
+     *
      * @return A clone.
-     * 
-     * @throws CloneNotSupportedException  this class will not throw this 
+     *
+     * @throws CloneNotSupportedException  this class will not throw this
      *         exception, but subclasses (if any) might.
      */
     public Object clone() throws CloneNotSupportedException {
-        
-        CombinedDomainXYPlot result = (CombinedDomainXYPlot) super.clone(); 
+
+        CombinedDomainXYPlot result = (CombinedDomainXYPlot) super.clone();
         result.subplots = (List) ObjectUtilities.deepClone(this.subplots);
         for (Iterator it = result.subplots.iterator(); it.hasNext();) {
             Plot child = (Plot) it.next();
             child.setParent(result);
         }
-        
-        // after setting up all the subplots, the shared domain axis may need 
+
+        // after setting up all the subplots, the shared domain axis may need
         // reconfiguring
         ValueAxis domainAxis = result.getDomainAxis();
         if (domainAxis != null) {
             domainAxis.configure();
         }
-        
+
         return result;
-        
+
     }
-    
+
 }
