@@ -2,7 +2,7 @@
  * JFreeChart : a free chart library for the Java(tm) platform
  * ===========================================================
  *
- * (C) Copyright 2000-2005, by Object Refinery Limited and Contributors.
+ * (C) Copyright 2000-2008, by Object Refinery Limited and Contributors.
  *
  * Project Info:  http://www.jfree.org/jfreechart/index.html
  *
@@ -27,13 +27,11 @@
  * ------------------------------
  * CombinedRangeCategoryPlot.java
  * ------------------------------
- * (C) Copyright 2003-2005, by Object Refinery Limited.
+ * (C) Copyright 2003-2008, by Object Refinery Limited.
  *
  * Original Author:  David Gilbert (for Object Refinery Limited);
  * Contributor(s):   Nicolas Brodu;
  *
- * $Id: CombinedRangeCategoryPlot.java,v 1.13.2.1 2005/10/25 20:52:08 mungady Exp $
- *
  * Changes:
  * --------
  * 16-May-2003 : Version 1 (DG);
@@ -51,6 +49,10 @@
  * 21-Feb-2005 : The getLegendItems() method now returns the fixed legend
  *               items if set (DG);
  * 05-May-2005 : Updated draw() method parameters (DG);
+ * 14-Nov-2007 : Updated setFixedDomainAxisSpaceForSubplots() method (DG);
+ * 27-Mar-2008 : Add documentation for getDataRange() method (DG);
+ * 31-Mar-2008 : Updated getSubplots() to return EMPTY_LIST for null 
+ *               subplots, as suggested by Richard West (DG);
  * 
  */
  
@@ -61,7 +63,6 @@ import java.awt.geom.Point2D;
 import java.awt.geom.Rectangle2D;
 import java.io.IOException;
 import java.io.ObjectInputStream;
-import java.io.Serializable;
 import java.util.Collections;
 import java.util.Iterator;
 import java.util.List;
@@ -77,16 +78,12 @@ import org.jfree.data.Range;
 import org.jfree.ui.RectangleEdge;
 import org.jfree.ui.RectangleInsets;
 import org.jfree.util.ObjectUtilities;
-import org.jfree.util.PublicCloneable;
 
 /**
  * A combined category plot where the range axis is shared.
  */
 public class CombinedRangeCategoryPlot extends CategoryPlot 
-                                       implements Zoomable,
-                                                  Cloneable, PublicCloneable, 
-                                                  Serializable,
-                                                  PlotChangeListener {
+        implements PlotChangeListener {
 
     /** For serialization. */
     private static final long serialVersionUID = 7260210007554504515L;
@@ -139,12 +136,15 @@ public class CombinedRangeCategoryPlot extends CategoryPlot
      */
     public void setGap(double gap) {
         this.gap = gap;
-        notifyListeners(new PlotChangeEvent(this));
+        fireChangeEvent();
     }
 
     /**
      * Adds a subplot (with a default 'weight' of 1) and sends a 
      * {@link PlotChangeEvent} to all registered listeners.
+     * <br><br>
+     * You must ensure that the subplot has a non-null domain axis.  The range
+     * axis for the subplot will be set to <code>null</code>.  
      *
      * @param subplot  the subplot (<code>null</code> not permitted).
      */
@@ -156,6 +156,9 @@ public class CombinedRangeCategoryPlot extends CategoryPlot
     /**
      * Adds a subplot and sends a {@link PlotChangeEvent} to all registered 
      * listeners.
+     * <br><br>
+     * You must ensure that the subplot has a non-null domain axis.  The range
+     * axis for the subplot will be set to <code>null</code>.  
      *
      * @param subplot  the subplot (<code>null</code> not permitted).
      * @param weight  the weight (must be >= 1).
@@ -182,7 +185,7 @@ public class CombinedRangeCategoryPlot extends CategoryPlot
         if (axis != null) {
             axis.configure();
         }
-        notifyListeners(new PlotChangeEvent(this));
+        fireChangeEvent();
     }
 
     /**
@@ -218,17 +221,23 @@ public class CombinedRangeCategoryPlot extends CategoryPlot
             if (range2 != null) {
                 range2.configure();
             }
-            notifyListeners(new PlotChangeEvent(this));
+            fireChangeEvent();
         }
     }
 
     /**
-     * Returns the list of subplots.
+     * Returns the list of subplots.  The returned list may be empty, but is
+     * never <code>null</code>.
      *
-     * @return The list (unmodifiable).
+     * @return An unmodifiable list of subplots.
      */
     public List getSubplots() {
-        return Collections.unmodifiableList(this.subplots);
+        if (this.subplots != null) {
+            return Collections.unmodifiableList(this.subplots);
+        }
+        else {
+            return Collections.EMPTY_LIST;
+        }
     }
 
     /**
@@ -260,12 +269,10 @@ public class CombinedRangeCategoryPlot extends CategoryPlot
         else {
             ValueAxis valueAxis = getRangeAxis();
             RectangleEdge valueEdge = Plot.resolveRangeAxisLocation(
-                getRangeAxisLocation(), orientation
-            );
+                    getRangeAxisLocation(), orientation);
             if (valueAxis != null) {
-                space = valueAxis.reserveSpace(
-                    g2, this, plotArea, valueEdge, space
-                );
+                space = valueAxis.reserveSpace(g2, this, plotArea, valueEdge, 
+                        space);
             }
         }
         
@@ -292,22 +299,19 @@ public class CombinedRangeCategoryPlot extends CategoryPlot
             // calculate sub-plot area
             if (orientation == PlotOrientation.VERTICAL) {
                 double w = usableSize * plot.getWeight() / this.totalWeight;
-                this.subplotArea[i] = new Rectangle2D.Double(
-                    x, y, w, adjustedPlotArea.getHeight()
-                );
+                this.subplotArea[i] = new Rectangle2D.Double(x, y, w, 
+                        adjustedPlotArea.getHeight());
                 x = x + w + this.gap;
             }
             else if (orientation == PlotOrientation.HORIZONTAL) {
                 double h = usableSize * plot.getWeight() / this.totalWeight;
-                this.subplotArea[i] = new Rectangle2D.Double(
-                    x, y, adjustedPlotArea.getWidth(), h
-                );
+                this.subplotArea[i] = new Rectangle2D.Double(x, y, 
+                        adjustedPlotArea.getWidth(), h);
                 y = y + h + this.gap;
             }
 
-            AxisSpace subSpace = plot.calculateDomainAxisSpace(
-                g2, this.subplotArea[i], null
-            );
+            AxisSpace subSpace = plot.calculateDomainAxisSpace(g2, 
+                    this.subplotArea[i], null);
             space.ensureAtLeast(subSpace);
 
         }
@@ -352,9 +356,8 @@ public class CombinedRangeCategoryPlot extends CategoryPlot
         ValueAxis axis = getRangeAxis();
         RectangleEdge rangeEdge = getRangeAxisEdge();
         double cursor = RectangleEdge.coordinate(dataArea, rangeEdge);
-        AxisState state = axis.draw(
-            g2, cursor, area, dataArea, rangeEdge, info
-        );
+        AxisState state = axis.draw(g2, cursor, area, dataArea, rangeEdge, 
+                info);
         if (parentState == null) {
             parentState = new PlotState();
         }
@@ -395,15 +398,19 @@ public class CombinedRangeCategoryPlot extends CategoryPlot
     }
     
     /**
-      * Returns the range for the axis.  This is the combined range of all the
-      * subplots.
+     * Returns a range representing the extent of the data values in this plot
+     * (obtained from the subplots) that will be rendered against the specified 
+     * axis.  NOTE: This method is intended for internal JFreeChart use, and 
+     * is public only so that code in the axis classes can call it.  Since 
+     * only the range axis is shared between subplots, the JFreeChart code 
+     * will only call this method for the range values (although this is not 
+     * checked/enforced).
       *
       * @param axis  the axis.
       *
       * @return The range.
       */
      public Range getDataRange(ValueAxis axis) {
-
          Range result = null;
          if (this.subplots != null) {
              Iterator iterator = this.subplots.iterator();
@@ -413,7 +420,6 @@ public class CombinedRangeCategoryPlot extends CategoryPlot
              }
          }
          return result;
-
      }
 
     /**
@@ -444,13 +450,11 @@ public class CombinedRangeCategoryPlot extends CategoryPlot
      * @param space  the space.
      */
     protected void setFixedDomainAxisSpaceForSubplots(AxisSpace space) {
-
         Iterator iterator = this.subplots.iterator();
         while (iterator.hasNext()) {
             CategoryPlot plot = (CategoryPlot) iterator.next();
-            plot.setFixedDomainAxisSpace(space);
+            plot.setFixedDomainAxisSpace(space, false);
         }
-
     }
 
     /**
