@@ -223,7 +223,7 @@ public class CompassPlot extends Plot implements Cloneable, Serializable {
         }
         if (this.labelType != type) {
             this.labelType = type;
-            notifyListeners(new PlotChangeEvent(this));
+            fireChangeEvent();
         }
     }
 
@@ -253,7 +253,7 @@ public class CompassPlot extends Plot implements Cloneable, Serializable {
             throw new IllegalArgumentException("Null 'font' not allowed.");
         }
         this.labelFont = font;
-        notifyListeners(new PlotChangeEvent(this));
+        fireChangeEvent();
     }
 
     /**
@@ -280,7 +280,7 @@ public class CompassPlot extends Plot implements Cloneable, Serializable {
             throw new IllegalArgumentException("Null 'paint' argument.");
         }
         this.rosePaint = paint;
-        notifyListeners(new PlotChangeEvent(this));        
+        fireChangeEvent();       
     }
 
     /**
@@ -308,7 +308,7 @@ public class CompassPlot extends Plot implements Cloneable, Serializable {
             throw new IllegalArgumentException("Null 'paint' argument.");
         }
         this.roseCenterPaint = paint;
-        notifyListeners(new PlotChangeEvent(this));        
+        fireChangeEvent();     
     }
     
     /**
@@ -336,7 +336,7 @@ public class CompassPlot extends Plot implements Cloneable, Serializable {
             throw new IllegalArgumentException("Null 'paint' argument.");
         }
         this.roseHighlightPaint = paint;
-        notifyListeners(new PlotChangeEvent(this));        
+        fireChangeEvent();     
     }
     
     /**
@@ -359,7 +359,7 @@ public class CompassPlot extends Plot implements Cloneable, Serializable {
      */
     public void setDrawBorder(boolean status) {
         this.drawBorder = status;
-        notifyListeners(new PlotChangeEvent(this));
+        fireChangeEvent();
     }
 
     /**
@@ -493,12 +493,10 @@ public class CompassPlot extends Plot implements Cloneable, Serializable {
      * @param needle  the needle.
      */
     public void setSeriesNeedle(int index, MeterNeedle needle) {
-
         if ((needle != null) && (index < this.seriesNeedle.length)) {
             this.seriesNeedle[index] = needle;
         }
-        notifyListeners(new PlotChangeEvent(this));
-
+        fireChangeEvent();
     }
 
     /**
