@@ -2,7 +2,7 @@
  * JFreeChart : a free chart library for the Java(tm) platform
  * ===========================================================
  *
- * (C) Copyright 2000-2005, by Object Refinery Limited and Contributors.
+ * (C) Copyright 2000-2008, by Object Refinery Limited and Contributors.
  *
  * Project Info:  http://www.jfree.org/jfreechart/index.html
  *
@@ -27,15 +27,13 @@
  * --------------------
  * FastScatterPlot.java
  * --------------------
- * (C) Copyright 2002-2005, by Object Refinery Limited.
+ * (C) Copyright 2002-2008, by Object Refinery Limited.
  *
  * Original Author:  David Gilbert (for Object Refinery Limited);
  * Contributor(s):   Arnaud Lelievre;
  *
- * $Id: FastScatterPlot.java,v 1.11.2.3 2005/10/25 20:52:08 mungady Exp $
- *
- * Changes (from 29-Oct-2002)
- * --------------------------
+ * Changes
+ * -------
  * 29-Oct-2002 : Added standard header (DG);
  * 07-Nov-2002 : Fixed errors reported by Checkstyle (DG);
  * 26-Mar-2003 : Implemented Serializable (DG);
@@ -53,6 +51,11 @@
  * 12-Nov-2004 : Implemented the new Zoomable interface (DG);
  * 05-May-2005 : Updated draw() method parameters (DG);
  * 16-Jun-2005 : Added get/setData() methods (DG);
+ * ------------- JFREECHART 1.0.x ---------------------------------------------
+ * 10-Nov-2006 : Fixed bug 1593150, by not allowing null axes, and added
+ *               setDomainAxis() and setRangeAxis() methods (DG);
+ * 24-Sep-2007 : Implemented new zooming methods (DG);
+ * 25-Mar-2008 : Make use of new fireChangeEvent() method (DG);
  *
  */
 
@@ -79,6 +82,7 @@ import java.util.ResourceBundle;
 
 import org.jfree.chart.axis.AxisSpace;
 import org.jfree.chart.axis.AxisState;
+import org.jfree.chart.axis.NumberAxis;
 import org.jfree.chart.axis.ValueAxis;
 import org.jfree.chart.axis.ValueTick;
 import org.jfree.chart.event.PlotChangeEvent;
@@ -94,19 +98,15 @@ import org.jfree.util.PaintUtilities;
  * A fast scatter plot.
  */
 public class FastScatterPlot extends Plot implements ValueAxisPlot, 
-                                                     Zoomable, 
-                                                     Cloneable, Serializable {
+        Zoomable, Cloneable, Serializable {
 
     /** For serialization. */
     private static final long serialVersionUID = 7871545897358563521L;
     
     /** The default grid line stroke. */
     public static final Stroke DEFAULT_GRIDLINE_STROKE = new BasicStroke(0.5f,
-            BasicStroke.CAP_BUTT,
-            BasicStroke.JOIN_BEVEL,
-            0.0f,
-            new float[] {2.0f, 2.0f},
-            0.0f);
+            BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL, 0.0f, new float[] 
+            {2.0f, 2.0f}, 0.0f);
 
     /** The default grid line paint. */
     public static final Paint DEFAULT_GRIDLINE_PAINT = Color.lightGray;
@@ -149,43 +149,46 @@ public class FastScatterPlot extends Plot implements ValueAxisPlot,
 
     /** The resourceBundle for the localization. */
     protected static ResourceBundle localizationResources = 
-        ResourceBundle.getBundle("org.jfree.chart.plot.LocalizationBundle");
+            ResourceBundle.getBundle(
+            "org.jfree.chart.plot.LocalizationBundle");
 
     /**
-     * Creates an empty plot.
+     * Creates a new instance of <code>FastScatterPlot</code> with default 
+     * axes.
      */
     public FastScatterPlot() {
-        this(null, null, null);    
+        this(null, new NumberAxis("X"), new NumberAxis("Y"));    
     }
     
     /**
      * Creates a new fast scatter plot.
-     * <P>
+     * <p>
      * The data is an array of x, y values:  data[0][i] = x, data[1][i] = y.
-     *
-     * @param data  the data.
-     * @param domainAxis  the domain (x) axis.
-     * @param rangeAxis  the range (y) axis.
+     * 
+     * @param data  the data (<code>null</code> permitted).
+     * @param domainAxis  the domain (x) axis (<code>null</code> not permitted).
+     * @param rangeAxis  the range (y) axis (<code>null</code> not permitted).
      */
     public FastScatterPlot(float[][] data, 
                            ValueAxis domainAxis, ValueAxis rangeAxis) {
 
         super();
-
+        if (domainAxis == null) {
+            throw new IllegalArgumentException("Null 'domainAxis' argument.");
+        }
+        if (rangeAxis == null) {
+            throw new IllegalArgumentException("Null 'rangeAxis' argument.");
+        }
+        
         this.data = data;
         this.xDataRange = calculateXDataRange(data);
         this.yDataRange = calculateYDataRange(data);
         this.domainAxis = domainAxis;
-        if (domainAxis != null) {
-            domainAxis.setPlot(this);
-            domainAxis.addChangeListener(this);
-        }
-
+        this.domainAxis.setPlot(this);
+        this.domainAxis.addChangeListener(this);
         this.rangeAxis = rangeAxis;
-        if (rangeAxis != null) {
-            rangeAxis.setPlot(this);
-            rangeAxis.addChangeListener(this);
-        }
+        this.rangeAxis.setPlot(this);
+        this.rangeAxis.addChangeListener(this);
 
         this.paint = Color.red;
         
@@ -212,6 +215,8 @@ public class FastScatterPlot extends Plot implements ValueAxisPlot,
      * Returns the data array used by the plot.
      * 
      * @return The data array (possibly <code>null</code>).
+     * 
+     * @see #setData(float[][])
      */
     public float[][] getData() {
         return this.data;   
@@ -222,10 +227,12 @@ public class FastScatterPlot extends Plot implements ValueAxisPlot,
      * to all registered listeners.
      * 
      * @param data  the data array (<code>null</code> permitted).
+     * 
+     * @see #getData()
      */
     public void setData(float[][] data) {
         this.data = data;
-        notifyListeners(new PlotChangeEvent(this));
+        fireChangeEvent();
     }
     
     /**
@@ -238,31 +245,70 @@ public class FastScatterPlot extends Plot implements ValueAxisPlot,
     }
     
     /**
-     * Returns the domain axis for the plot.  If the domain axis for this plot
-     * is null, then the method will return the parent plot's domain axis (if
-     * there is a parent plot).
+     * Returns the domain axis for the plot.
      *
-     * @return The domain axis.
+     * @return The domain axis (never <code>null</code>).
+     * 
+     * @see #setDomainAxis(ValueAxis)
      */
     public ValueAxis getDomainAxis() {
         return this.domainAxis;
     }
+    
+    /**
+     * Sets the domain axis and sends a {@link PlotChangeEvent} to all 
+     * registered listeners.
+     * 
+     * @param axis  the axis (<code>null</code> not permitted).
+     * 
+     * @since 1.0.3
+     * 
+     * @see #getDomainAxis()
+     */
+    public void setDomainAxis(ValueAxis axis) {
+        if (axis == null) {
+            throw new IllegalArgumentException("Null 'axis' argument.");
+        }
+        this.domainAxis = axis;
+        fireChangeEvent();
+    }
 
     /**
-     * Returns the range axis for the plot.  If the range axis for this plot is
-     * null, then the method will return the parent plot's range axis (if
-     * there is a parent plot).
+     * Returns the range axis for the plot.
      *
-     * @return The range axis.
+     * @return The range axis (never <code>null</code>).
+     * 
+     * @see #setRangeAxis(ValueAxis)
      */
     public ValueAxis getRangeAxis() {
         return this.rangeAxis;
     }
 
     /**
-     * Returns the paint used to plot data points.
+     * Sets the range axis and sends a {@link PlotChangeEvent} to all 
+     * registered listeners.
+     * 
+     * @param axis  the axis (<code>null</code> not permitted).
+     * 
+     * @since 1.0.3
+     * 
+     * @see #getRangeAxis()
+     */
+    public void setRangeAxis(ValueAxis axis) {
+        if (axis == null) {
+            throw new IllegalArgumentException("Null 'axis' argument.");
+        }
+        this.rangeAxis = axis;
+        fireChangeEvent();
+    }
+
+    /**
+     * Returns the paint used to plot data points.  The default is 
+     * <code>Color.red</code>.
      *
      * @return The paint.
+     * 
+     * @see #setPaint(Paint)
      */
     public Paint getPaint() {
         return this.paint;
@@ -273,13 +319,15 @@ public class FastScatterPlot extends Plot implements ValueAxisPlot,
      * to all registered listeners.
      *
      * @param paint  the paint (<code>null</code> not permitted).
+     * 
+     * @see #getPaint()
      */
     public void setPaint(Paint paint) {
         if (paint == null) {
             throw new IllegalArgumentException("Null 'paint' argument.");
         }
         this.paint = paint;
-        notifyListeners(new PlotChangeEvent(this));
+        fireChangeEvent();
     }
 
     /**
@@ -287,6 +335,9 @@ public class FastScatterPlot extends Plot implements ValueAxisPlot,
      * <code>false<code> otherwise.
      *
      * @return <code>true</code> or <code>false</code>.
+     * 
+     * @see #setDomainGridlinesVisible(boolean)
+     * @see #setDomainGridlinePaint(Paint)
      */
     public boolean isDomainGridlinesVisible() {
         return this.domainGridlinesVisible;
@@ -298,11 +349,13 @@ public class FastScatterPlot extends Plot implements ValueAxisPlot,
      * sent to all registered listeners.
      *
      * @param visible  the new value of the flag.
+     * 
+     * @see #getDomainGridlinePaint()
      */
     public void setDomainGridlinesVisible(boolean visible) {
         if (this.domainGridlinesVisible != visible) {
             this.domainGridlinesVisible = visible;
-            notifyListeners(new PlotChangeEvent(this));
+            fireChangeEvent();
         }
     }
 
@@ -310,44 +363,56 @@ public class FastScatterPlot extends Plot implements ValueAxisPlot,
      * Returns the stroke for the grid-lines (if any) plotted against the 
      * domain axis.
      *
-     * @return The stroke.
+     * @return The stroke (never <code>null</code>).
+     * 
+     * @see #setDomainGridlineStroke(Stroke)
      */
     public Stroke getDomainGridlineStroke() {
         return this.domainGridlineStroke;
     }
 
     /**
-     * Sets the stroke for the grid lines plotted against the domain axis.
-     * <p>
-     * If you set this to <code>null</code>, no grid lines will be drawn.
+     * Sets the stroke for the grid lines plotted against the domain axis and
+     * sends a {@link PlotChangeEvent} to all registered listeners.
      *
-     * @param stroke  the stroke (<code>null</code> permitted).
+     * @param stroke  the stroke (<code>null</code> not permitted).
+     * 
+     * @see #getDomainGridlineStroke()
      */
     public void setDomainGridlineStroke(Stroke stroke) {
+        if (stroke == null) {
+            throw new IllegalArgumentException("Null 'stroke' argument.");
+        }
         this.domainGridlineStroke = stroke;
-        notifyListeners(new PlotChangeEvent(this));
+        fireChangeEvent();
     }
 
     /**
      * Returns the paint for the grid lines (if any) plotted against the domain
      * axis.
      *
-     * @return The paint.
+     * @return The paint (never <code>null</code>).
+     * 
+     * @see #setDomainGridlinePaint(Paint)
      */
     public Paint getDomainGridlinePaint() {
         return this.domainGridlinePaint;
     }
 
     /**
-     * Sets the paint for the grid lines plotted against the domain axis.
-     * <p>
-     * If you set this to <code>null</code>, no grid lines will be drawn.
+     * Sets the paint for the grid lines plotted against the domain axis and
+     * sends a {@link PlotChangeEvent} to all registered listeners.
      *
-     * @param paint  the paint (<code>null</code> permitted).
+     * @param paint  the paint (<code>null</code> not permitted).
+     * 
+     * @see #getDomainGridlinePaint()
      */
     public void setDomainGridlinePaint(Paint paint) {
+        if (paint == null) {
+            throw new IllegalArgumentException("Null 'paint' argument.");
+        }
         this.domainGridlinePaint = paint;
-        notifyListeners(new PlotChangeEvent(this));
+        fireChangeEvent();
     }
 
     /**
@@ -355,6 +420,8 @@ public class FastScatterPlot extends Plot implements ValueAxisPlot,
      * <code>false<code> otherwise.
      *
      * @return <code>true</code> or <code>false</code>.
+     * 
+     * @see #setRangeGridlinesVisible(boolean)
      */
     public boolean isRangeGridlinesVisible() {
         return this.rangeGridlinesVisible;
@@ -366,11 +433,13 @@ public class FastScatterPlot extends Plot implements ValueAxisPlot,
      * sent to all registered listeners.
      *
      * @param visible  the new value of the flag.
+     * 
+     * @see #isRangeGridlinesVisible()
      */
     public void setRangeGridlinesVisible(boolean visible) {
         if (this.rangeGridlinesVisible != visible) {
             this.rangeGridlinesVisible = visible;
-            notifyListeners(new PlotChangeEvent(this));
+            fireChangeEvent();
         }
     }
 
@@ -378,44 +447,56 @@ public class FastScatterPlot extends Plot implements ValueAxisPlot,
      * Returns the stroke for the grid lines (if any) plotted against the range
      * axis.
      *
-     * @return The stroke.
+     * @return The stroke (never <code>null</code>).
+     * 
+     * @see #setRangeGridlineStroke(Stroke)
      */
     public Stroke getRangeGridlineStroke() {
         return this.rangeGridlineStroke;
     }
 
     /**
-     * Sets the stroke for the grid lines plotted against the range axis.
-     * <p>
-     * If you set this to <code>null</code>, no grid lines will be drawn.
+     * Sets the stroke for the grid lines plotted against the range axis and 
+     * sends a {@link PlotChangeEvent} to all registered listeners.
      *
      * @param stroke  the stroke (<code>null</code> permitted).
+     * 
+     * @see #getRangeGridlineStroke()
      */
     public void setRangeGridlineStroke(Stroke stroke) {
+        if (stroke == null) {
+            throw new IllegalArgumentException("Null 'stroke' argument.");
+        }
         this.rangeGridlineStroke = stroke;
-        notifyListeners(new PlotChangeEvent(this));
+        fireChangeEvent();
     }
 
     /**
      * Returns the paint for the grid lines (if any) plotted against the range 
      * axis.
      *
-     * @return The paint.
+     * @return The paint (never <code>null</code>).
+     * 
+     * @see #setRangeGridlinePaint(Paint)
      */
     public Paint getRangeGridlinePaint() {
         return this.rangeGridlinePaint;
     }
 
     /**
-     * Sets the paint for the grid lines plotted against the range axis.
-     * <p>
-     * If you set this to <code>null</code>, no grid lines will be drawn.
+     * Sets the paint for the grid lines plotted against the range axis and 
+     * sends a {@link PlotChangeEvent} to all registered listeners.
      *
-     * @param paint  the paint (<code>null</code> permitted).
+     * @param paint  the paint (<code>null</code> not permitted).
+     * 
+     * @see #getRangeGridlinePaint()
      */
     public void setRangeGridlinePaint(Paint paint) {
+        if (paint == null) {
+            throw new IllegalArgumentException("Null 'paint' argument.");
+        }
         this.rangeGridlinePaint = paint;
-        notifyListeners(new PlotChangeEvent(this));
+        fireChangeEvent();
     }
 
     /**
@@ -426,7 +507,7 @@ public class FastScatterPlot extends Plot implements ValueAxisPlot,
      * @param area   the area within which the plot (including axis labels)
      *                   should be drawn.
      * @param anchor  the anchor point (<code>null</code> permitted).
-     * @param parentState  the state from the parent plot, if there is one.
+     * @param parentState  the state from the parent plot (ignored).
      * @param info  collects chart drawing information (<code>null</code> 
      *              permitted).
      */
@@ -444,12 +525,10 @@ public class FastScatterPlot extends Plot implements ValueAxisPlot,
         insets.trim(area);
 
         AxisSpace space = new AxisSpace();
-        space = this.domainAxis.reserveSpace(
-            g2, this, area, RectangleEdge.BOTTOM, space
-        );
-        space = this.rangeAxis.reserveSpace(
-            g2, this, area, RectangleEdge.LEFT, space
-        );
+        space = this.domainAxis.reserveSpace(g2, this, area, 
+                RectangleEdge.BOTTOM, space);
+        space = this.rangeAxis.reserveSpace(g2, this, area, RectangleEdge.LEFT, 
+                space);
         Rectangle2D dataArea = space.shrink(area, null);
 
         if (info != null) {
@@ -459,20 +538,10 @@ public class FastScatterPlot extends Plot implements ValueAxisPlot,
         // draw the plot background and axes...
         drawBackground(g2, dataArea);
 
-        AxisState domainAxisState = null;
-        AxisState rangeAxisState = null;
-        if (this.domainAxis != null) {
-            domainAxisState = this.domainAxis.draw(
-                g2, dataArea.getMaxY(), area, dataArea, 
-                RectangleEdge.BOTTOM, info
-            );
-        }
-        if (this.rangeAxis != null) {
-            rangeAxisState = this.rangeAxis.draw(
-                g2, dataArea.getMinX(), area, dataArea, 
-                RectangleEdge.LEFT, info
-            );
-        }
+        AxisState domainAxisState = this.domainAxis.draw(g2, 
+                dataArea.getMaxY(), area, dataArea, RectangleEdge.BOTTOM, info);
+        AxisState rangeAxisState = this.rangeAxis.draw(g2, dataArea.getMinX(), 
+                area, dataArea, RectangleEdge.LEFT, info);
         drawDomainGridlines(g2, dataArea, domainAxisState.getTicks());
         drawRangeGridlines(g2, dataArea, rangeAxisState.getTicks());
         
@@ -480,11 +549,8 @@ public class FastScatterPlot extends Plot implements ValueAxisPlot,
         Composite originalComposite = g2.getComposite();
 
         g2.clip(dataArea);
-        g2.setComposite(
-            AlphaComposite.getInstance(
-                AlphaComposite.SRC_OVER, getForegroundAlpha()
-            )
-        );
+        g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 
+                getForegroundAlpha()));
 
         render(g2, dataArea, info, null);
 
@@ -533,12 +599,10 @@ public class FastScatterPlot extends Plot implements ValueAxisPlot,
                 
                 //int transX = (int) (xx + ww * (x - domainMin) / domainLength);
                 //int transY = (int) (yy - hh * (y - rangeMin) / rangeLength); 
-                int transX = (int) this.domainAxis.valueToJava2D(
-                    x, dataArea, RectangleEdge.BOTTOM
-                );
-                int transY = (int) this.rangeAxis.valueToJava2D(
-                    y, dataArea, RectangleEdge.LEFT
-                );
+                int transX = (int) this.domainAxis.valueToJava2D(x, dataArea, 
+                        RectangleEdge.BOTTOM);
+                int transY = (int) this.rangeAxis.valueToJava2D(y, dataArea, 
+                        RectangleEdge.LEFT);
                 g2.fillRect(transX, transY, 1, 1);
             }
         }
@@ -558,24 +622,18 @@ public class FastScatterPlot extends Plot implements ValueAxisPlot,
     protected void drawDomainGridlines(Graphics2D g2, Rectangle2D dataArea, 
                                        List ticks) {
 
-        // draw the domain grid lines, if any...
+        // draw the domain grid lines, if the flag says they're visible...
         if (isDomainGridlinesVisible()) {
-            Stroke gridStroke = getDomainGridlineStroke();
-            Paint gridPaint = getDomainGridlinePaint();
-            if ((gridStroke != null) && (gridPaint != null)) {
-                Iterator iterator = ticks.iterator();
-                while (iterator.hasNext()) {
-                    ValueTick tick = (ValueTick) iterator.next();
-                    double v = this.domainAxis.valueToJava2D(
-                        tick.getValue(), dataArea, RectangleEdge.BOTTOM
-                    );
-                    Line2D line = new Line2D.Double(
-                        v, dataArea.getMinY(), v, dataArea.getMaxY()
-                    );
-                    g2.setPaint(gridPaint);
-                    g2.setStroke(gridStroke);
-                    g2.draw(line);                
-                }
+            Iterator iterator = ticks.iterator();
+            while (iterator.hasNext()) {
+                ValueTick tick = (ValueTick) iterator.next();
+                double v = this.domainAxis.valueToJava2D(tick.getValue(), 
+                        dataArea, RectangleEdge.BOTTOM);
+                Line2D line = new Line2D.Double(v, dataArea.getMinY(), v, 
+                        dataArea.getMaxY());
+                g2.setPaint(getDomainGridlinePaint());
+                g2.setStroke(getDomainGridlineStroke());
+                g2.draw(line);                
             }
         }
     }
@@ -590,38 +648,33 @@ public class FastScatterPlot extends Plot implements ValueAxisPlot,
     protected void drawRangeGridlines(Graphics2D g2, Rectangle2D dataArea, 
                                       List ticks) {
 
-        // draw the range grid lines, if any...
+        // draw the range grid lines, if the flag says they're visible...
         if (isRangeGridlinesVisible()) {
-            Stroke gridStroke = getRangeGridlineStroke();
-            Paint gridPaint = getRangeGridlinePaint();
-            if ((gridStroke != null) && (gridPaint != null)) {
-                Iterator iterator = ticks.iterator();
-                while (iterator.hasNext()) {
-                    ValueTick tick = (ValueTick) iterator.next();
-                    double v = this.rangeAxis.valueToJava2D(
-                        tick.getValue(), dataArea, RectangleEdge.LEFT
-                    );
-                    Line2D line = new Line2D.Double(
-                        dataArea.getMinX(), v, dataArea.getMaxX(), v
-                    );
-                    g2.setPaint(gridPaint);
-                    g2.setStroke(gridStroke);
-                    g2.draw(line);                
-                }
+            Iterator iterator = ticks.iterator();
+            while (iterator.hasNext()) {
+                ValueTick tick = (ValueTick) iterator.next();
+                double v = this.rangeAxis.valueToJava2D(tick.getValue(), 
+                        dataArea, RectangleEdge.LEFT);
+                Line2D line = new Line2D.Double(dataArea.getMinX(), v, 
+                        dataArea.getMaxX(), v);
+                g2.setPaint(getRangeGridlinePaint());
+                g2.setStroke(getRangeGridlineStroke());
+                g2.draw(line);                
             }
         }
 
     }
 
     /**
-     * Returns the range of data values to be plotted along the axis.
+     * Returns the range of data values to be plotted along the axis, or
+     * <code>null</code> if the specified axis isn't the domain axis or the
+     * range axis for the plot.
      *
-     * @param axis  the axis.
+     * @param axis  the axis (<code>null</code> permitted).
      *
-     * @return The range.
+     * @return The range (possibly <code>null</code>).
      */
     public Range getDataRange(ValueAxis axis) {
-
         Range result = null;
         if (axis == this.domainAxis) {
             result = this.xDataRange;
@@ -635,7 +688,7 @@ public class FastScatterPlot extends Plot implements ValueAxisPlot,
     /**
      * Calculates the X data range.
      *
-     * @param data  the data.
+     * @param data  the data (<code>null</code> permitted).
      *
      * @return The range.
      */
@@ -667,7 +720,7 @@ public class FastScatterPlot extends Plot implements ValueAxisPlot,
     /**
      * Calculates the Y data range.
      *
-     * @param data  the data.
+     * @param data  the data (<code>null</code> permitted).
      *
      * @return The range.
      */
@@ -696,7 +749,7 @@ public class FastScatterPlot extends Plot implements ValueAxisPlot,
     }
 
     /**
-     * Multiplies the range on the domain axis/axes by the specified factor.
+     * Multiplies the range on the domain axis by the specified factor.
      *
      * @param factor  the zoom factor.
      * @param info  the plot rendering info.
@@ -706,6 +759,35 @@ public class FastScatterPlot extends Plot implements ValueAxisPlot,
                                Point2D source) {
         this.domainAxis.resizeRange(factor);
     }
+    
+    /**
+     * Multiplies the range on the domain axis by the specified factor.
+     *
+     * @param factor  the zoom factor.
+     * @param info  the plot rendering info.
+     * @param source  the source point (in Java2D space).
+     * @param useAnchor  use source point as zoom anchor?
+     * 
+     * @see #zoomRangeAxes(double, PlotRenderingInfo, Point2D, boolean)
+     * 
+     * @since 1.0.7
+     */
+    public void zoomDomainAxes(double factor, PlotRenderingInfo info,
+                               Point2D source, boolean useAnchor) {
+                
+        if (useAnchor) {
+            // get the source coordinate - this plot has always a VERTICAL
+            // orientation
+            double sourceX = source.getX();
+            double anchorX = this.domainAxis.java2DToValue(sourceX, 
+                    info.getDataArea(), RectangleEdge.BOTTOM);
+            this.domainAxis.resizeRange(factor, anchorX);
+        }
+        else {
+            this.domainAxis.resizeRange(factor);
+        }
+        
+    }
 
     /**
      * Zooms in on the domain axes.
@@ -734,6 +816,35 @@ public class FastScatterPlot extends Plot implements ValueAxisPlot,
         this.rangeAxis.resizeRange(factor);
     }
 
+    /**
+     * Multiplies the range on the range axis by the specified factor.
+     *
+     * @param factor  the zoom factor.
+     * @param info  the plot rendering info.
+     * @param source  the source point (in Java2D space).
+     * @param useAnchor  use source point as zoom anchor?
+     * 
+     * @see #zoomDomainAxes(double, PlotRenderingInfo, Point2D, boolean)
+     * 
+     * @since 1.0.7
+     */
+    public void zoomRangeAxes(double factor, PlotRenderingInfo info,
+                              Point2D source, boolean useAnchor) {
+                
+        if (useAnchor) {
+            // get the source coordinate - this plot has always a VERTICAL
+            // orientation
+            double sourceX = source.getX();
+            double anchorX = this.rangeAxis.java2DToValue(sourceX, 
+                    info.getDataArea(), RectangleEdge.LEFT);
+            this.rangeAxis.resizeRange(factor, anchorX);
+        }
+        else {
+            this.rangeAxis.resizeRange(factor);
+        }
+        
+    }
+    
     /**
      * Zooms in on the range axes.
      * 
