@@ -2,7 +2,7 @@
  * JFreeChart : a free chart library for the Java(tm) platform
  * ===========================================================
  *
- * (C) Copyright 2000-2005, by Object Refinery Limited and Contributors.
+ * (C) Copyright 2000-2008, by Object Refinery Limited and Contributors.
  *
  * Project Info:  http://www.jfree.org/jfreechart/index.html
  *
@@ -27,7 +27,7 @@
  * --------------
  * PiePlot3D.java
  * --------------
- * (C) Copyright 2000-2005, by Object Refinery and Contributors.
+ * (C) Copyright 2000-2008, by Object Refinery and Contributors.
  *
  * Original Author:  Tomer Peretz;
  * Contributor(s):   Richard Atkinson;
@@ -37,8 +37,6 @@
  *                   Arnaud Lelievre;
  *                   Dave Crane;
  *
- * $Id: PiePlot3D.java,v 1.10.2.2 2005/10/25 20:52:08 mungady Exp $
- *
  * Changes
  * -------
  * 21-Jun-2002 : Version 1;
@@ -70,7 +68,17 @@
  *               Added pieIndex to PieSectionEntity (DG);
  * 15-Nov-2004 : Removed creation of default tool tip generator (DG);
  * 16-Jun-2005 : Added default constructor (DG);
- * 
+ * ------------- JFREECHART 1.0.x ---------------------------------------------
+ * 27-Sep-2006 : Updated draw() method for new lookup methods (DG);
+ * 22-Mar-2007 : Added equals() override (DG);
+ * 18-Jun-2007 : Added handling for simple label option (DG);
+ * 04-Oct-2007 : Added option to darken sides of plot - thanks to Alex Moots 
+ *               (see patch 1805262) (DG);
+ * 21-Nov-2007 : Changed default depth factor, fixed labelling bugs and added
+ *               debug code - see debug flags in PiePlot class (DG);
+ * 20-Mar-2008 : Fixed bug 1920854 - multiple redraws of the section 
+ *               labels (DG);
+ *
  */
 
 package org.jfree.chart.plot;
@@ -97,6 +105,7 @@ import java.util.List;
 
 import org.jfree.chart.entity.EntityCollection;
 import org.jfree.chart.entity.PieSectionEntity;
+import org.jfree.chart.event.PlotChangeEvent;
 import org.jfree.chart.labels.PieToolTipGenerator;
 import org.jfree.data.general.DatasetUtilities;
 import org.jfree.data.general.PieDataset;
@@ -115,8 +124,17 @@ public class PiePlot3D extends PiePlot implements Serializable {
     private static final long serialVersionUID = 3408984188945161432L;
     
     /** The factor of the depth of the pie from the plot height */
-    private double depthFactor = 0.2;
+    private double depthFactor = 0.12;
 
+    /** 
+     * A flag that controls whether or not the sides of the pie chart
+     * are rendered using a darker colour.
+     * 
+     *  @since 1.0.7.
+     */
+    private boolean darkerSides = false;  // default preserves previous 
+                                          // behaviour
+    
     /**
      * Creates a new instance with no dataset.
      */
@@ -136,21 +154,60 @@ public class PiePlot3D extends PiePlot implements Serializable {
     }
 
     /**
-     * Sets the pie depth as a percentage of the height of the plot area.
+     * Returns the depth factor for the chart.
      *
-     * @param factor  the depth factor (for example, 0.20 is twenty percent).
+     * @return The depth factor.
+     * 
+     * @see #setDepthFactor(double)
      */
-    public void setDepthFactor(double factor) {
-        this.depthFactor = factor;
+    public double getDepthFactor() {
+        return this.depthFactor;
     }
 
     /**
-     * The depth factor for the chart.
+     * Sets the pie depth as a percentage of the height of the plot area, and
+     * sends a {@link PlotChangeEvent} to all registered listeners.
      *
-     * @return The depth factor.
+     * @param factor  the depth factor (for example, 0.20 is twenty percent).
+     * 
+     * @see #getDepthFactor()
      */
-    public double getDepthFactor () {
-        return this.depthFactor;
+    public void setDepthFactor(double factor) {
+        this.depthFactor = factor;
+        fireChangeEvent();
+    }
+
+    /**
+     * Returns a flag that controls whether or not the sides of the pie chart
+     * are rendered using a darker colour.  This is only applied if the
+     * section colour is an instance of {@link java.awt.Color}.
+     *
+     * @return A boolean.
+     * 
+     * @see #setDarkerSides(boolean)
+     * 
+     * @since 1.0.7
+     */
+    public boolean getDarkerSides() {
+        return this.darkerSides;
+    }
+
+    /**
+     * Sets a flag that controls whether or not the sides of the pie chart
+     * are rendered using a darker colour, and sends a {@link PlotChangeEvent} 
+     * to all registered listeners.  This is only applied if the
+     * section colour is an instance of {@link java.awt.Color}.
+     *
+     * @param darker true to darken the sides, false to use the default 
+     *         behaviour.
+     * 
+     * @see #getDarkerSides()
+     * 
+     * @since 1.0.7.
+     */
+    public void setDarkerSides(boolean darker) {
+        this.darkerSides = darker;
+        fireChangeEvent();
     }
 
     /**
@@ -180,6 +237,8 @@ public class PiePlot3D extends PiePlot implements Serializable {
             info.setDataArea(plotArea);
         }
 
+        drawBackground(g2, plotArea);
+
         Shape savedClip = g2.getClip();
         g2.clip(plotArea);
 
@@ -187,12 +246,23 @@ public class PiePlot3D extends PiePlot implements Serializable {
         double gapPercent = getInteriorGap();
         double labelPercent = 0.0;
         if (getLabelGenerator() != null) {
-            labelPercent = getLabelGap() + getMaximumLabelWidth() 
-                           + getLabelLinkMargin();   
+            labelPercent = getLabelGap() + getMaximumLabelWidth();   
+        }
+        double gapHorizontal = plotArea.getWidth() * (gapPercent 
+                + labelPercent) * 2.0;
+        double gapVertical = plotArea.getHeight() * gapPercent * 2.0;
+
+        if (DEBUG_DRAW_INTERIOR) {
+            double hGap = plotArea.getWidth() * getInteriorGap();
+            double vGap = plotArea.getHeight() * getInteriorGap();
+            double igx1 = plotArea.getX() + hGap;
+            double igx2 = plotArea.getMaxX() - hGap;
+            double igy1 = plotArea.getY() + vGap;
+            double igy2 = plotArea.getMaxY() - vGap;
+            g2.setPaint(Color.lightGray);
+            g2.draw(new Rectangle2D.Double(igx1, igy1, igx2 - igx1, 
+                    igy2 - igy1));
         }
-        double gapHorizontal = plotArea.getWidth() 
-                               * (gapPercent + labelPercent);
-        double gapVertical = plotArea.getHeight() * gapPercent;
 
         double linkX = plotArea.getX() + gapHorizontal / 2;
         double linkY = plotArea.getY() + gapVertical / 2;
@@ -209,14 +279,28 @@ public class PiePlot3D extends PiePlot implements Serializable {
         }
         
         PiePlotState state = initialise(g2, plotArea, this, null, info);
+
+        // the link area defines the dog leg points for the linking lines to 
+        // the labels
+        Rectangle2D linkAreaXX = new Rectangle2D.Double(linkX, linkY, linkW, 
+                linkH * (1 - this.depthFactor));
+        state.setLinkArea(linkAreaXX);
+
+        if (DEBUG_DRAW_LINK_AREA) {
+            g2.setPaint(Color.blue);
+            g2.draw(linkAreaXX);
+            g2.setPaint(Color.yellow);
+            g2.draw(new Ellipse2D.Double(linkAreaXX.getX(), linkAreaXX.getY(), 
+                    linkAreaXX.getWidth(), linkAreaXX.getHeight()));
+        }
+        
         // the explode area defines the max circle/ellipse for the exploded pie 
         // sections.
         // it is defined by shrinking the linkArea by the linkMargin factor.
         double hh = linkW * getLabelLinkMargin();
         double vv = linkH * getLabelLinkMargin();
-        Rectangle2D explodeArea = new Rectangle2D.Double(
-            linkX + hh / 2.0, linkY + vv / 2.0, linkW - hh, linkH - vv
-        );
+        Rectangle2D explodeArea = new Rectangle2D.Double(linkX + hh / 2.0, 
+                linkY + vv / 2.0, linkW - hh, linkH - vv);
        
         state.setExplodedPieArea(explodeArea);
         
@@ -228,17 +312,15 @@ public class PiePlot3D extends PiePlot implements Serializable {
         
         double h1 = explodeArea.getWidth() * percent;
         double v1 = explodeArea.getHeight() * percent;
-        Rectangle2D pieArea = new Rectangle2D.Double(
-            explodeArea.getX() + h1 / 2.0, explodeArea.getY() + v1 / 2.0,
-            explodeArea.getWidth() - h1, explodeArea.getHeight() - v1
-        );
+        Rectangle2D pieArea = new Rectangle2D.Double(explodeArea.getX() 
+                + h1 / 2.0, explodeArea.getY() + v1 / 2.0,
+                explodeArea.getWidth() - h1, explodeArea.getHeight() - v1);
 
-        int depth = (int) (pieArea.getHeight() * this.depthFactor);
         // the link area defines the dog-leg point for the linking lines to 
         // the labels
-        Rectangle2D linkArea = new Rectangle2D.Double(
-            linkX, linkY, linkW, linkH - depth
-        );
+        int depth = (int) (pieArea.getHeight() * this.depthFactor);
+        Rectangle2D linkArea = new Rectangle2D.Double(linkX, linkY, linkW, 
+                linkH - depth);
         state.setLinkArea(linkArea);   
 
         state.setPieArea(pieArea);
@@ -247,7 +329,6 @@ public class PiePlot3D extends PiePlot implements Serializable {
         state.setPieWRadius(pieArea.getWidth() / 2.0);
         state.setPieHRadius((pieArea.getHeight() - depth) / 2.0);
 
-        drawBackground(g2, plotArea);
         // get the data source - return if null;
         PieDataset dataset = getDataset();
         if (DatasetUtilities.isEmptyOrNull(getDataset())) {
@@ -265,12 +346,9 @@ public class PiePlot3D extends PiePlot implements Serializable {
             FontMetrics fm = g2.getFontMetrics(sfont);
             int stringWidth = fm.stringWidth(text);
 
-            g2.drawString(
-                text, 
-                (int) (plotArea.getX() + (plotArea.getWidth() - stringWidth) 
-                        / 2),
-                (int) (plotArea.getY() + (plotArea.getHeight() / 2))
-            );
+            g2.drawString(text, (int) (plotArea.getX() + (plotArea.getWidth() 
+                    - stringWidth) / 2), (int) (plotArea.getY() 
+                    + (plotArea.getHeight() / 2)));
             return;
         }
         // if we are drawing a perfect circle, we need to readjust the top left
@@ -279,10 +357,8 @@ public class PiePlot3D extends PiePlot implements Serializable {
         if (isCircular()) {
             double min = Math.min(plotArea.getWidth(), 
                     plotArea.getHeight()) / 2;
-            plotArea = new Rectangle2D.Double(
-                plotArea.getCenterX() - min, plotArea.getCenterY() - min, 
-                2 * min, 2 * min
-            );
+            plotArea = new Rectangle2D.Double(plotArea.getCenterX() - min, 
+                    plotArea.getCenterY() - min, 2 * min, 2 * min);
         }
         // get a list of keys...
         List sectionKeys = dataset.getKeys();
@@ -329,17 +405,13 @@ public class PiePlot3D extends PiePlot implements Serializable {
             double startAngle = getStartAngle();
             double direction = getDirection().getFactor();
             double angle1 = startAngle + (direction * (runningTotal * 360)) 
-            / totalValue;
+                    / totalValue;
             double angle2 = startAngle + (direction * (runningTotal + value) 
                     * 360) / totalValue;
             if (Math.abs(angle2 - angle1) > getMinimumArcAngleToDraw()) {
-                arcList.add(
-                    new Arc2D.Double(
-                        arcX, arcY + depth, pieArea.getWidth(), 
-                        pieArea.getHeight() - depth,
-                        angle1, angle2 - angle1, Arc2D.PIE
-                    )
-                );
+                arcList.add(new Arc2D.Double(arcX, arcY + depth, 
+                        pieArea.getWidth(), pieArea.getHeight() - depth,
+                        angle1, angle2 - angle1, Arc2D.PIE));
             }
             else {
                 arcList.add(null);
@@ -349,25 +421,18 @@ public class PiePlot3D extends PiePlot implements Serializable {
 
         Shape oldClip = g2.getClip();
 
-        Ellipse2D top = new Ellipse2D.Double(
-            pieArea.getX(), pieArea.getY(), pieArea.getWidth(), 
-            pieArea.getHeight() - depth
-        );
+        Ellipse2D top = new Ellipse2D.Double(pieArea.getX(), pieArea.getY(), 
+                pieArea.getWidth(), pieArea.getHeight() - depth);
 
-        Ellipse2D bottom = new Ellipse2D.Double(
-            pieArea.getX(), pieArea.getY() + depth, pieArea.getWidth(), 
-            pieArea.getHeight() - depth
-        );
+        Ellipse2D bottom = new Ellipse2D.Double(pieArea.getX(), pieArea.getY() 
+                + depth, pieArea.getWidth(), pieArea.getHeight() - depth);
 
-        Rectangle2D lower = new Rectangle2D.Double(
-            top.getX(), top.getCenterY(), pieArea.getWidth(), 
-            bottom.getMaxY() - top.getCenterY()
-        );
+        Rectangle2D lower = new Rectangle2D.Double(top.getX(), 
+                top.getCenterY(), pieArea.getWidth(), bottom.getMaxY() 
+                - top.getCenterY());
 
-        Rectangle2D upper = new Rectangle2D.Double(
-            pieArea.getX(), top.getY(), pieArea.getWidth(),
-            bottom.getCenterY() - top.getY()
-        );
+        Rectangle2D upper = new Rectangle2D.Double(pieArea.getX(), top.getY(), 
+                pieArea.getWidth(), bottom.getCenterY() - top.getY());
 
         Area a = new Area(top);
         a.add(new Area(lower));
@@ -385,22 +450,20 @@ public class PiePlot3D extends PiePlot implements Serializable {
         // draw the bottom circle
         int[] xs;
         int[] ys;
-        outlinePaint = getSectionOutlinePaint(0);
-        arc = new Arc2D.Double(
-            arcX, arcY + depth, pieArea.getWidth(), pieArea.getHeight() - depth,
-            0, 360, Arc2D.PIE
-        );
+        arc = new Arc2D.Double(arcX, arcY + depth, pieArea.getWidth(), 
+                pieArea.getHeight() - depth, 0, 360, Arc2D.PIE);
 
         int categoryCount = arcList.size();
         for (int categoryIndex = 0; categoryIndex < categoryCount; 
-             categoryIndex++) {
+                 categoryIndex++) {
             arc = (Arc2D.Double) arcList.get(categoryIndex);
             if (arc == null) {
                 continue;
             }
-            paint = getSectionPaint(categoryIndex);
-            outlinePaint = getSectionOutlinePaint(categoryIndex);
-            outlineStroke = getSectionOutlineStroke(categoryIndex);
+            Comparable key = getSectionKey(categoryIndex);
+            paint = lookupSectionPaint(key, true);
+            outlinePaint = lookupSectionOutlinePaint(key);
+            outlineStroke = lookupSectionOutlineStroke(key);
             g2.setPaint(paint);
             g2.fill(arc);
             g2.setPaint(outlinePaint);
@@ -411,14 +474,10 @@ public class PiePlot3D extends PiePlot implements Serializable {
             Point2D p1 = arc.getStartPoint();
 
             // draw the height
-            xs = new int[] {
-                (int) arc.getCenterX(), (int) arc.getCenterX(),
-                (int) p1.getX(), (int) p1.getX() 
-            };
-            ys = new int[] {
-                (int) arc.getCenterY(), (int) arc.getCenterY() - depth,
-                (int) p1.getY() - depth, (int) p1.getY() 
-            };
+            xs = new int[] {(int) arc.getCenterX(), (int) arc.getCenterX(),
+                    (int) p1.getX(), (int) p1.getX()};
+            ys = new int[] {(int) arc.getCenterY(), (int) arc.getCenterY() 
+                    - depth, (int) p1.getY() - depth, (int) p1.getY()};
             Polygon polygon = new Polygon(xs, ys, 4);
             g2.setPaint(java.awt.Color.lightGray);
             g2.fill(polygon);
@@ -439,14 +498,12 @@ public class PiePlot3D extends PiePlot implements Serializable {
         while (iterator.hasNext()) {
             Arc2D segment = (Arc2D) iterator.next();
             if (segment != null) {
-                paint = getSectionPaint(cat);
-                outlinePaint = getSectionOutlinePaint(cat);
-                outlineStroke = getSectionOutlineStroke(cat);
-                drawSide(
-                    g2, pieArea, segment, front, back, paint, 
-                    outlinePaint, outlineStroke, 
-                    false, true
-                );
+                Comparable key = getSectionKey(cat);
+                paint = lookupSectionPaint(key, true);
+                outlinePaint = lookupSectionOutlinePaint(key);
+                outlineStroke = lookupSectionOutlineStroke(key);
+                drawSide(g2, pieArea, segment, front, back, paint, 
+                        outlinePaint, outlineStroke, false, true);
             }
             cat++;
         }
@@ -457,14 +514,12 @@ public class PiePlot3D extends PiePlot implements Serializable {
         while (iterator.hasNext()) {
             Arc2D segment = (Arc2D) iterator.next();
             if (segment != null) {
-                paint = getSectionPaint(cat);
-                outlinePaint = getSectionOutlinePaint(cat);
-                outlineStroke = getSectionOutlineStroke(cat);
-                drawSide(
-                    g2, pieArea, segment, front, back, paint, 
-                    outlinePaint, outlineStroke, 
-                    true, false
-                );
+                Comparable key = getSectionKey(cat);
+                paint = lookupSectionPaint(key);
+                outlinePaint = lookupSectionOutlinePaint(key);
+                outlineStroke = lookupSectionOutlineStroke(key);
+                drawSide(g2, pieArea, segment, front, back, paint, 
+                        outlinePaint, outlineStroke, true, false);
             }
             cat++;
         }
@@ -474,19 +529,19 @@ public class PiePlot3D extends PiePlot implements Serializable {
         // draw the sections at the top of the pie (and set up tooltips)...
         Arc2D upperArc;
         for (int sectionIndex = 0; sectionIndex < categoryCount; 
-             sectionIndex++) {
+                 sectionIndex++) {
             arc = (Arc2D.Double) arcList.get(sectionIndex);
             if (arc == null) {
                 continue;
             }
-            upperArc = new Arc2D.Double(
-                arcX, arcY, pieArea.getWidth(), pieArea.getHeight() - depth,
-                arc.getAngleStart(), arc.getAngleExtent(), Arc2D.PIE
-            );
+            upperArc = new Arc2D.Double(arcX, arcY, pieArea.getWidth(),
+                    pieArea.getHeight() - depth, arc.getAngleStart(), 
+                    arc.getAngleExtent(), Arc2D.PIE);
             
-            paint = getSectionPaint(sectionIndex);
-            outlinePaint = getSectionOutlinePaint(sectionIndex);
-            outlineStroke = getSectionOutlineStroke(sectionIndex);
+            Comparable currentKey = (Comparable) sectionKeys.get(sectionIndex);
+            paint = lookupSectionPaint(currentKey, true);
+            outlinePaint = lookupSectionOutlinePaint(currentKey);
+            outlineStroke = lookupSectionOutlineStroke(currentKey);
             g2.setPaint(paint);
             g2.fill(upperArc);
             g2.setStroke(outlineStroke);
@@ -494,10 +549,9 @@ public class PiePlot3D extends PiePlot implements Serializable {
             g2.draw(upperArc);
 
            // add a tooltip for the section...
-            Comparable currentKey = (Comparable) sectionKeys.get(sectionIndex);
             if (info != null) {
                 EntityCollection entities 
-                    = info.getOwner().getEntityCollection();
+                        = info.getOwner().getEntityCollection();
                 if (entities != null) {
                     String tip = null;
                     PieToolTipGenerator tipster = getToolTipGenerator();
@@ -511,19 +565,25 @@ public class PiePlot3D extends PiePlot implements Serializable {
                                 getPieIndex());
                     }
                     PieSectionEntity entity = new PieSectionEntity(
-                        upperArc, dataset, getPieIndex(), sectionIndex, 
-                        currentKey, tip, url
-                    );
+                            upperArc, dataset, getPieIndex(), sectionIndex, 
+                            currentKey, tip, url);
                     entities.add(entity);
                 }
             }
-            List keys = dataset.getKeys();
-            Rectangle2D adjustedPlotArea = new Rectangle2D.Double(
+        }
+
+        List keys = dataset.getKeys();
+        Rectangle2D adjustedPlotArea = new Rectangle2D.Double(
                 originalPlotArea.getX(), originalPlotArea.getY(), 
-                originalPlotArea.getWidth(), 
-                originalPlotArea.getHeight() - depth
-            );
-            drawLabels(g2, keys, totalValue, adjustedPlotArea, linkArea, state);
+                originalPlotArea.getWidth(), originalPlotArea.getHeight() 
+                - depth);
+        if (getSimpleLabels()) {
+            drawSimpleLabels(g2, keys, totalValue, adjustedPlotArea, 
+                    linkArea, state);
+        }
+        else {
+            drawLabels(g2, keys, totalValue, adjustedPlotArea, linkArea, 
+                    state);
         }
 
         g2.setClip(savedClip);
@@ -557,6 +617,14 @@ public class PiePlot3D extends PiePlot implements Serializable {
                             boolean drawFront, 
                             boolean drawBack) {
 
+        if (getDarkerSides()) {
+            if (paint instanceof Color) {
+                Color c = (Color) paint;
+                c = c.darker();
+                paint = c;
+            }
+        }
+
         double start = arc.getAngleStart();
         double extent = arc.getAngleExtent();
         double end = start + extent;
@@ -573,14 +641,11 @@ public class PiePlot3D extends PiePlot implements Serializable {
                     if (extent > -180.0) {  // the segment is entirely at the 
                                             // front of the chart
                         if (drawFront) {
-                            Area side = new Area(
-                                new Rectangle2D.Double(
-                                    arc.getEndPoint().getX(), plotArea.getY(),
+                            Area side = new Area(new Rectangle2D.Double(
+                                    arc.getEndPoint().getX(), plotArea.getY(), 
                                     arc.getStartPoint().getX() 
                                     - arc.getEndPoint().getX(),
-                                    plotArea.getHeight()
-                                )
-                            );
+                                    plotArea.getHeight()));
                             side.intersect(front);
                             g2.setPaint(paint);
                             g2.fill(side);
@@ -591,22 +656,16 @@ public class PiePlot3D extends PiePlot implements Serializable {
                     else {  // the segment starts at the front, and wraps all 
                             // the way around
                             // the back and finishes at the front again
-                        Area side1 = new Area(
-                            new Rectangle2D.Double(
+                        Area side1 = new Area(new Rectangle2D.Double(
                                 plotArea.getX(), plotArea.getY(),
                                 arc.getStartPoint().getX() - plotArea.getX(), 
-                                plotArea.getHeight()
-                            )
-                        );
+                                plotArea.getHeight()));
                         side1.intersect(front);
 
-                        Area side2 = new Area(
-                            new Rectangle2D.Double(
+                        Area side2 = new Area(new Rectangle2D.Double(
                                 arc.getEndPoint().getX(), plotArea.getY(),
                                 plotArea.getMaxX() - arc.getEndPoint().getX(),
-                                plotArea.getHeight()
-                            )
-                        );
+                                plotArea.getHeight()));
 
                         side2.intersect(front);
                         g2.setPaint(paint);
@@ -635,13 +694,10 @@ public class PiePlot3D extends PiePlot implements Serializable {
                         // around the left side)
 
                     if (drawBack) {
-                        Area side2 = new Area(
-                            new Rectangle2D.Double(
+                        Area side2 = new Area(new Rectangle2D.Double(
                                 plotArea.getX(), plotArea.getY(),
                                 arc.getEndPoint().getX() - plotArea.getX(), 
-                                plotArea.getHeight()
-                            )
-                        );
+                                plotArea.getHeight()));
                         side2.intersect(back);
                         g2.setPaint(paint);
                         g2.fill(side2);
@@ -650,13 +706,10 @@ public class PiePlot3D extends PiePlot implements Serializable {
                     }
 
                     if (drawFront) {
-                        Area side1 = new Area(
-                            new Rectangle2D.Double(
+                        Area side1 = new Area(new Rectangle2D.Double(
                                 plotArea.getX(), plotArea.getY(),
                                 arc.getStartPoint().getX() - plotArea.getX(),
-                                plotArea.getHeight()
-                            )
-                        );
+                                plotArea.getHeight()));
                         side1.intersect(front);
                         g2.setPaint(paint);
                         g2.fill(side1);
@@ -671,14 +724,11 @@ public class PiePlot3D extends PiePlot implements Serializable {
                 if (!isAngleAtFront(end)) {
                     if (extent > -180.0) {  // whole segment stays at the back
                         if (drawBack) {
-                            Area side = new Area(
-                                new Rectangle2D.Double(
+                            Area side = new Area(new Rectangle2D.Double(
                                     arc.getStartPoint().getX(), plotArea.getY(),
                                     arc.getEndPoint().getX() 
                                     - arc.getStartPoint().getX(),
-                                    plotArea.getHeight()
-                                )
-                            );
+                                    plotArea.getHeight()));
                             side.intersect(back);
                             g2.setPaint(paint);
                             g2.fill(side);
@@ -688,22 +738,16 @@ public class PiePlot3D extends PiePlot implements Serializable {
                     }
                     else {  // starts at the back, wraps around front, and 
                             // finishes at back again
-                        Area side1 = new Area(
-                            new Rectangle2D.Double(
+                        Area side1 = new Area(new Rectangle2D.Double(
                                 arc.getStartPoint().getX(), plotArea.getY(),
                                 plotArea.getMaxX() - arc.getStartPoint().getX(),
-                                plotArea.getHeight()
-                            )
-                        );
+                                plotArea.getHeight()));
                         side1.intersect(back);
 
-                        Area side2 = new Area(
-                            new Rectangle2D.Double(
+                        Area side2 = new Area(new Rectangle2D.Double(
                                 plotArea.getX(), plotArea.getY(),
                                 arc.getEndPoint().getX() - plotArea.getX(),
-                                plotArea.getHeight()
-                            )
-                        );
+                                plotArea.getHeight()));
 
                         side2.intersect(back);
 
@@ -732,13 +776,10 @@ public class PiePlot3D extends PiePlot implements Serializable {
                 else {  // starts at back, finishes at front (CLOCKWISE)
 
                     if (drawBack) {
-                        Area side1 = new Area(
-                            new Rectangle2D.Double(
+                        Area side1 = new Area(new Rectangle2D.Double(
                                 arc.getStartPoint().getX(), plotArea.getY(),
                                 plotArea.getMaxX() - arc.getStartPoint().getX(),
-                                plotArea.getHeight()
-                            )
-                        );
+                                plotArea.getHeight()));
                         side1.intersect(back);
                         g2.setPaint(paint);
                         g2.fill(side1);
@@ -747,13 +788,10 @@ public class PiePlot3D extends PiePlot implements Serializable {
                     }
 
                     if (drawFront) {
-                        Area side2 = new Area(
-                            new Rectangle2D.Double(
+                        Area side2 = new Area(new Rectangle2D.Double(
                                 arc.getEndPoint().getX(), plotArea.getY(),
                                 plotArea.getMaxX() - arc.getEndPoint().getX(),
-                                plotArea.getHeight()
-                            )
-                        );
+                                plotArea.getHeight()));
                         side2.intersect(front);
                         g2.setPaint(paint);
                         g2.fill(side2);
@@ -772,14 +810,11 @@ public class PiePlot3D extends PiePlot implements Serializable {
 
                     if (extent < 180.0) {  // segment only occupies the front
                         if (drawFront) {
-                            Area side = new Area(
-                                new Rectangle2D.Double(
+                            Area side = new Area(new Rectangle2D.Double(
                                     arc.getStartPoint().getX(), plotArea.getY(),
                                     arc.getEndPoint().getX() 
                                     - arc.getStartPoint().getX(),
-                                    plotArea.getHeight()
-                                )
-                            );
+                                    plotArea.getHeight()));
                             side.intersect(front);
                             g2.setPaint(paint);
                             g2.fill(side);
@@ -788,22 +823,16 @@ public class PiePlot3D extends PiePlot implements Serializable {
                         }
                     }
                     else {  // segments wraps right around the back...
-                        Area side1 = new Area(
-                            new Rectangle2D.Double(
+                        Area side1 = new Area(new Rectangle2D.Double(
                                 arc.getStartPoint().getX(), plotArea.getY(),
                                 plotArea.getMaxX() - arc.getStartPoint().getX(),
-                                plotArea.getHeight()
-                            )
-                        );
+                                plotArea.getHeight()));
                         side1.intersect(front);
 
-                        Area side2 = new Area(
-                            new Rectangle2D.Double(
+                        Area side2 = new Area(new Rectangle2D.Double(
                                 plotArea.getX(), plotArea.getY(),
                                 arc.getEndPoint().getX() - plotArea.getX(),
-                                plotArea.getHeight()
-                            )
-                        );
+                                plotArea.getHeight()));
                         side2.intersect(front);
 
                         g2.setPaint(paint);
@@ -830,13 +859,10 @@ public class PiePlot3D extends PiePlot implements Serializable {
                 }
                 else {  // segments starts at front and finishes at back...
                     if (drawBack) {
-                        Area side2 = new Area(
-                            new Rectangle2D.Double(
+                        Area side2 = new Area(new Rectangle2D.Double(
                                 arc.getEndPoint().getX(), plotArea.getY(),
                                 plotArea.getMaxX() - arc.getEndPoint().getX(),
-                                plotArea.getHeight()
-                            )
-                        );
+                                plotArea.getHeight()));
                         side2.intersect(back);
                         g2.setPaint(paint);
                         g2.fill(side2);
@@ -845,13 +871,10 @@ public class PiePlot3D extends PiePlot implements Serializable {
                     }
 
                     if (drawFront) {
-                        Area side1 = new Area(
-                            new Rectangle2D.Double(
+                        Area side1 = new Area(new Rectangle2D.Double(
                                 arc.getStartPoint().getX(), plotArea.getY(),
                                 plotArea.getMaxX() - arc.getStartPoint().getX(),
-                                plotArea.getHeight()
-                            )
-                        );
+                                plotArea.getHeight()));
                         side1.intersect(front);
                         g2.setPaint(paint);
                         g2.fill(side1);
@@ -865,14 +888,11 @@ public class PiePlot3D extends PiePlot implements Serializable {
                 if (!isAngleAtFront(end)) {
                     if (extent < 180.0) {  // and finishes at back
                         if (drawBack) {
-                            Area side = new Area(
-                                new Rectangle2D.Double(
+                            Area side = new Area(new Rectangle2D.Double(
                                     arc.getEndPoint().getX(), plotArea.getY(),
                                     arc.getStartPoint().getX() 
                                     - arc.getEndPoint().getX(),
-                                    plotArea.getHeight()
-                                )
-                            );
+                                    plotArea.getHeight()));
                             side.intersect(back);
                             g2.setPaint(paint);
                             g2.fill(side);
@@ -882,22 +902,16 @@ public class PiePlot3D extends PiePlot implements Serializable {
                     }
                     else {  // starts at back and wraps right around to the 
                             // back again
-                        Area side1 = new Area(
-                            new Rectangle2D.Double(
+                        Area side1 = new Area(new Rectangle2D.Double(
                                 arc.getStartPoint().getX(), plotArea.getY(),
                                 plotArea.getX() - arc.getStartPoint().getX(),
-                                plotArea.getHeight()
-                            )
-                        );
+                                plotArea.getHeight()));
                         side1.intersect(back);
 
-                        Area side2 = new Area(
-                            new Rectangle2D.Double(
+                        Area side2 = new Area(new Rectangle2D.Double(
                                 arc.getEndPoint().getX(), plotArea.getY(),
                                 plotArea.getMaxX() - arc.getEndPoint().getX(),
-                                plotArea.getHeight()
-                            )
-                        );
+                                plotArea.getHeight()));
                         side2.intersect(back);
 
                         g2.setPaint(paint);
@@ -925,13 +939,10 @@ public class PiePlot3D extends PiePlot implements Serializable {
                 else {  // starts at the back and finishes at the front 
                         // (wrapping the left side)
                     if (drawBack) {
-                        Area side1 = new Area(
-                            new Rectangle2D.Double(
+                        Area side1 = new Area(new Rectangle2D.Double(
                                 plotArea.getX(), plotArea.getY(),
                                 arc.getStartPoint().getX() - plotArea.getX(),
-                                plotArea.getHeight()
-                            )
-                        );
+                                plotArea.getHeight()));
                         side1.intersect(back);
                         g2.setPaint(paint);
                         g2.fill(side1);
@@ -940,13 +951,10 @@ public class PiePlot3D extends PiePlot implements Serializable {
                     }
 
                     if (drawFront) {
-                        Area side2 = new Area(
-                            new Rectangle2D.Double(
+                        Area side2 = new Area(new Rectangle2D.Double(
                                 plotArea.getX(), plotArea.getY(),
                                 arc.getEndPoint().getX() - plotArea.getX(),
-                                plotArea.getHeight()
-                            )
-                        );
+                                plotArea.getHeight()));
                         side2.intersect(front);
                         g2.setPaint(paint);
                         g2.fill(side2);
@@ -965,7 +973,7 @@ public class PiePlot3D extends PiePlot implements Serializable {
      *
      * @return <i>Pie 3D Plot</i>.
      */
-    public String getPlotType () {
+    public String getPlotType() {
         return localizationResources.getString("Pie_3D_Plot");
     }
 
@@ -994,5 +1002,29 @@ public class PiePlot3D extends PiePlot implements Serializable {
     private boolean isAngleAtBack(double angle) {
         return (Math.sin(Math.toRadians(angle)) > 0.0);
     }
+    
+    /**
+     * Tests this plot for equality with an arbitrary object.
+     * 
+     * @param obj  the object (<code>null</code> permitted).
+     * 
+     * @return A boolean.
+     */
+    public boolean equals(Object obj) {
+        if (obj == this) {
+            return true;
+        }
+        if (!(obj instanceof PiePlot3D)) {
+            return false;
+        }
+        PiePlot3D that = (PiePlot3D) obj;
+        if (this.depthFactor != that.depthFactor) {
+            return false;
+        }
+        if (this.darkerSides != that.darkerSides) {
+            return false;
+        }
+        return super.equals(obj);
+    }
 
 }
