@@ -2,32 +2,32 @@
  * JFreeChart : a free chart library for the Java(tm) platform
  * ===========================================================
  *
- * (C) Copyright 2000-2005, by Object Refinery Limited and Contributors.
+ * (C) Copyright 2000-2007, by Object Refinery Limited and Contributors.
  *
  * Project Info:  http://www.jfree.org/jfreechart/index.html
  *
- * This library is free software; you can redistribute it and/or modify it 
- * under the terms of the GNU Lesser General Public License as published by 
- * the Free Software Foundation; either version 2.1 of the License, or 
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful, but 
- * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
- * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+ * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
  * License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, 
- * USA.  
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
+ * USA.
  *
- * [Java is a trademark or registered trademark of Sun Microsystems, Inc. 
+ * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
  * in the United States and other countries.]
  *
  * ------------------
  * BarRenderer3D.java
  * ------------------
- * (C) Copyright 2001-2004, by Serge V. Grachov and Contributors.
+ * (C) Copyright 2001-2007, by Serge V. Grachov and Contributors.
  *
  * Original Author:  Serge V. Grachov;
  * Contributor(s):   David Gilbert (for Object Refinery Limited);
@@ -37,32 +37,30 @@
  *                   Rich Unger;
  *                   Christian W. Zuckschwerdt;
  *
- * $Id: BarRenderer3D.java,v 1.10.2.3 2005/10/25 20:54:16 mungady Exp $
- *
  * Changes
  * -------
  * 31-Oct-2001 : First version, contributed by Serge V. Grachov (DG);
  * 15-Nov-2001 : Modified to allow for null data values (DG);
  * 13-Dec-2001 : Added tooltips (DG);
- * 16-Jan-2002 : Added fix for single category or single series datasets, 
+ * 16-Jan-2002 : Added fix for single category or single series datasets,
  *               pointed out by Taoufik Romdhane (DG);
  * 24-May-2002 : Incorporated tooltips into chart entities (DG);
- * 11-Jun-2002 : Added check for (permitted) null info object, bug and fix 
+ * 11-Jun-2002 : Added check for (permitted) null info object, bug and fix
  *               reported by David Basten.  Also updated Javadocs. (DG);
  * 19-Jun-2002 : Added code to draw labels on bars (TL);
  * 26-Jun-2002 : Added bar clipping to avoid PRExceptions (DG);
- * 05-Aug-2002 : Small modification to drawCategoryItem method to support URLs 
+ * 05-Aug-2002 : Small modification to drawCategoryItem method to support URLs
  *               for HTML image maps (RA);
- * 06-Aug-2002 : Value labels now use number formatter, thanks to Milo 
+ * 06-Aug-2002 : Value labels now use number formatter, thanks to Milo
  *               Simpson (DG);
  * 08-Aug-2002 : Applied fixed in bug id 592218 (DG);
- * 20-Sep-2002 : Added fix for categoryPaint by Rich Unger, and fixed errors 
+ * 20-Sep-2002 : Added fix for categoryPaint by Rich Unger, and fixed errors
  *               reported by Checkstyle (DG);
- * 24-Oct-2002 : Amendments for changes in CategoryDataset interface and 
+ * 24-Oct-2002 : Amendments for changes in CategoryDataset interface and
  *               CategoryToolTipGenerator interface (DG);
  * 05-Nov-2002 : Replaced references to CategoryDataset with TableDataset (DG);
  * 06-Nov-2002 : Moved to the com.jrefinery.chart.renderer package (DG);
- * 28-Jan-2003 : Added an attribute to control the shading of the left and 
+ * 28-Jan-2003 : Added an attribute to control the shading of the left and
  *               bottom walls in the plot background (DG);
  * 25-Mar-2003 : Implemented Serializable (DG);
  * 10-Apr-2003 : Removed category paint usage (DG);
@@ -71,23 +69,29 @@
  * 30-Jul-2003 : Modified entity constructor (CZ);
  * 19-Aug-2003 : Implemented Cloneable and PublicCloneable (DG);
  * 07-Oct-2003 : Added renderer state (DG);
- * 08-Oct-2003 : Removed clipping (replaced with flag in CategoryPlot to 
+ * 08-Oct-2003 : Removed clipping (replaced with flag in CategoryPlot to
  *               control order in which the data items are processed) (DG);
- * 20-Oct-2003 : Fixed bug (outline stroke not being used for bar 
+ * 20-Oct-2003 : Fixed bug (outline stroke not being used for bar
  *               outlines) (DG);
  * 21-Oct-2003 : Bar width moved into CategoryItemRendererState (DG);
  * 24-Nov-2003 : Fixed bug 846324 (item labels not showing) (DG);
  * 27-Nov-2003 : Added code to respect maxBarWidth setting (DG);
  * 02-Feb-2004 : Fixed bug where 'drawBarOutline' flag is not respected (DG);
- * 10-Feb-2004 : Small change to drawItem() method to make cut-and-paste 
+ * 10-Feb-2004 : Small change to drawItem() method to make cut-and-paste
  *               overriding easier (DG);
- * 04-Oct-2004 : Fixed bug with item label positioning when plot alignment is 
+ * 04-Oct-2004 : Fixed bug with item label positioning when plot alignment is
  *               horizontal (DG);
  * 05-Nov-2004 : Modified drawItem() signature (DG);
- * 20-Apr-2005 : Renamed CategoryLabelGenerator 
+ * 20-Apr-2005 : Renamed CategoryLabelGenerator
  *               --> CategoryItemLabelGenerator (DG);
  * 25-Apr-2005 : Override initialise() method to fix bug 1189642 (DG);
  * 09-Jun-2005 : Use addEntityItem from super class (DG);
+ * ------------- JFREECHART 1.0.x ---------------------------------------------
+ * 07-Dec-2006 : Implemented equals() override (DG);
+ * 17-Jan-2007 : Fixed bug in drawDomainGridline() method (DG);
+ * 03-Apr-2007 : Fixed bugs in drawBackground() method (DG);
+ * 16-Oct-2007 : Fixed bug in range marker drawing (DG);
+ *
  */
 
 package org.jfree.chart.renderer.category;
@@ -113,6 +117,7 @@ import org.jfree.chart.Effect3D;
 import org.jfree.chart.axis.CategoryAxis;
 import org.jfree.chart.axis.ValueAxis;
 import org.jfree.chart.entity.EntityCollection;
+import org.jfree.chart.event.RendererChangeEvent;
 import org.jfree.chart.labels.CategoryItemLabelGenerator;
 import org.jfree.chart.labels.ItemLabelAnchor;
 import org.jfree.chart.labels.ItemLabelPosition;
@@ -130,21 +135,19 @@ import org.jfree.ui.LengthAdjustmentType;
 import org.jfree.ui.RectangleAnchor;
 import org.jfree.ui.RectangleEdge;
 import org.jfree.ui.TextAnchor;
+import org.jfree.util.PaintUtilities;
 import org.jfree.util.PublicCloneable;
 
 /**
- * A renderer for bars with a 3D effect, for use with the 
+ * A renderer for bars with a 3D effect, for use with the
  * {@link org.jfree.chart.plot.CategoryPlot} class.
- *
- * @author Serge V. Grachov
  */
-public class BarRenderer3D extends BarRenderer 
-                           implements Effect3D, Cloneable, PublicCloneable, 
-                                      Serializable {
+public class BarRenderer3D extends BarRenderer
+        implements Effect3D, Cloneable, PublicCloneable, Serializable {
 
     /** For serialization. */
     private static final long serialVersionUID = 7686976503536003636L;
-    
+
     /** The default x-offset for the 3D effect. */
     public static final double DEFAULT_X_OFFSET = 12.0;
 
@@ -164,7 +167,7 @@ public class BarRenderer3D extends BarRenderer
     private transient Paint wallPaint;
 
     /**
-     * Default constructor, creates a renderer with a ten pixel '3D effect'.
+     * Default constructor, creates a renderer with a default '3D effect'.
      */
     public BarRenderer3D() {
         this(DEFAULT_X_OFFSET, DEFAULT_Y_OFFSET);
@@ -183,14 +186,12 @@ public class BarRenderer3D extends BarRenderer
         this.yOffset = yOffset;
         this.wallPaint = DEFAULT_WALL_PAINT;
         // set the default item label positions
-        ItemLabelPosition p1 = new ItemLabelPosition(
-            ItemLabelAnchor.INSIDE12, TextAnchor.TOP_CENTER
-        );
-        setPositiveItemLabelPosition(p1);
-        ItemLabelPosition p2 = new ItemLabelPosition(
-            ItemLabelAnchor.INSIDE12, TextAnchor.TOP_CENTER
-        );
-        setNegativeItemLabelPosition(p2);
+        ItemLabelPosition p1 = new ItemLabelPosition(ItemLabelAnchor.INSIDE12,
+                TextAnchor.TOP_CENTER);
+        setBasePositiveItemLabelPosition(p1);
+        ItemLabelPosition p2 = new ItemLabelPosition(ItemLabelAnchor.INSIDE12,
+                TextAnchor.TOP_CENTER);
+        setBaseNegativeItemLabelPosition(p2);
 
     }
 
@@ -198,6 +199,8 @@ public class BarRenderer3D extends BarRenderer
      * Returns the x-offset for the 3D effect.
      *
      * @return The 3D effect.
+     *
+     * @see #getYOffset()
      */
     public double getXOffset() {
         return this.xOffset;
@@ -217,6 +220,8 @@ public class BarRenderer3D extends BarRenderer
      * background.
      *
      * @return The paint.
+     *
+     * @see #setWallPaint(Paint)
      */
     public Paint getWallPaint() {
         return this.wallPaint;
@@ -224,18 +229,25 @@ public class BarRenderer3D extends BarRenderer
 
     /**
      * Sets the paint used to hightlight the left and bottom walls in the plot
-     * background.
+     * background, and sends a {@link RendererChangeEvent} to all registered
+     * listeners.
      *
-     * @param paint  the paint.
+     * @param paint  the paint (<code>null</code> not permitted).
+     *
+     * @see #getWallPaint()
      */
     public void setWallPaint(Paint paint) {
+        if (paint == null) {
+            throw new IllegalArgumentException("Null 'paint' argument.");
+        }
         this.wallPaint = paint;
+        fireChangeEvent();
     }
 
 
     /**
-     * Initialises the renderer and returns a state object that will be passed 
-     * to subsequent calls to the drawItem method.  This method gets called 
+     * Initialises the renderer and returns a state object that will be passed
+     * to subsequent calls to the drawItem method.  This method gets called
      * once at the start of the process of drawing a chart.
      *
      * @param g2  the graphics device.
@@ -243,7 +255,7 @@ public class BarRenderer3D extends BarRenderer
      * @param plot  the plot.
      * @param rendererIndex  the renderer index.
      * @param info  collects chart rendering information for return to caller.
-     * 
+     *
      * @return The renderer state.
      */
     public CategoryItemRendererState initialise(Graphics2D g2,
@@ -252,18 +264,15 @@ public class BarRenderer3D extends BarRenderer
                                                 int rendererIndex,
                                                 PlotRenderingInfo info) {
 
-        Rectangle2D adjusted = new Rectangle2D.Double(
-            dataArea.getX(), dataArea.getY() + getYOffset(),
-            dataArea.getWidth() - getXOffset(), 
-            dataArea.getHeight() - getYOffset()
-        );
-        CategoryItemRendererState state = super.initialise(
-            g2, adjusted, plot, rendererIndex, info
-        );
+        Rectangle2D adjusted = new Rectangle2D.Double(dataArea.getX(),
+                dataArea.getY() + getYOffset(), dataArea.getWidth()
+                - getXOffset(), dataArea.getHeight() - getYOffset());
+        CategoryItemRendererState state = super.initialise(g2, adjusted, plot,
+                rendererIndex, info);
         return state;
-        
+
     }
-    
+
     /**
      * Draws the background for the plot.
      *
@@ -271,7 +280,7 @@ public class BarRenderer3D extends BarRenderer
      * @param plot  the plot.
      * @param dataArea  the area inside the axes.
      */
-    public void drawBackground(Graphics2D g2, CategoryPlot plot, 
+    public void drawBackground(Graphics2D g2, CategoryPlot plot,
                                Rectangle2D dataArea) {
 
         float x0 = (float) dataArea.getX();
@@ -293,6 +302,10 @@ public class BarRenderer3D extends BarRenderer
         clip.lineTo(x2, y0);
         clip.closePath();
 
+        Composite originalComposite = g2.getComposite();
+        g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,
+                plot.getBackgroundAlpha()));
+
         // fill background...
         Paint backgroundPaint = plot.getBackgroundPaint();
         if (backgroundPaint != null) {
@@ -318,7 +331,7 @@ public class BarRenderer3D extends BarRenderer
         g2.setPaint(getWallPaint());
         g2.fill(bottomWall);
 
-        // higlight the background corners...
+        // highlight the background corners...
         g2.setPaint(Color.lightGray);
         Line2D corner = new Line2D.Double(x0, y0, x1, y1);
         g2.draw(corner);
@@ -330,21 +343,15 @@ public class BarRenderer3D extends BarRenderer
         // draw background image, if there is one...
         Image backgroundImage = plot.getBackgroundImage();
         if (backgroundImage != null) {
-            Composite originalComposite = g2.getComposite();
-            g2.setComposite(
-                AlphaComposite.getInstance(
-                    AlphaComposite.SRC, plot.getBackgroundAlpha()
-                )
-            );
-            g2.drawImage(
-                backgroundImage,
-                (int) x1, (int) y3,
-                (int) (x3 - x1 + 1), (int) (y1 - y3 + 1),
-                null
-            );
-            g2.setComposite(originalComposite);
+            Rectangle2D adjusted = new Rectangle2D.Double(dataArea.getX()
+                    + getXOffset(), dataArea.getY(),
+                    dataArea.getWidth() - getXOffset(),
+                    dataArea.getHeight() - getYOffset());
+            plot.drawBackgroundImage(g2, adjusted);
         }
 
+        g2.setComposite(originalComposite);
+
     }
 
     /**
@@ -354,7 +361,7 @@ public class BarRenderer3D extends BarRenderer
      * @param plot  the plot.
      * @param dataArea  the area inside the axes.
      */
-    public void drawOutline(Graphics2D g2, CategoryPlot plot, 
+    public void drawOutline(Graphics2D g2, CategoryPlot plot,
                             Rectangle2D dataArea) {
 
         float x0 = (float) dataArea.getX();
@@ -392,7 +399,7 @@ public class BarRenderer3D extends BarRenderer
      *
      * @param g2  the graphics device.
      * @param plot  the plot.
-     * @param dataArea  the area for plotting data (not yet adjusted for any 
+     * @param dataArea  the area for plotting data (not yet adjusted for any
      *                  3D effect).
      * @param value  the Java2D value at which the grid line should be drawn.
      *
@@ -410,7 +417,7 @@ public class BarRenderer3D extends BarRenderer
             double y1 = value - getYOffset();
             double x0 = dataArea.getMinX();
             double x1 = x0 + getXOffset();
-            double x2 = dataArea.getMaxY();
+            double x2 = dataArea.getMaxX();
             line1 = new Line2D.Double(x0, y0, x1, y1);
             line2 = new Line2D.Double(x1, y1, x2, y1);
         }
@@ -438,7 +445,7 @@ public class BarRenderer3D extends BarRenderer
      * @param g2  the graphics device.
      * @param plot  the plot.
      * @param axis  the value axis.
-     * @param dataArea  the area for plotting data (not yet adjusted for any 
+     * @param dataArea  the area for plotting data (not yet adjusted for any
      *                  3D effect).
      * @param value  the value at which the grid line should be drawn.
      *
@@ -455,20 +462,16 @@ public class BarRenderer3D extends BarRenderer
             return;
         }
 
-        Rectangle2D adjusted = new Rectangle2D.Double(
-            dataArea.getX(),
-            dataArea.getY() + getYOffset(),
-            dataArea.getWidth() - getXOffset(),
-            dataArea.getHeight() - getYOffset()
-        );
+        Rectangle2D adjusted = new Rectangle2D.Double(dataArea.getX(),
+                dataArea.getY() + getYOffset(), dataArea.getWidth()
+                - getXOffset(), dataArea.getHeight() - getYOffset());
 
         Line2D line1 = null;
         Line2D line2 = null;
         PlotOrientation orientation = plot.getOrientation();
         if (orientation == PlotOrientation.HORIZONTAL) {
-            double x0 = axis.valueToJava2D(
-                value, adjusted, plot.getRangeAxisEdge()
-            );
+            double x0 = axis.valueToJava2D(value, adjusted,
+                    plot.getRangeAxisEdge());
             double x1 = x0 + getXOffset();
             double y0 = dataArea.getMaxY();
             double y1 = y0 - getYOffset();
@@ -477,9 +480,8 @@ public class BarRenderer3D extends BarRenderer
             line2 = new Line2D.Double(x1, y1, x1, y2);
         }
         else if (orientation == PlotOrientation.VERTICAL) {
-            double y0 = axis.valueToJava2D(
-                value, adjusted, plot.getRangeAxisEdge()
-            );
+            double y0 = axis.valueToJava2D(value, adjusted,
+                    plot.getRangeAxisEdge());
             double y1 = y0 - getYOffset();
             double x0 = dataArea.getMinX();
             double x1 = x0 + getXOffset();
@@ -511,6 +513,10 @@ public class BarRenderer3D extends BarRenderer
                                 Marker marker,
                                 Rectangle2D dataArea) {
 
+
+        Rectangle2D adjusted = new Rectangle2D.Double(dataArea.getX(),
+                dataArea.getY() + getYOffset(), dataArea.getWidth()
+                - getXOffset(), dataArea.getHeight() - getYOffset());
         if (marker instanceof ValueMarker) {
             ValueMarker vm = (ValueMarker) marker;
             double value = vm.getValue();
@@ -519,43 +525,30 @@ public class BarRenderer3D extends BarRenderer
                 return;
             }
 
-            Rectangle2D adjusted = new Rectangle2D.Double(
-                dataArea.getX(), dataArea.getY() + getYOffset(),
-                dataArea.getWidth() - getXOffset(), 
-                dataArea.getHeight() - getYOffset()
-            );
-
             GeneralPath path = null;
             PlotOrientation orientation = plot.getOrientation();
             if (orientation == PlotOrientation.HORIZONTAL) {
-                float x = (float) axis.valueToJava2D(
-                    value, adjusted, plot.getRangeAxisEdge()
-                );
+                float x = (float) axis.valueToJava2D(value, adjusted,
+                        plot.getRangeAxisEdge());
                 float y = (float) adjusted.getMaxY();
                 path = new GeneralPath();
                 path.moveTo(x, y);
-                path.lineTo(
-                    (float) (x + getXOffset()), y - (float) getYOffset()
-                );
-                path.lineTo(
-                    (float) (x + getXOffset()), 
-                    (float) (adjusted.getMinY() - getYOffset())
-                );
+                path.lineTo((float) (x + getXOffset()),
+                        y - (float) getYOffset());
+                path.lineTo((float) (x + getXOffset()),
+                        (float) (adjusted.getMinY() - getYOffset()));
                 path.lineTo(x, (float) adjusted.getMinY());
                 path.closePath();
             }
             else if (orientation == PlotOrientation.VERTICAL) {
-                float y = (float) axis.valueToJava2D(
-                    value, adjusted, plot.getRangeAxisEdge()
-                );
+                float y = (float) axis.valueToJava2D(value, adjusted,
+                        plot.getRangeAxisEdge());
                 float x = (float) dataArea.getX();
                 path = new GeneralPath();
                 path.moveTo(x, y);
                 path.lineTo(x + (float) this.xOffset, y - (float) this.yOffset);
-                path.lineTo(
-                    (float) (adjusted.getMaxX() + this.xOffset), 
-                    y - (float) this.yOffset
-                );
+                path.lineTo((float) (adjusted.getMaxX() + this.xOffset),
+                        y - (float) this.yOffset);
                 path.lineTo((float) (adjusted.getMaxX()), y);
                 path.closePath();
             }
@@ -563,7 +556,7 @@ public class BarRenderer3D extends BarRenderer
             g2.fill(path);
             g2.setPaint(marker.getOutlinePaint());
             g2.draw(path);
-        
+
             String label = marker.getLabel();
             RectangleAnchor anchor = marker.getLabelAnchor();
             if (label != null) {
@@ -571,19 +564,17 @@ public class BarRenderer3D extends BarRenderer
                 g2.setFont(labelFont);
                 g2.setPaint(marker.getLabelPaint());
                 Point2D coordinates = calculateRangeMarkerTextAnchorPoint(
-                    g2, orientation, dataArea, path.getBounds2D(), 
-                    marker.getLabelOffset(), LengthAdjustmentType.EXPAND, anchor
-                );
-                TextUtilities.drawAlignedString(
-                    label, g2, 
-                    (float) coordinates.getX(), (float) coordinates.getY(), 
-                    marker.getLabelTextAnchor()
-                );
+                        g2, orientation, dataArea, path.getBounds2D(),
+                        marker.getLabelOffset(), LengthAdjustmentType.EXPAND,
+                        anchor);
+                TextUtilities.drawAlignedString(label, g2,
+                        (float) coordinates.getX(), (float) coordinates.getY(),
+                        marker.getLabelTextAnchor());
             }
-        
+
         }
         else {
-            super.drawRangeMarker(g2, plot, axis, marker, dataArea);
+            super.drawRangeMarker(g2, plot, axis, marker, adjusted);
             // TODO: draw the interval marker with a 3D effect
         }
     }
@@ -612,27 +603,24 @@ public class BarRenderer3D extends BarRenderer
                          int row,
                          int column,
                          int pass) {
-    
+
         // check the value we are plotting...
         Number dataValue = dataset.getValue(row, column);
         if (dataValue == null) {
             return;
         }
-        
+
         double value = dataValue.doubleValue();
-        
-        Rectangle2D adjusted = new Rectangle2D.Double(
-            dataArea.getX(),
-            dataArea.getY() + getYOffset(),
-            dataArea.getWidth() - getXOffset(),
-            dataArea.getHeight() - getYOffset()
-        );
+
+        Rectangle2D adjusted = new Rectangle2D.Double(dataArea.getX(),
+                dataArea.getY() + getYOffset(),
+                dataArea.getWidth() - getXOffset(),
+                dataArea.getHeight() - getYOffset());
 
         PlotOrientation orientation = plot.getOrientation();
-        
-        double barW0 = calculateBarW0(
-            plot, orientation, adjusted, domainAxis, state, row, column
-        );
+
+        double barW0 = calculateBarW0(plot, orientation, adjusted, domainAxis,
+                state, row, column);
         double[] barL0L1 = calculateBarL0L1(value);
         if (barL0L1 == null) {
             return;  // the bar is not visible
@@ -643,18 +631,16 @@ public class BarRenderer3D extends BarRenderer
         double transL1 = rangeAxis.valueToJava2D(barL0L1[1], adjusted, edge);
         double barL0 = Math.min(transL0, transL1);
         double barLength = Math.abs(transL1 - transL0);
-        
+
         // draw the bar...
         Rectangle2D bar = null;
         if (orientation == PlotOrientation.HORIZONTAL) {
-            bar = new Rectangle2D.Double(
-                barL0, barW0, barLength, state.getBarWidth()
-            );
+            bar = new Rectangle2D.Double(barL0, barW0, barLength,
+                    state.getBarWidth());
         }
         else {
-            bar = new Rectangle2D.Double(
-                barW0, barL0, state.getBarWidth(), barLength
-            );
+            bar = new Rectangle2D.Double(barW0, barL0, state.getBarWidth(),
+                    barLength);
         }
         Paint itemPaint = getItemPaint(row, column);
         g2.setPaint(itemPaint);
@@ -664,12 +650,12 @@ public class BarRenderer3D extends BarRenderer
         double x1 = x0 + getXOffset();
         double x2 = bar.getMaxX();
         double x3 = x2 + getXOffset();
-        
+
         double y0 = bar.getMinY() - getYOffset();
         double y1 = bar.getMinY();
         double y2 = bar.getMaxY() - getYOffset();
         double y3 = bar.getMaxY();
-        
+
         GeneralPath bar3dRight = null;
         GeneralPath bar3dTop = null;
         if (barLength > 0.0) {
@@ -694,7 +680,7 @@ public class BarRenderer3D extends BarRenderer
         bar3dTop.closePath();
         g2.fill(bar3dTop);
 
-        if (isDrawBarOutline() 
+        if (isDrawBarOutline()
                 && state.getBarWidth() > BAR_OUTLINE_WIDTH_THRESHOLD) {
             g2.setStroke(getItemOutlineStroke(row, column));
             g2.setPaint(getItemOutlinePaint(row, column));
@@ -707,13 +693,12 @@ public class BarRenderer3D extends BarRenderer
             }
         }
 
-        CategoryItemLabelGenerator generator 
+        CategoryItemLabelGenerator generator
             = getItemLabelGenerator(row, column);
         if (generator != null && isItemLabelVisible(row, column)) {
-            drawItemLabel(
-                g2, dataset, row, column, plot, generator, bar, (value < 0.0)
-            );
-        }        
+            drawItemLabel(g2, dataset, row, column, plot, generator, bar,
+                    (value < 0.0));
+        }
 
         // add an item entity, if this information is being collected
         EntityCollection entities = state.getEntityCollection();
@@ -731,6 +716,33 @@ public class BarRenderer3D extends BarRenderer
 
     }
 
+    /**
+     * Tests this renderer for equality with an arbitrary object.
+     *
+     * @param obj  the object (<code>null</code> permitted).
+     *
+     * @return A boolean.
+     */
+    public boolean equals(Object obj) {
+        if (obj == this) {
+            return true;
+        }
+        if (!(obj instanceof BarRenderer3D)) {
+            return false;
+        }
+        BarRenderer3D that = (BarRenderer3D) obj;
+        if (this.xOffset != that.xOffset) {
+            return false;
+        }
+        if (this.yOffset != that.yOffset) {
+            return false;
+        }
+        if (!PaintUtilities.equal(this.wallPaint, that.wallPaint)) {
+            return false;
+        }
+        return super.equals(obj);
+    }
+
     /**
      * Provides serialization support.
      *
@@ -751,11 +763,10 @@ public class BarRenderer3D extends BarRenderer
      * @throws IOException  if there is an I/O error.
      * @throws ClassNotFoundException  if there is a classpath problem.
      */
-    private void readObject(ObjectInputStream stream) 
+    private void readObject(ObjectInputStream stream)
         throws IOException, ClassNotFoundException {
         stream.defaultReadObject();
         this.wallPaint = SerialUtilities.readPaint(stream);
     }
 
-
 }
