@@ -6,22 +6,22 @@
  *
  * Project Info:  http://www.jfree.org/jfreechart/index.html
  *
- * This library is free software; you can redistribute it and/or modify it 
- * under the terms of the GNU Lesser General Public License as published by 
- * the Free Software Foundation; either version 2.1 of the License, or 
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful, but 
- * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
- * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+ * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
  * License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, 
- * USA.  
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
+ * USA.
  *
- * [Java is a trademark or registered trademark of Sun Microsystems, Inc. 
+ * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
  * in the United States and other countries.]
  *
  * --------------------------
@@ -29,26 +29,27 @@
  * --------------------------
  * (C) Copyright 2003-2008, by David Browning and Contributors.
  *
- * Original Author:  David Browning (for the Australian Institute of Marine 
+ * Original Author:  David Browning (for the Australian Institute of Marine
  *                   Science);
  * Contributor(s):   David Gilbert (for Object Refinery Limited);
  *                   Tim Bardzil;
+ *                   Rob Van der Sanden (patches 1866446 and 1888422);
  *
  * Changes
  * -------
- * 21-Aug-2003 : Version 1, contributed by David Browning (for the Australian 
+ * 21-Aug-2003 : Version 1, contributed by David Browning (for the Australian
  *               Institute of Marine Science);
- * 01-Sep-2003 : Incorporated outlier and farout symbols for low values 
+ * 01-Sep-2003 : Incorporated outlier and farout symbols for low values
  *               also (DG);
  * 08-Sep-2003 : Changed ValueAxis API (DG);
  * 16-Sep-2003 : Changed ChartRenderingInfo --> PlotRenderingInfo (DG);
  * 07-Oct-2003 : Added renderer state (DG);
  * 12-Nov-2003 : Fixed casting bug reported by Tim Bardzil (DG);
- * 13-Nov-2003 : Added drawHorizontalItem() method contributed by Tim 
+ * 13-Nov-2003 : Added drawHorizontalItem() method contributed by Tim
  *               Bardzil (DG);
- * 25-Apr-2004 : Added fillBox attribute, equals() method and added 
+ * 25-Apr-2004 : Added fillBox attribute, equals() method and added
  *               serialization code (DG);
- * 29-Apr-2004 : Changed drawing of upper and lower shadows - see bug report 
+ * 29-Apr-2004 : Changed drawing of upper and lower shadows - see bug report
  *               944011 (DG);
  * 05-Nov-2004 : Modified drawItem() signature (DG);
  * 09-Mar-2005 : Override getLegendItem() method so that legend item shapes
@@ -64,6 +65,11 @@
  * 17-May-2007 : Set datasetIndex and seriesIndex in getLegendItem() (DG);
  * 18-May-2007 : Set dataset and seriesKey for LegendItem (DG);
  * 03-Jan-2008 : Check visibility of average marker before drawing it (DG);
+ * 15-Jan-2008 : Add getMaximumBarWidth() and setMaximumBarWidth()
+ *               methods (RVdS);
+ * 14-Feb-2008 : Fix bar position for horizontal chart, see patch
+ *               1888422 (RVdS);
+ * 27-Mar-2008 : Boxes should use outlinePaint/Stroke settings (DG);
  *
  */
 
@@ -90,10 +96,8 @@ import java.util.List;
 import org.jfree.chart.LegendItem;
 import org.jfree.chart.axis.CategoryAxis;
 import org.jfree.chart.axis.ValueAxis;
-import org.jfree.chart.entity.CategoryItemEntity;
 import org.jfree.chart.entity.EntityCollection;
 import org.jfree.chart.event.RendererChangeEvent;
-import org.jfree.chart.labels.CategoryToolTipGenerator;
 import org.jfree.chart.plot.CategoryPlot;
 import org.jfree.chart.plot.PlotOrientation;
 import org.jfree.chart.plot.PlotRenderingInfo;
@@ -108,26 +112,31 @@ import org.jfree.util.PaintUtilities;
 import org.jfree.util.PublicCloneable;
 
 /**
- * A box-and-whisker renderer.  This renderer requires a 
- * {@link BoxAndWhiskerCategoryDataset} and is for use with the 
+ * A box-and-whisker renderer.  This renderer requires a
+ * {@link BoxAndWhiskerCategoryDataset} and is for use with the
  * {@link CategoryPlot} class.
  */
-public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer 
-                                   implements Cloneable, PublicCloneable, 
-                                              Serializable {
+public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
+        implements Cloneable, PublicCloneable, Serializable {
 
     /** For serialization. */
     private static final long serialVersionUID = 632027470694481177L;
-    
+
     /** The color used to paint the median line and average marker. */
     private transient Paint artifactPaint;
 
     /** A flag that controls whether or not the box is filled. */
     private boolean fillBox;
-    
+
     /** The margin between items (boxes) within a category. */
     private double itemMargin;
 
+    /**
+     * The maximum bar width as percentage of the available space in the plot,
+     * where 0.05 is five percent.
+     */
+    private double maximumBarWidth;
+
     /**
      * Default constructor.
      */
@@ -135,11 +144,12 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
         this.artifactPaint = Color.black;
         this.fillBox = true;
         this.itemMargin = 0.20;
+        this.maximumBarWidth = 1.0;
     }
 
     /**
      * Returns the paint used to color the median and average markers.
-     * 
+     *
      * @return The paint used to draw the median and average markers (never
      *     <code>null</code>).
      *
@@ -152,7 +162,7 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
     /**
      * Sets the paint used to color the median and average markers and sends
      * a {@link RendererChangeEvent} to all registered listeners.
-     * 
+     *
      * @param paint  the paint (<code>null</code> not permitted).
      *
      * @see #getArtifactPaint()
@@ -167,19 +177,19 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
 
     /**
      * Returns the flag that controls whether or not the box is filled.
-     * 
+     *
      * @return A boolean.
      *
      * @see #setFillBox(boolean)
      */
     public boolean getFillBox() {
-        return this.fillBox;   
+        return this.fillBox;
     }
-    
+
     /**
-     * Sets the flag that controls whether or not the box is filled and sends a 
+     * Sets the flag that controls whether or not the box is filled and sends a
      * {@link RendererChangeEvent} to all registered listeners.
-     * 
+     *
      * @param flag  the flag.
      *
      * @see #getFillBox()
@@ -190,9 +200,9 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
     }
 
     /**
-     * Returns the item margin.  This is a percentage of the available space 
+     * Returns the item margin.  This is a percentage of the available space
      * that is allocated to the space between items in the chart.
-     * 
+     *
      * @return The margin.
      *
      * @see #setItemMargin(double)
@@ -204,7 +214,7 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
     /**
      * Sets the item margin and sends a {@link RendererChangeEvent} to all
      * registered listeners.
-     * 
+     *
      * @param margin  the margin (a percentage).
      *
      * @see #getItemMargin()
@@ -214,6 +224,36 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
         fireChangeEvent();
     }
 
+    /**
+     * Returns the maximum bar width as a percentage of the available drawing
+     * space.
+     *
+     * @return The maximum bar width.
+     *
+     * @see #setMaximumBarWidth(double)
+     *
+     * @since 1.0.10
+     */
+    public double getMaximumBarWidth() {
+        return this.maximumBarWidth;
+    }
+
+    /**
+     * Sets the maximum bar width, which is specified as a percentage of the
+     * available space for all bars, and sends a {@link RendererChangeEvent}
+     * to all registered listeners.
+     *
+     * @param percent  the maximum Bar Width (a percentage).
+     *
+     * @see #getMaximumBarWidth()
+     *
+     * @since 1.0.10
+     */
+    public void setMaximumBarWidth(double percent) {
+        this.maximumBarWidth = percent;
+        fireChangeEvent();
+    }
+
     /**
      * Returns a legend item for a series.
      *
@@ -235,24 +275,24 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
         }
 
         CategoryDataset dataset = cp.getDataset(datasetIndex);
-        String label = getLegendItemLabelGenerator().generateLabel(dataset, 
+        String label = getLegendItemLabelGenerator().generateLabel(dataset,
                 series);
         String description = label;
-        String toolTipText = null; 
+        String toolTipText = null;
         if (getLegendItemToolTipGenerator() != null) {
             toolTipText = getLegendItemToolTipGenerator().generateLabel(
-                    dataset, series);   
+                    dataset, series);
         }
         String urlText = null;
         if (getLegendItemURLGenerator() != null) {
-            urlText = getLegendItemURLGenerator().generateLabel(dataset, 
-                    series);   
+            urlText = getLegendItemURLGenerator().generateLabel(dataset,
+                    series);
         }
         Shape shape = new Rectangle2D.Double(-4.0, -4.0, 8.0, 8.0);
         Paint paint = lookupSeriesPaint(series);
         Paint outlinePaint = lookupSeriesOutlinePaint(series);
         Stroke outlineStroke = lookupSeriesOutlineStroke(series);
-        LegendItem result = new LegendItem(label, description, toolTipText, 
+        LegendItem result = new LegendItem(label, description, toolTipText,
                 urlText, shape, paint, outlineStroke, outlinePaint);
         result.setDataset(dataset);
         result.setDatasetIndex(datasetIndex);
@@ -263,7 +303,7 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
     }
 
     /**
-     * Initialises the renderer.  This method gets called once at the start of 
+     * Initialises the renderer.  This method gets called once at the start of
      * the process of drawing a chart.
      *
      * @param g2  the graphics device.
@@ -297,6 +337,7 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
             else if (orientation == PlotOrientation.VERTICAL) {
                 space = dataArea.getWidth();
             }
+            double maxWidth = space * getMaximumBarWidth();
             double categoryMargin = 0.0;
             double currentItemMargin = 0.0;
             if (columns > 1) {
@@ -305,18 +346,18 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
             if (rows > 1) {
                 currentItemMargin = getItemMargin();
             }
-            double used = space * (1 - domainAxis.getLowerMargin() 
+            double used = space * (1 - domainAxis.getLowerMargin()
                                      - domainAxis.getUpperMargin()
                                      - categoryMargin - currentItemMargin);
             if ((rows * columns) > 0) {
-                state.setBarWidth(used / (dataset.getColumnCount() 
-                        * dataset.getRowCount()));
+                state.setBarWidth(Math.min(used / (dataset.getColumnCount()
+                        * dataset.getRowCount()), maxWidth));
             }
             else {
-                state.setBarWidth(used);
+                state.setBarWidth(Math.min(used, maxWidth));
             }
         }
-        
+
         return state;
 
     }
@@ -330,7 +371,8 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
      * @param plot  the plot.
      * @param domainAxis  the domain axis.
      * @param rangeAxis  the range axis.
-     * @param dataset  the data.
+     * @param dataset  the data (must be an instance of
+     *                 {@link BoxAndWhiskerCategoryDataset}).
      * @param row  the row index (zero-based).
      * @param column  the column index (zero-based).
      * @param pass  the pass index.
@@ -345,38 +387,39 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
                          int row,
                          int column,
                          int pass) {
-                             
+
         if (!(dataset instanceof BoxAndWhiskerCategoryDataset)) {
             throw new IllegalArgumentException(
-                    "BoxAndWhiskerRenderer.drawItem() : the data should be " 
+                    "BoxAndWhiskerRenderer.drawItem() : the data should be "
                     + "of type BoxAndWhiskerCategoryDataset only.");
         }
 
         PlotOrientation orientation = plot.getOrientation();
 
         if (orientation == PlotOrientation.HORIZONTAL) {
-            drawHorizontalItem(g2, state, dataArea, plot, domainAxis, 
-                    rangeAxis, dataset, row, column);
-        } 
-        else if (orientation == PlotOrientation.VERTICAL) {
-            drawVerticalItem(g2, state, dataArea, plot, domainAxis, 
+            drawHorizontalItem(g2, state, dataArea, plot, domainAxis,
                     rangeAxis, dataset, row, column);
         }
-        
+        else if (orientation == PlotOrientation.VERTICAL) {
+            drawVerticalItem(g2, state, dataArea, plot, domainAxis,
+                    rangeAxis, dataset, row, column);
+        }
+
     }
 
     /**
-     * Draws the visual representation of a single data item when the plot has 
+     * Draws the visual representation of a single data item when the plot has
      * a horizontal orientation.
      *
      * @param g2  the graphics device.
      * @param state  the renderer state.
      * @param dataArea  the area within which the plot is being drawn.
-     * @param plot  the plot (can be used to obtain standard color 
+     * @param plot  the plot (can be used to obtain standard color
      *              information etc).
      * @param domainAxis  the domain axis.
      * @param rangeAxis  the range axis.
-     * @param dataset  the dataset.
+     * @param dataset  the dataset (must be an instance of
+     *                 {@link BoxAndWhiskerCategoryDataset}).
      * @param row  the row index (zero-based).
      * @param column  the column index (zero-based).
      */
@@ -390,12 +433,12 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
                                    int row,
                                    int column) {
 
-        BoxAndWhiskerCategoryDataset bawDataset 
+        BoxAndWhiskerCategoryDataset bawDataset
                 = (BoxAndWhiskerCategoryDataset) dataset;
 
-        double categoryEnd = domainAxis.getCategoryEnd(column, 
+        double categoryEnd = domainAxis.getCategoryEnd(column,
                 getColumnCount(), dataArea, plot.getDomainAxisEdge());
-        double categoryStart = domainAxis.getCategoryStart(column, 
+        double categoryStart = domainAxis.getCategoryStart(column,
                 getColumnCount(), dataArea, plot.getDomainAxisEdge());
         double categoryWidth = Math.abs(categoryEnd - categoryStart);
 
@@ -404,26 +447,23 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
         int categoryCount = getColumnCount();
 
         if (seriesCount > 1) {
-            double seriesGap = dataArea.getWidth() * getItemMargin()
+            double seriesGap = dataArea.getHeight() * getItemMargin()
                                / (categoryCount * (seriesCount - 1));
-            double usedWidth = (state.getBarWidth() * seriesCount) 
+            double usedWidth = (state.getBarWidth() * seriesCount)
                                + (seriesGap * (seriesCount - 1));
             // offset the start of the boxes if the total width used is smaller
             // than the category width
             double offset = (categoryWidth - usedWidth) / 2;
             yy = yy + offset + (row * (state.getBarWidth() + seriesGap));
-        } 
+        }
         else {
-            // offset the start of the box if the box width is smaller than 
+            // offset the start of the box if the box width is smaller than
             // the category width
             double offset = (categoryWidth - state.getBarWidth()) / 2;
             yy = yy + offset;
         }
 
-        Paint p = getItemPaint(row, column);
-        if (p != null) {
-            g2.setPaint(p);
-        }
+        g2.setPaint(getItemPaint(row, column));
         Stroke s = getItemStroke(row, column);
         g2.setStroke(s);
 
@@ -437,7 +477,7 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
         Shape box = null;
         if (xQ1 != null && xQ3 != null && xMax != null && xMin != null) {
 
-            double xxQ1 = rangeAxis.valueToJava2D(xQ1.doubleValue(), dataArea, 
+            double xxQ1 = rangeAxis.valueToJava2D(xQ1.doubleValue(), dataArea,
                     location);
             double xxQ3 = rangeAxis.valueToJava2D(xQ3.doubleValue(), dataArea,
                     location);
@@ -446,10 +486,10 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
             double xxMin = rangeAxis.valueToJava2D(xMin.doubleValue(), dataArea,
                     location);
             double yymid = yy + state.getBarWidth() / 2.0;
-            
+
             // draw the upper shadow...
             g2.draw(new Line2D.Double(xxMax, yymid, xxQ3, yymid));
-            g2.draw(new Line2D.Double(xxMax, yy, xxMax, 
+            g2.draw(new Line2D.Double(xxMax, yy, xxMax,
                     yy + state.getBarWidth()));
 
             // draw the lower shadow...
@@ -458,13 +498,14 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
                     yy + state.getBarWidth()));
 
             // draw the box...
-            box = new Rectangle2D.Double(Math.min(xxQ1, xxQ3), yy, 
+            box = new Rectangle2D.Double(Math.min(xxQ1, xxQ3), yy,
                     Math.abs(xxQ1 - xxQ3), state.getBarWidth());
             if (this.fillBox) {
                 g2.fill(box);
-            } 
+            }
+            g2.setStroke(getItemOutlineStroke(row, column));
+            g2.setPaint(getItemOutlinePaint(row, column));
             g2.draw(box);
-
         }
 
         g2.setPaint(this.artifactPaint);
@@ -473,14 +514,14 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
         // draw mean - SPECIAL AIMS REQUIREMENT...
         Number xMean = bawDataset.getMeanValue(row, column);
         if (xMean != null) {
-            double xxMean = rangeAxis.valueToJava2D(xMean.doubleValue(), 
+            double xxMean = rangeAxis.valueToJava2D(xMean.doubleValue(),
                     dataArea, location);
             aRadius = state.getBarWidth() / 4;
             // here we check that the average marker will in fact be visible
             // before drawing it...
-            if ((xxMean > (dataArea.getMinX() - aRadius)) 
+            if ((xxMean > (dataArea.getMinX() - aRadius))
                     && (xxMean < (dataArea.getMaxX() + aRadius))) {
-                Ellipse2D.Double avgEllipse = new Ellipse2D.Double(xxMean 
+                Ellipse2D.Double avgEllipse = new Ellipse2D.Double(xxMean
                         - aRadius, yy + aRadius, aRadius * 2, aRadius * 2);
                 g2.fill(avgEllipse);
                 g2.draw(avgEllipse);
@@ -490,67 +531,54 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
         // draw median...
         Number xMedian = bawDataset.getMedianValue(row, column);
         if (xMedian != null) {
-            double xxMedian = rangeAxis.valueToJava2D(xMedian.doubleValue(), 
+            double xxMedian = rangeAxis.valueToJava2D(xMedian.doubleValue(),
                     dataArea, location);
-            g2.draw(new Line2D.Double(xxMedian, yy, xxMedian, 
+            g2.draw(new Line2D.Double(xxMedian, yy, xxMedian,
                     yy + state.getBarWidth()));
         }
-        
+
         // collect entity and tool tip information...
         if (state.getInfo() != null && box != null) {
             EntityCollection entities = state.getEntityCollection();
             if (entities != null) {
-                String tip = null;
-                CategoryToolTipGenerator tipster 
-                        = getToolTipGenerator(row, column);
-                if (tipster != null) {
-                    tip = tipster.generateToolTip(dataset, row, column);
-                }
-                String url = null;
-                if (getItemURLGenerator(row, column) != null) {
-                    url = getItemURLGenerator(row, column).generateURL(
-                            dataset, row, column);
-                }
-                CategoryItemEntity entity = new CategoryItemEntity(box, tip, 
-                        url, dataset, dataset.getRowKey(row), 
-                        dataset.getColumnKey(column));
-                entities.add(entity);
+                addItemEntity(entities, dataset, row, column, box);
             }
         }
 
-    } 
-        
+    }
+
     /**
-     * Draws the visual representation of a single data item when the plot has 
+     * Draws the visual representation of a single data item when the plot has
      * a vertical orientation.
      *
      * @param g2  the graphics device.
      * @param state  the renderer state.
      * @param dataArea  the area within which the plot is being drawn.
-     * @param plot  the plot (can be used to obtain standard color information 
+     * @param plot  the plot (can be used to obtain standard color information
      *              etc).
      * @param domainAxis  the domain axis.
      * @param rangeAxis  the range axis.
-     * @param dataset  the dataset.
+     * @param dataset  the dataset (must be an instance of
+     *                 {@link BoxAndWhiskerCategoryDataset}).
      * @param row  the row index (zero-based).
      * @param column  the column index (zero-based).
      */
-    public void drawVerticalItem(Graphics2D g2, 
+    public void drawVerticalItem(Graphics2D g2,
                                  CategoryItemRendererState state,
                                  Rectangle2D dataArea,
-                                 CategoryPlot plot, 
-                                 CategoryAxis domainAxis, 
+                                 CategoryPlot plot,
+                                 CategoryAxis domainAxis,
                                  ValueAxis rangeAxis,
-                                 CategoryDataset dataset, 
-                                 int row, 
+                                 CategoryDataset dataset,
+                                 int row,
                                  int column) {
 
-        BoxAndWhiskerCategoryDataset bawDataset 
+        BoxAndWhiskerCategoryDataset bawDataset
                 = (BoxAndWhiskerCategoryDataset) dataset;
-        
-        double categoryEnd = domainAxis.getCategoryEnd(column, 
+
+        double categoryEnd = domainAxis.getCategoryEnd(column,
                 getColumnCount(), dataArea, plot.getDomainAxisEdge());
-        double categoryStart = domainAxis.getCategoryStart(column, 
+        double categoryStart = domainAxis.getCategoryStart(column,
                 getColumnCount(), dataArea, plot.getDomainAxisEdge());
         double categoryWidth = categoryEnd - categoryStart;
 
@@ -559,29 +587,27 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
         int categoryCount = getColumnCount();
 
         if (seriesCount > 1) {
-            double seriesGap = dataArea.getWidth() * getItemMargin() 
+            double seriesGap = dataArea.getWidth() * getItemMargin()
                                / (categoryCount * (seriesCount - 1));
-            double usedWidth = (state.getBarWidth() * seriesCount) 
+            double usedWidth = (state.getBarWidth() * seriesCount)
                                + (seriesGap * (seriesCount - 1));
             // offset the start of the boxes if the total width used is smaller
             // than the category width
             double offset = (categoryWidth - usedWidth) / 2;
             xx = xx + offset + (row * (state.getBarWidth() + seriesGap));
-        } 
+        }
         else {
-            // offset the start of the box if the box width is smaller than the 
+            // offset the start of the box if the box width is smaller than the
             // category width
             double offset = (categoryWidth - state.getBarWidth()) / 2;
             xx = xx + offset;
-        } 
-        
+        }
+
         double yyAverage = 0.0;
         double yyOutlier;
 
-        Paint p = getItemPaint(row, column);
-        if (p != null) {
-            g2.setPaint(p);
-        }
+        Paint itemPaint = getItemPaint(row, column);
+        g2.setPaint(itemPaint);
         Stroke s = getItemStroke(row, column);
         g2.setStroke(s);
 
@@ -598,47 +624,48 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
 
             double yyQ1 = rangeAxis.valueToJava2D(yQ1.doubleValue(), dataArea,
                     location);
-            double yyQ3 = rangeAxis.valueToJava2D(yQ3.doubleValue(), dataArea, 
+            double yyQ3 = rangeAxis.valueToJava2D(yQ3.doubleValue(), dataArea,
                     location);
-            double yyMax = rangeAxis.valueToJava2D(yMax.doubleValue(), 
+            double yyMax = rangeAxis.valueToJava2D(yMax.doubleValue(),
                     dataArea, location);
-            double yyMin = rangeAxis.valueToJava2D(yMin.doubleValue(), 
+            double yyMin = rangeAxis.valueToJava2D(yMin.doubleValue(),
                     dataArea, location);
             double xxmid = xx + state.getBarWidth() / 2.0;
-            
+
             // draw the upper shadow...
             g2.draw(new Line2D.Double(xxmid, yyMax, xxmid, yyQ3));
-            g2.draw(new Line2D.Double(xx, yyMax, xx + state.getBarWidth(), 
+            g2.draw(new Line2D.Double(xx, yyMax, xx + state.getBarWidth(),
                     yyMax));
 
             // draw the lower shadow...
             g2.draw(new Line2D.Double(xxmid, yyMin, xxmid, yyQ1));
-            g2.draw(new Line2D.Double(xx, yyMin, xx + state.getBarWidth(), 
+            g2.draw(new Line2D.Double(xx, yyMin, xx + state.getBarWidth(),
                     yyMin));
 
             // draw the body...
-            box = new Rectangle2D.Double(xx, Math.min(yyQ1, yyQ3), 
+            box = new Rectangle2D.Double(xx, Math.min(yyQ1, yyQ3),
                     state.getBarWidth(), Math.abs(yyQ1 - yyQ3));
             if (this.fillBox) {
                 g2.fill(box);
             }
+            g2.setStroke(getItemOutlineStroke(row, column));
+            g2.setPaint(getItemOutlinePaint(row, column));
             g2.draw(box);
-  
         }
-        
+
         g2.setPaint(this.artifactPaint);
 
         // draw mean - SPECIAL AIMS REQUIREMENT...
         Number yMean = bawDataset.getMeanValue(row, column);
         if (yMean != null) {
-            yyAverage = rangeAxis.valueToJava2D(yMean.doubleValue(), 
+            yyAverage = rangeAxis.valueToJava2D(yMean.doubleValue(),
                     dataArea, location);
             aRadius = state.getBarWidth() / 4;
             // here we check that the average marker will in fact be visible
             // before drawing it...
-            if ((yyAverage > (dataArea.getMinY() - aRadius)) 
+            if ((yyAverage > (dataArea.getMinY() - aRadius))
                     && (yyAverage < (dataArea.getMaxY() + aRadius))) {
-                Ellipse2D.Double avgEllipse = new Ellipse2D.Double(xx + aRadius, 
+                Ellipse2D.Double avgEllipse = new Ellipse2D.Double(xx + aRadius,
                         yyAverage - aRadius, aRadius * 2, aRadius * 2);
                 g2.fill(avgEllipse);
                 g2.draw(avgEllipse);
@@ -648,28 +675,28 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
         // draw median...
         Number yMedian = bawDataset.getMedianValue(row, column);
         if (yMedian != null) {
-            double yyMedian = rangeAxis.valueToJava2D(yMedian.doubleValue(), 
+            double yyMedian = rangeAxis.valueToJava2D(yMedian.doubleValue(),
                     dataArea, location);
-            g2.draw(new Line2D.Double(xx, yyMedian, xx + state.getBarWidth(), 
+            g2.draw(new Line2D.Double(xx, yyMedian, xx + state.getBarWidth(),
                     yyMedian));
         }
-        
+
         // draw yOutliers...
         double maxAxisValue = rangeAxis.valueToJava2D(
                 rangeAxis.getUpperBound(), dataArea, location) + aRadius;
         double minAxisValue = rangeAxis.valueToJava2D(
                 rangeAxis.getLowerBound(), dataArea, location) - aRadius;
 
-        g2.setPaint(p);
+        g2.setPaint(itemPaint);
 
         // draw outliers
         double oRadius = state.getBarWidth() / 3;    // outlier radius
         List outliers = new ArrayList();
-        OutlierListCollection outlierListCollection 
+        OutlierListCollection outlierListCollection
                 = new OutlierListCollection();
 
-        // From outlier array sort out which are outliers and put these into a 
-        // list If there are any farouts, set the flag on the 
+        // From outlier array sort out which are outliers and put these into a
+        // list If there are any farouts, set the flag on the
         // OutlierListCollection
         List yOutliers = bawDataset.getOutliers(row, column);
         if (yOutliers != null) {
@@ -681,42 +708,42 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
                 Number maxRegular = bawDataset.getMaxRegularValue(row, column);
                 if (outlier > maxOutlier.doubleValue()) {
                     outlierListCollection.setHighFarOut(true);
-                } 
+                }
                 else if (outlier < minOutlier.doubleValue()) {
                     outlierListCollection.setLowFarOut(true);
                 }
                 else if (outlier > maxRegular.doubleValue()) {
-                    yyOutlier = rangeAxis.valueToJava2D(outlier, dataArea, 
+                    yyOutlier = rangeAxis.valueToJava2D(outlier, dataArea,
                             location);
-                    outliers.add(new Outlier(xx + state.getBarWidth() / 2.0, 
+                    outliers.add(new Outlier(xx + state.getBarWidth() / 2.0,
                             yyOutlier, oRadius));
                 }
                 else if (outlier < minRegular.doubleValue()) {
-                    yyOutlier = rangeAxis.valueToJava2D(outlier, dataArea, 
+                    yyOutlier = rangeAxis.valueToJava2D(outlier, dataArea,
                             location);
-                    outliers.add(new Outlier(xx + state.getBarWidth() / 2.0, 
+                    outliers.add(new Outlier(xx + state.getBarWidth() / 2.0,
                             yyOutlier, oRadius));
                 }
                 Collections.sort(outliers);
             }
 
-            // Process outliers. Each outlier is either added to the 
+            // Process outliers. Each outlier is either added to the
             // appropriate outlier list or a new outlier list is made
             for (Iterator iterator = outliers.iterator(); iterator.hasNext();) {
                 Outlier outlier = (Outlier) iterator.next();
                 outlierListCollection.add(outlier);
             }
 
-            for (Iterator iterator = outlierListCollection.iterator(); 
+            for (Iterator iterator = outlierListCollection.iterator();
                      iterator.hasNext();) {
                 OutlierList list = (OutlierList) iterator.next();
                 Outlier outlier = list.getAveragedOutlier();
                 Point2D point = outlier.getPoint();
 
                 if (list.isMultiple()) {
-                    drawMultipleEllipse(point, state.getBarWidth(), oRadius, 
+                    drawMultipleEllipse(point, state.getBarWidth(), oRadius,
                             g2);
-                } 
+                }
                 else {
                     drawEllipse(point, oRadius, g2);
                 }
@@ -724,12 +751,12 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
 
             // draw farout indicators
             if (outlierListCollection.isHighFarOut()) {
-                drawHighFarOut(aRadius / 2.0, g2, 
+                drawHighFarOut(aRadius / 2.0, g2,
                         xx + state.getBarWidth() / 2.0, maxAxisValue);
             }
-        
+
             if (outlierListCollection.isLowFarOut()) {
-                drawLowFarOut(aRadius / 2.0, g2, 
+                drawLowFarOut(aRadius / 2.0, g2,
                         xx + state.getBarWidth() / 2.0, minAxisValue);
             }
         }
@@ -737,53 +764,39 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
         if (state.getInfo() != null && box != null) {
             EntityCollection entities = state.getEntityCollection();
             if (entities != null) {
-                String tip = null;
-                CategoryToolTipGenerator tipster 
-                        = getToolTipGenerator(row, column);
-                if (tipster != null) {
-                    tip = tipster.generateToolTip(dataset, row, column);
-                }
-                String url = null;
-                if (getItemURLGenerator(row, column) != null) {
-                    url = getItemURLGenerator(row, column).generateURL(dataset,
-                            row, column);
-                }
-                CategoryItemEntity entity = new CategoryItemEntity(box, tip, 
-                        url, dataset, dataset.getRowKey(row), 
-                        dataset.getColumnKey(column));
-                entities.add(entity);
+                addItemEntity(entities, dataset, row, column, box);
             }
         }
 
     }
 
     /**
-     * Draws a dot to represent an outlier. 
-     * 
+     * Draws a dot to represent an outlier.
+     *
      * @param point  the location.
      * @param oRadius  the radius.
      * @param g2  the graphics device.
      */
     private void drawEllipse(Point2D point, double oRadius, Graphics2D g2) {
-        Ellipse2D dot = new Ellipse2D.Double(point.getX() + oRadius / 2, 
+        Ellipse2D dot = new Ellipse2D.Double(point.getX() + oRadius / 2,
                 point.getY(), oRadius, oRadius);
         g2.draw(dot);
     }
 
     /**
      * Draws two dots to represent the average value of more than one outlier.
-     * 
+     *
      * @param point  the location
      * @param boxWidth  the box width.
      * @param oRadius  the radius.
      * @param g2  the graphics device.
      */
-    private void drawMultipleEllipse(Point2D point, double boxWidth, 
+    private void drawMultipleEllipse(Point2D point, double boxWidth,
                                      double oRadius, Graphics2D g2)  {
-                                         
-        Ellipse2D dot1 = new Ellipse2D.Double(point.getX() - (boxWidth / 2) 
+
+        Ellipse2D dot1 = new Ellipse2D.Double(point.getX() - (boxWidth / 2)
                 + oRadius, point.getY(), oRadius, oRadius);
-        Ellipse2D dot2 = new Ellipse2D.Double(point.getX() + (boxWidth / 2), 
+        Ellipse2D dot2 = new Ellipse2D.Double(point.getX() + (boxWidth / 2),
                 point.getY(), oRadius, oRadius);
         g2.draw(dot1);
         g2.draw(dot2);
@@ -791,13 +804,13 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
 
     /**
      * Draws a triangle to indicate the presence of far-out values.
-     * 
+     *
      * @param aRadius  the radius.
      * @param g2  the graphics device.
      * @param xx  the x coordinate.
      * @param m  the y coordinate.
      */
-    private void drawHighFarOut(double aRadius, Graphics2D g2, double xx, 
+    private void drawHighFarOut(double aRadius, Graphics2D g2, double xx,
                                 double m) {
         double side = aRadius * 2;
         g2.draw(new Line2D.Double(xx - side, m + side, xx + side, m + side));
@@ -807,20 +820,20 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
 
     /**
      * Draws a triangle to indicate the presence of far-out values.
-     * 
+     *
      * @param aRadius  the radius.
      * @param g2  the graphics device.
      * @param xx  the x coordinate.
      * @param m  the y coordinate.
      */
-    private void drawLowFarOut(double aRadius, Graphics2D g2, double xx, 
+    private void drawLowFarOut(double aRadius, Graphics2D g2, double xx,
                                double m) {
         double side = aRadius * 2;
         g2.draw(new Line2D.Double(xx - side, m - side, xx + side, m - side));
         g2.draw(new Line2D.Double(xx - side, m - side, xx, m));
         g2.draw(new Line2D.Double(xx + side, m - side, xx, m));
     }
-    
+
     /**
      * Tests this renderer for equality with an arbitrary object.
      *
@@ -830,10 +843,10 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
      */
     public boolean equals(Object obj) {
         if (obj == this) {
-            return true;   
+            return true;
         }
         if (!(obj instanceof BoxAndWhiskerRenderer)) {
-            return false;   
+            return false;
         }
         if (!super.equals(obj)) {
             return false;
@@ -842,15 +855,18 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
         if (!PaintUtilities.equal(this.artifactPaint, that.artifactPaint)) {
             return false;
         }
-        if (!(this.fillBox == that.fillBox)) {
-            return false;   
+        if (this.fillBox != that.fillBox) {
+            return false;
         }
-        if (!(this.itemMargin == that.itemMargin)) {
-            return false;   
+        if (this.itemMargin != that.itemMargin) {
+            return false;
+        }
+        if (this.maximumBarWidth != that.maximumBarWidth) {
+            return false;
         }
         return true;
     }
-    
+
     /**
      * Provides serialization support.
      *
@@ -871,10 +887,10 @@ public class BoxAndWhiskerRenderer extends AbstractCategoryItemRenderer
      * @throws IOException  if there is an I/O error.
      * @throws ClassNotFoundException  if there is a classpath problem.
      */
-    private void readObject(ObjectInputStream stream) 
+    private void readObject(ObjectInputStream stream)
             throws IOException, ClassNotFoundException {
         stream.defaultReadObject();
         this.artifactPaint = SerialUtilities.readPaint(stream);
     }
-   
+
 }
