@@ -2,38 +2,36 @@
  * JFreeChart : a free chart library for the Java(tm) platform
  * ===========================================================
  *
- * (C) Copyright 2000-2006, by Object Refinery Limited and Contributors.
+ * (C) Copyright 2000-2008, by Object Refinery Limited and Contributors.
  *
  * Project Info:  http://www.jfree.org/jfreechart/index.html
  *
- * This library is free software; you can redistribute it and/or modify it 
- * under the terms of the GNU Lesser General Public License as published by 
- * the Free Software Foundation; either version 2.1 of the License, or 
+ * This library is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful, but 
- * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
- * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+ * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
  * License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, 
- * USA.  
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
+ * USA.
  *
- * [Java is a trademark or registered trademark of Sun Microsystems, Inc. 
+ * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
  * in the United States and other countries.]
  *
  * -------------------
  * LineRenderer3D.java
  * -------------------
- * (C) Copyright 2004-2006, by Tobias Selb and Contributors.
+ * (C) Copyright 2004-2008, by Tobias Selb and Contributors.
  *
- * Original Author:  Tobias Selb;
+ * Original Author:  Tobias Selb (http://www.uepselon.com);
  * Contributor(s):   David Gilbert (for Object Refinery Limited);
  *
- * $Id: LineRenderer3D.java,v 1.10.2.5 2006/08/04 11:30:39 mungady Exp $
- *
  * Changes
  * -------
  * 15-Oct-2004 : Version 1 (TS);
@@ -43,7 +41,13 @@
  * 13-Apr-2005 : Check item visibility in drawItem() method (DG);
  * 09-Jun-2005 : Use addItemEntity() in drawItem() method (DG);
  * 10-Jun-2005 : Fixed capitalisation of setXOffset() and setYOffset() (DG);
- * 
+ * ------------- JFREECHART 1.0.x ---------------------------------------------
+ * 01-Dec-2006 : Fixed equals() and serialization (DG);
+ * 17-Jan-2007 : Fixed bug in drawDomainGridline() method and added
+ *               argument check to setWallPaint() (DG);
+ * 03-Apr-2007 : Fixed bugs in drawBackground() method (DG);
+ * 16-Oct-2007 : Fixed bug in range marker drawing (DG);
+ *
  */
 
 package org.jfree.chart.renderer.category;
@@ -59,6 +63,9 @@ import java.awt.Stroke;
 import java.awt.geom.GeneralPath;
 import java.awt.geom.Line2D;
 import java.awt.geom.Rectangle2D;
+import java.io.IOException;
+import java.io.ObjectInputStream;
+import java.io.ObjectOutputStream;
 import java.io.Serializable;
 
 import org.jfree.chart.Effect3D;
@@ -72,37 +79,37 @@ import org.jfree.chart.plot.PlotOrientation;
 import org.jfree.chart.plot.ValueMarker;
 import org.jfree.data.Range;
 import org.jfree.data.category.CategoryDataset;
+import org.jfree.io.SerialUtilities;
+import org.jfree.util.PaintUtilities;
 import org.jfree.util.ShapeUtilities;
 
 /**
  * A line renderer with a 3D effect.
- * 
- * @author Tobias Selb (http://www.uepselon.com) 
  */
-public class LineRenderer3D extends LineAndShapeRenderer 
+public class LineRenderer3D extends LineAndShapeRenderer
                             implements Effect3D, Serializable {
-   
+
     /** For serialization. */
     private static final long serialVersionUID = 5467931468380928736L;
-    
+
     /** The default x-offset for the 3D effect. */
     public static final double DEFAULT_X_OFFSET = 12.0;
 
     /** The default y-offset for the 3D effect. */
     public static final double DEFAULT_Y_OFFSET = 8.0;
-   
+
     /** The default wall paint. */
     public static final Paint DEFAULT_WALL_PAINT = new Color(0xDD, 0xDD, 0xDD);
-   
+
     /** The size of x-offset for the 3D effect. */
     private double xOffset;
 
     /** The size of y-offset for the 3D effect. */
     private double yOffset;
-   
+
     /** The paint used to shade the left and lower 3D wall. */
     private transient Paint wallPaint;
-   
+
     /**
      * Creates a new renderer.
      */
@@ -112,11 +119,14 @@ public class LineRenderer3D extends LineAndShapeRenderer
         this.yOffset = DEFAULT_Y_OFFSET;
         this.wallPaint = DEFAULT_WALL_PAINT;
     }
-   
+
     /**
      * Returns the x-offset for the 3D effect.
      *
      * @return The x-offset.
+     *
+     * @see #setXOffset(double)
+     * @see #getYOffset()
      */
     public double getXOffset() {
         return this.xOffset;
@@ -126,29 +136,38 @@ public class LineRenderer3D extends LineAndShapeRenderer
      * Returns the y-offset for the 3D effect.
      *
      * @return The y-offset.
+     *
+     * @see #setYOffset(double)
+     * @see #getXOffset()
      */
     public double getYOffset() {
         return this.yOffset;
     }
-   
+
     /**
-     * Sets the x-offset.
-     * 
+     * Sets the x-offset and sends a {@link RendererChangeEvent} to all
+     * registered listeners.
+     *
      * @param xOffset  the x-offset.
+     *
+     * @see #getXOffset()
      */
     public void setXOffset(double xOffset) {
         this.xOffset = xOffset;
-        notifyListeners(new RendererChangeEvent(this));
+        fireChangeEvent();
     }
 
     /**
-     * Sets the y-offset.
-     * 
+     * Sets the y-offset and sends a {@link RendererChangeEvent} to all
+     * registered listeners.
+     *
      * @param yOffset  the y-offset.
+     *
+     * @see #getYOffset()
      */
     public void setYOffset(double yOffset) {
         this.yOffset = yOffset;
-        notifyListeners(new RendererChangeEvent(this));
+        fireChangeEvent();
     }
 
     /**
@@ -156,6 +175,8 @@ public class LineRenderer3D extends LineAndShapeRenderer
      * background.
      *
      * @return The paint.
+     *
+     * @see #setWallPaint(Paint)
      */
     public Paint getWallPaint() {
         return this.wallPaint;
@@ -163,15 +184,21 @@ public class LineRenderer3D extends LineAndShapeRenderer
 
     /**
      * Sets the paint used to hightlight the left and bottom walls in the plot
-     * background.
+     * background, and sends a {@link RendererChangeEvent} to all
+     * registered listeners.
      *
-     * @param paint  the paint.
+     * @param paint  the paint (<code>null</code> not permitted).
+     *
+     * @see #getWallPaint()
      */
     public void setWallPaint(Paint paint) {
+        if (paint == null) {
+            throw new IllegalArgumentException("Null 'paint' argument.");
+        }
         this.wallPaint = paint;
-        notifyListeners(new RendererChangeEvent(this));
+        fireChangeEvent();
     }
-   
+
     /**
      * Draws the background for the plot.
      *
@@ -179,7 +206,7 @@ public class LineRenderer3D extends LineAndShapeRenderer
      * @param plot  the plot.
      * @param dataArea  the area inside the axes.
      */
-    public void drawBackground(Graphics2D g2, CategoryPlot plot, 
+    public void drawBackground(Graphics2D g2, CategoryPlot plot,
                                Rectangle2D dataArea) {
 
         float x0 = (float) dataArea.getX();
@@ -201,6 +228,10 @@ public class LineRenderer3D extends LineAndShapeRenderer
         clip.lineTo(x2, y0);
         clip.closePath();
 
+        Composite originalComposite = g2.getComposite();
+        g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,
+                plot.getBackgroundAlpha()));
+
         // fill background...
         Paint backgroundPaint = plot.getBackgroundPaint();
         if (backgroundPaint != null) {
@@ -238,14 +269,15 @@ public class LineRenderer3D extends LineAndShapeRenderer
         // draw background image, if there is one...
         Image backgroundImage = plot.getBackgroundImage();
         if (backgroundImage != null) {
-            Composite originalComposite = g2.getComposite();
-            g2.setComposite(AlphaComposite.getInstance(
-                    AlphaComposite.SRC, plot.getBackgroundAlpha()));
-            g2.drawImage(backgroundImage, (int) x1, (int) y3, 
-                    (int) (x3 - x1 + 1), (int) (y1 - y3 + 1), null);
-            g2.setComposite(originalComposite);
+            Rectangle2D adjusted = new Rectangle2D.Double(dataArea.getX()
+                    + getXOffset(), dataArea.getY(),
+                    dataArea.getWidth() - getXOffset(),
+                    dataArea.getHeight() - getYOffset());
+            plot.drawBackgroundImage(g2, adjusted);
         }
 
+        g2.setComposite(originalComposite);
+
     }
 
     /**
@@ -255,7 +287,7 @@ public class LineRenderer3D extends LineAndShapeRenderer
      * @param plot  the plot.
      * @param dataArea  the area inside the axes.
      */
-    public void drawOutline(Graphics2D g2, CategoryPlot plot, 
+    public void drawOutline(Graphics2D g2, CategoryPlot plot,
                             Rectangle2D dataArea) {
 
         float x0 = (float) dataArea.getX();
@@ -293,7 +325,7 @@ public class LineRenderer3D extends LineAndShapeRenderer
      *
      * @param g2  the graphics device.
      * @param plot  the plot.
-     * @param dataArea  the area for plotting data (not yet adjusted for any 
+     * @param dataArea  the area for plotting data (not yet adjusted for any
      *                  3D effect).
      * @param value  the Java2D value at which the grid line should be drawn.
      *
@@ -311,7 +343,7 @@ public class LineRenderer3D extends LineAndShapeRenderer
             double y1 = value - getYOffset();
             double x0 = dataArea.getMinX();
             double x1 = x0 + getXOffset();
-            double x2 = dataArea.getMaxY();
+            double x2 = dataArea.getMaxX();
             line1 = new Line2D.Double(x0, y0, x1, y1);
             line2 = new Line2D.Double(x1, y1, x2, y1);
         }
@@ -337,7 +369,7 @@ public class LineRenderer3D extends LineAndShapeRenderer
      * @param g2  the graphics device.
      * @param plot  the plot.
      * @param axis  the value axis.
-     * @param dataArea  the area for plotting data (not yet adjusted for any 
+     * @param dataArea  the area for plotting data (not yet adjusted for any
      *                  3D effect).
      * @param value  the value at which the grid line should be drawn.
      *
@@ -363,7 +395,7 @@ public class LineRenderer3D extends LineAndShapeRenderer
         Line2D line2 = null;
         PlotOrientation orientation = plot.getOrientation();
         if (orientation == PlotOrientation.HORIZONTAL) {
-            double x0 = axis.valueToJava2D(value, adjusted, 
+            double x0 = axis.valueToJava2D(value, adjusted,
                     plot.getRangeAxisEdge());
             double x1 = x0 + getXOffset();
             double y0 = dataArea.getMaxY();
@@ -404,6 +436,11 @@ public class LineRenderer3D extends LineAndShapeRenderer
                                 Marker marker,
                                 Rectangle2D dataArea) {
 
+        Rectangle2D adjusted = new Rectangle2D.Double(dataArea.getX(),
+                dataArea.getY() + getYOffset(),
+                dataArea.getWidth() - getXOffset(),
+                dataArea.getHeight() - getYOffset());
+
         if (marker instanceof ValueMarker) {
             ValueMarker vm = (ValueMarker) marker;
             double value = vm.getValue();
@@ -412,34 +449,29 @@ public class LineRenderer3D extends LineAndShapeRenderer
                 return;
             }
 
-            Rectangle2D adjusted = new Rectangle2D.Double(dataArea.getX(), 
-                    dataArea.getY() + getYOffset(), 
-                    dataArea.getWidth() - getXOffset(), 
-                    dataArea.getHeight() - getYOffset());
-
             GeneralPath path = null;
             PlotOrientation orientation = plot.getOrientation();
             if (orientation == PlotOrientation.HORIZONTAL) {
-                float x = (float) axis.valueToJava2D(value, adjusted, 
+                float x = (float) axis.valueToJava2D(value, adjusted,
                         plot.getRangeAxisEdge());
                 float y = (float) adjusted.getMaxY();
                 path = new GeneralPath();
                 path.moveTo(x, y);
-                path.lineTo((float) (x + getXOffset()), 
+                path.lineTo((float) (x + getXOffset()),
                         y - (float) getYOffset());
-                path.lineTo((float) (x + getXOffset()), 
+                path.lineTo((float) (x + getXOffset()),
                         (float) (adjusted.getMinY() - getYOffset()));
                 path.lineTo(x, (float) adjusted.getMinY());
                 path.closePath();
             }
             else if (orientation == PlotOrientation.VERTICAL) {
-                float y = (float) axis.valueToJava2D(value, adjusted, 
+                float y = (float) axis.valueToJava2D(value, adjusted,
                         plot.getRangeAxisEdge());
                 float x = (float) dataArea.getX();
                 path = new GeneralPath();
                 path.moveTo(x, y);
                 path.lineTo(x + (float) this.xOffset, y - (float) this.yOffset);
-                path.lineTo((float) (adjusted.getMaxX() + this.xOffset), 
+                path.lineTo((float) (adjusted.getMaxX() + this.xOffset),
                         y - (float) this.yOffset);
                 path.lineTo((float) (adjusted.getMaxX()), y);
                 path.closePath();
@@ -449,8 +481,12 @@ public class LineRenderer3D extends LineAndShapeRenderer
             g2.setPaint(marker.getOutlinePaint());
             g2.draw(path);
         }
+        else {
+            super.drawRangeMarker(g2, plot, axis, marker, adjusted);
+            // TODO: draw the interval marker with a 3D effect
+        }
     }
-   
+
    /**
      * Draw a single data item.
      *
@@ -477,27 +513,27 @@ public class LineRenderer3D extends LineAndShapeRenderer
                          int pass) {
 
         if (!getItemVisible(row, column)) {
-            return;   
+            return;
         }
-        
+
         // nothing is drawn for null...
         Number v = dataset.getValue(row, column);
         if (v == null) {
             return;
         }
-       
+
         Rectangle2D adjusted = new Rectangle2D.Double(dataArea.getX(),
-                dataArea.getY() + getYOffset(), 
+                dataArea.getY() + getYOffset(),
                 dataArea.getWidth() - getXOffset(),
                 dataArea.getHeight() - getYOffset());
-       
+
         PlotOrientation orientation = plot.getOrientation();
 
         // current data point...
-        double x1 = domainAxis.getCategoryMiddle(column, getColumnCount(), 
+        double x1 = domainAxis.getCategoryMiddle(column, getColumnCount(),
                 adjusted, plot.getDomainAxisEdge());
         double value = v.doubleValue();
-        double y1 = rangeAxis.valueToJava2D(value, adjusted, 
+        double y1 = rangeAxis.valueToJava2D(value, adjusted,
                 plot.getRangeAxisEdge());
 
         Shape shape = getItemShape(row, column);
@@ -507,7 +543,7 @@ public class LineRenderer3D extends LineAndShapeRenderer
         else if (orientation == PlotOrientation.VERTICAL) {
             shape = ShapeUtilities.createTranslatedShape(shape, x1, y1);
         }
-       
+
         if (getItemLineVisible(row, column)) {
             if (column != 0) {
 
@@ -516,19 +552,19 @@ public class LineRenderer3D extends LineAndShapeRenderer
 
                     // previous data point...
                     double previous = previousValue.doubleValue();
-                    double x0 = domainAxis.getCategoryMiddle(column - 1, 
-                            getColumnCount(), adjusted, 
+                    double x0 = domainAxis.getCategoryMiddle(column - 1,
+                            getColumnCount(), adjusted,
                             plot.getDomainAxisEdge());
-                    double y0 = rangeAxis.valueToJava2D(previous, adjusted, 
+                    double y0 = rangeAxis.valueToJava2D(previous, adjusted,
                             plot.getRangeAxisEdge());
 
                     double x2 = x0 + getXOffset();
                     double y2 = y0 - getYOffset();
                     double x3 = x1 + getXOffset();
                     double y3 = y1 - getYOffset();
-                   
+
                     GeneralPath clip = new GeneralPath();
-                   
+
                     if (orientation == PlotOrientation.HORIZONTAL) {
                         clip.moveTo((float) y0, (float) x0);
                         clip.lineTo((float) y1, (float) x1);
@@ -545,7 +581,7 @@ public class LineRenderer3D extends LineAndShapeRenderer
                         clip.lineTo((float) x0, (float) y0);
                         clip.closePath();
                     }
-                   
+
                     g2.setPaint(getItemPaint(row, column));
                     g2.fill(clip);
                     g2.setStroke(getItemOutlineStroke(row, column));
@@ -557,7 +593,7 @@ public class LineRenderer3D extends LineAndShapeRenderer
 
         // draw the item label if there is one...
         if (isItemLabelVisible(row, column)) {
-            drawItemLabel(g2, orientation, dataset, row, column, x1, y1, 
+            drawItemLabel(g2, orientation, dataset, row, column, x1, y1,
                     (value < 0.0));
         }
 
@@ -569,4 +605,57 @@ public class LineRenderer3D extends LineAndShapeRenderer
 
     }
 
+    /**
+     * Checks this renderer for equality with an arbitrary object.
+     *
+     * @param obj  the object (<code>null</code> permitted).
+     *
+     * @return A boolean.
+     */
+    public boolean equals(Object obj) {
+        if (obj == this) {
+            return true;
+        }
+        if (!(obj instanceof LineRenderer3D)) {
+            return false;
+        }
+        LineRenderer3D that = (LineRenderer3D) obj;
+        if (this.xOffset != that.xOffset) {
+            return false;
+        }
+        if (this.yOffset != that.yOffset) {
+            return false;
+        }
+        if (!PaintUtilities.equal(this.wallPaint, that.wallPaint)) {
+            return false;
+        }
+        return super.equals(obj);
+    }
+
+    /**
+     * Provides serialization support.
+     *
+     * @param stream  the output stream.
+     *
+     * @throws IOException  if there is an I/O error.
+     */
+    private void writeObject(ObjectOutputStream stream) throws IOException {
+        stream.defaultWriteObject();
+        SerialUtilities.writePaint(this.wallPaint, stream);
+    }
+
+    /**
+     * Provides serialization support.
+     *
+     * @param stream  the input stream.
+     *
+     * @throws IOException  if there is an I/O error.
+     * @throws ClassNotFoundException  if there is a classpath problem.
+     */
+    private void readObject(ObjectInputStream stream)
+            throws IOException, ClassNotFoundException {
+        stream.defaultReadObject();
+        this.wallPaint = SerialUtilities.readPaint(stream);
+    }
+
 }
