@@ -2,7 +2,7 @@
  * JFreeChart : a free chart library for the Java(tm) platform
  * ===========================================================
  *
- * (C) Copyright 2000-2005, by Object Refinery Limited and Contributors.
+ * (C) Copyright 2000-2007, by Object Refinery Limited and Contributors.
  *
  * Project Info:  http://www.jfree.org/jfreechart/index.html
  *
@@ -27,15 +27,13 @@
  * --------------------
  * XYAreaRenderer2.java
  * --------------------
- * (C) Copyright 2004, 2005, by Hari and Contributors.
+ * (C) Copyright 2004-2007, by Hari and Contributors.
  *
  * Original Author:  Hari (ourhari@hotmail.com);
  * Contributor(s):   David Gilbert (for Object Refinery Limited);
  *                   Richard Atkinson;
  *                   Christian W. Zuckschwerdt;
  *
- * $Id: XYAreaRenderer2.java,v 1.12.2.4 2005/12/02 11:59:43 mungady Exp $
- *
  * Changes:
  * --------
  * 03-Apr-2002 : Version 1, contributed by Hari.  This class is based on the 
@@ -69,11 +67,19 @@
  * 19-Jan-2005 : Now accesses only primitives from the dataset (DG);
  * 21-Mar-2005 : Override getLegendItem() (DG);
  * 20-Apr-2005 : Use generators for legend tooltips and URLs (DG);
- * 
+ * ------------- JFREECHART 1.0.x ---------------------------------------------
+ * 30-Nov-2006 : Fixed equals() and clone() implementations (DG);
+ * 06-Feb-2007 : Fixed bug 1086307, crosshairs with multiple axes (DG);
+ * 20-Apr-2007 : Updated getLegendItem() and drawItem() for renderer 
+ *               change (DG); 
+ * 17-May-2007 : Set datasetIndex and seriesIndex in getLegendItem() (DG);
+ * 18-May-2007 : Set dataset and seriesKey for LegendItem (DG);
+ *
  */
 
 package org.jfree.chart.renderer.xy;
 
+
 import java.awt.Graphics2D;
 import java.awt.Paint;
 import java.awt.Polygon;
@@ -101,6 +107,7 @@ import org.jfree.chart.urls.XYURLGenerator;
 import org.jfree.data.xy.XYDataset;
 import org.jfree.io.SerialUtilities;
 import org.jfree.util.PublicCloneable;
+import org.jfree.util.ShapeUtilities;
 
 /**
  * Area item renderer for an {@link XYPlot}.  
@@ -114,9 +121,6 @@ public class XYAreaRenderer2 extends AbstractXYItemRenderer
     /** For serialization. */
     private static final long serialVersionUID = -7378069681579984133L;
 
-    /** A flag indicating whether or not lines are drawn between XY points. */
-    private boolean plotLines;
-
     /** A flag that controls whether or not the outline is shown. */
     private boolean showOutline;
 
@@ -135,9 +139,6 @@ public class XYAreaRenderer2 extends AbstractXYItemRenderer
 
     /**
      * Constructs a new renderer.
-     * <p>
-     * To specify the type of renderer, use one of the constants: SHAPES, LINES,
-     * SHAPES_AND_LINES, AREA or AREA_AND_SHAPES.
      *
      * @param labelGenerator  the tool tip generator to use.  <code>null</code> 
      *                        is none.
@@ -146,7 +147,6 @@ public class XYAreaRenderer2 extends AbstractXYItemRenderer
     public XYAreaRenderer2(XYToolTipGenerator labelGenerator, 
                            XYURLGenerator urlGenerator) {
         super();
-        this.plotLines = false;
         this.showOutline = false;
         setBaseToolTipGenerator(labelGenerator);
         setURLGenerator(urlGenerator);
@@ -165,33 +165,46 @@ public class XYAreaRenderer2 extends AbstractXYItemRenderer
      * drawn.
      *
      * @return The flag.
+     * 
+     * @see #setOutline(boolean)
      */
     public boolean isOutline() {
         return this.showOutline;
     }
 
     /**
-     * Sets a flag that controls whether or not outlines of the areas are drawn.
+     * Sets a flag that controls whether or not outlines of the areas are 
+     * drawn, and sends a {@link RendererChangeEvent} to all registered 
+     * listeners.
      *
      * @param show  the flag.
+     * 
+     * @see #isOutline()
      */
     public void setOutline(boolean show) {
         this.showOutline = show;
+        fireChangeEvent();
     }
 
     /**
-     * Returns true if lines are being plotted by the renderer.
+     * This method should not be used.
      *
-     * @return <code>true</code> if lines are being plotted by the renderer.
+     * @return <code>false</code> always.
+     * 
+     * @deprecated This method was included in the API by mistake and serves
+     *     no useful purpose.  It has always returned <code>false</code>.
+     *   
      */
     public boolean getPlotLines() {
-        return this.plotLines;
+        return false;
     }
 
     /**
      * Returns the shape used to represent an area in the legend.
      * 
      * @return The legend area (never <code>null</code>).
+     * 
+     * @see #setLegendArea(Shape)
      */
     public Shape getLegendArea() {
         return this.legendArea;   
@@ -202,13 +215,15 @@ public class XYAreaRenderer2 extends AbstractXYItemRenderer
      * {@link RendererChangeEvent} to all registered listeners.
      * 
      * @param area  the area (<code>null</code> not permitted).
+     * 
+     * @see #getLegendArea()
      */
     public void setLegendArea(Shape area) {
         if (area == null) {
             throw new IllegalArgumentException("Null 'area' argument.");   
         }
         this.legendArea = area;
-        notifyListeners(new RendererChangeEvent(this));
+        fireChangeEvent();
     }
 
     /**
@@ -232,18 +247,20 @@ public class XYAreaRenderer2 extends AbstractXYItemRenderer
                 String toolTipText = null;
                 if (getLegendItemToolTipGenerator() != null) {
                     toolTipText = getLegendItemToolTipGenerator().generateLabel(
-                        dataset, series
-                    );
+                            dataset, series);
                 }
                 String urlText = null;
                 if (getLegendItemURLGenerator() != null) {
                     urlText = getLegendItemURLGenerator().generateLabel(
-                        dataset, series
-                    );
+                            dataset, series);
                 }
-                Paint paint = getSeriesPaint(series);
+                Paint paint = lookupSeriesPaint(series);
                 result = new LegendItem(label, description, toolTipText, 
                         urlText, this.legendArea, paint);
+                result.setDataset(dataset);
+                result.setDatasetIndex(datasetIndex);
+                result.setSeriesKey(dataset.getSeriesKey(series));
+                result.setSeriesIndex(series);
             }
         }
         return result;
@@ -290,12 +307,10 @@ public class XYAreaRenderer2 extends AbstractXYItemRenderer
             y1 = 0.0;
         }
         
-        double transX1 = domainAxis.valueToJava2D(
-            x1, dataArea, plot.getDomainAxisEdge()
-        );
-        double transY1 = rangeAxis.valueToJava2D(
-            y1, dataArea, plot.getRangeAxisEdge()
-        );
+        double transX1 = domainAxis.valueToJava2D(x1, dataArea, 
+                plot.getDomainAxisEdge());
+        double transY1 = rangeAxis.valueToJava2D(y1, dataArea, 
+                plot.getRangeAxisEdge());
         
         // get the previous point and the next point so we can calculate a 
         // "hot spot" for the area (used by the chart entity)...
@@ -304,69 +319,50 @@ public class XYAreaRenderer2 extends AbstractXYItemRenderer
         if (Double.isNaN(y0)) {
             y0 = 0.0;
         }
-        double transX0 = domainAxis.valueToJava2D(
-            x0, dataArea, plot.getDomainAxisEdge()
-        );
-        double transY0 = rangeAxis.valueToJava2D(
-            y0, dataArea, plot.getRangeAxisEdge()
-        );
+        double transX0 = domainAxis.valueToJava2D(x0, dataArea, 
+                plot.getDomainAxisEdge());
+        double transY0 = rangeAxis.valueToJava2D(y0, dataArea, 
+                plot.getRangeAxisEdge());
         
         int itemCount = dataset.getItemCount(series);
-        double x2 = dataset.getXValue(
-            series, Math.min(item + 1, itemCount - 1)
-        );
-        double y2 = dataset.getYValue(
-            series, Math.min(item + 1, itemCount - 1)
-        );
+        double x2 = dataset.getXValue(series, Math.min(item + 1, 
+                itemCount - 1));
+        double y2 = dataset.getYValue(series, Math.min(item + 1, 
+                itemCount - 1));
         if (Double.isNaN(y2)) {
             y2 = 0.0;
         }
-        double transX2 = domainAxis.valueToJava2D(
-            x2, dataArea, plot.getDomainAxisEdge()
-        );
-        double transY2 = rangeAxis.valueToJava2D(
-            y2, dataArea, plot.getRangeAxisEdge()
-        );
+        double transX2 = domainAxis.valueToJava2D(x2, dataArea, 
+                plot.getDomainAxisEdge());
+        double transY2 = rangeAxis.valueToJava2D(y2, dataArea, 
+                plot.getRangeAxisEdge());
         
-        double transZero = rangeAxis.valueToJava2D(
-            0.0, dataArea, plot.getRangeAxisEdge()
-        );
+        double transZero = rangeAxis.valueToJava2D(0.0, dataArea, 
+                plot.getRangeAxisEdge());
         Polygon hotspot = null;
         if (plot.getOrientation() == PlotOrientation.HORIZONTAL) {
             hotspot = new Polygon();
-            hotspot.addPoint(
-                (int) transZero, (int) ((transX0 + transX1) / 2.0)
-            );
-            hotspot.addPoint(
-                (int) ((transY0 + transY1) / 2.0), 
-                (int) ((transX0 + transX1) / 2.0)
-            );
+            hotspot.addPoint((int) transZero, 
+                    (int) ((transX0 + transX1) / 2.0));
+            hotspot.addPoint((int) ((transY0 + transY1) / 2.0), 
+                    (int) ((transX0 + transX1) / 2.0));
             hotspot.addPoint((int) transY1, (int) transX1);
-            hotspot.addPoint(
-                (int) ((transY1 + transY2) / 2.0), 
-                (int) ((transX1 + transX2) / 2.0)
-            );
-            hotspot.addPoint(
-                (int) transZero, (int) ((transX1 + transX2) / 2.0)
-            );
+            hotspot.addPoint((int) ((transY1 + transY2) / 2.0), 
+                    (int) ((transX1 + transX2) / 2.0));
+            hotspot.addPoint((int) transZero, 
+                    (int) ((transX1 + transX2) / 2.0));
         }
         else {  // vertical orientation
             hotspot = new Polygon();
-            hotspot.addPoint(
-                (int) ((transX0 + transX1) / 2.0), (int) transZero
-            );
-            hotspot.addPoint(
-                (int) ((transX0 + transX1) / 2.0), 
-                (int) ((transY0 + transY1) / 2.0)
-            );
+            hotspot.addPoint((int) ((transX0 + transX1) / 2.0), 
+                    (int) transZero);
+            hotspot.addPoint((int) ((transX0 + transX1) / 2.0), 
+                    (int) ((transY0 + transY1) / 2.0));
             hotspot.addPoint((int) transX1, (int) transY1);
-            hotspot.addPoint(
-                (int) ((transX1 + transX2) / 2.0), 
-                (int) ((transY1 + transY2) / 2.0)
-            );
-            hotspot.addPoint(
-                (int) ((transX1 + transX2) / 2.0), (int) transZero
-            );
+            hotspot.addPoint((int) ((transX1 + transX2) / 2.0), 
+                    (int) ((transY1 + transY2) / 2.0));
+            hotspot.addPoint((int) ((transX1 + transX2) / 2.0), 
+                    (int) transZero);
         }
                 
         PlotOrientation orientation = plot.getOrientation();
@@ -378,14 +374,12 @@ public class XYAreaRenderer2 extends AbstractXYItemRenderer
         if (getPlotLines()) {
             if (item > 0) {
                 if (plot.getOrientation() == PlotOrientation.VERTICAL) {
-                    state.workingLine.setLine(
-                        transX0, transY0, transX1, transY1
-                    );
+                    state.workingLine.setLine(transX0, transY0, transX1, 
+                            transY1);
                 }
                 else if (plot.getOrientation() == PlotOrientation.HORIZONTAL) {
-                    state.workingLine.setLine(
-                        transY0, transX0, transY1, transX1
-                    );
+                    state.workingLine.setLine(transY0, transX0, transY1, 
+                            transX1);
                 }
                 g2.draw(state.workingLine);
             }
@@ -397,13 +391,14 @@ public class XYAreaRenderer2 extends AbstractXYItemRenderer
 
         // draw an outline around the Area.
         if (isOutline()) {
-            g2.setStroke(getSeriesOutlineStroke(series));
-            g2.setPaint(getSeriesOutlinePaint(series));
+            g2.setStroke(lookupSeriesOutlineStroke(series));
+            g2.setPaint(lookupSeriesOutlinePaint(series));
             g2.draw(hotspot);
         }
-        updateCrosshairValues(
-            crosshairState, x1, y1, transX1, transY1, orientation
-        );
+        int domainAxisIndex = plot.getDomainAxisIndex(domainAxis);
+        int rangeAxisIndex = plot.getRangeAxisIndex(rangeAxis);
+        updateCrosshairValues(crosshairState, x1, y1, domainAxisIndex, 
+                rangeAxisIndex, transX1, transY1, orientation);
         
         // collect entity and tool tip information...
         if (state.getInfo() != null) {
@@ -420,15 +415,38 @@ public class XYAreaRenderer2 extends AbstractXYItemRenderer
                 if (getURLGenerator() != null) {
                     url = getURLGenerator().generateURL(dataset, series, item);
                 }
-                XYItemEntity entity = new XYItemEntity(
-                    hotspot, dataset, series, item, tip, url
-                );
+                XYItemEntity entity = new XYItemEntity(hotspot, dataset, 
+                        series, item, tip, url);
                 entities.add(entity);
             }
         }
 
     }
 
+    /**
+     * Tests this renderer for equality with an arbitrary object.
+     * 
+     * @param obj  the object (<code>null</code> not permitted).
+     * 
+     * @return A boolean.
+     */
+    public boolean equals(Object obj) {
+        if (obj == this) {    
+            return true;
+        }
+        if (!(obj instanceof XYAreaRenderer2)) {
+            return false;
+        }
+        XYAreaRenderer2 that = (XYAreaRenderer2) obj;
+        if (this.showOutline != that.showOutline) {
+            return false;
+        }
+        if (!ShapeUtilities.equal(this.legendArea, that.legendArea)) {
+            return false;
+        }
+        return super.equals(obj);
+    }
+    
     /**
      * Returns a clone of the renderer.
      * 
@@ -437,7 +455,9 @@ public class XYAreaRenderer2 extends AbstractXYItemRenderer
      * @throws CloneNotSupportedException  if the renderer cannot be cloned.
      */
     public Object clone() throws CloneNotSupportedException {
-        return super.clone();
+        XYAreaRenderer2 clone = (XYAreaRenderer2) super.clone();
+        clone.legendArea = ShapeUtilities.clone(this.legendArea);
+        return clone;
     }
     
     /**
