@@ -2,7 +2,7 @@
  * JFreeChart : a free chart library for the Java(tm) platform
  * ===========================================================
  *
- * (C) Copyright 2000-2005, by Object Refinery Limited and Contributors.
+ * (C) Copyright 2000-2008, by Object Refinery Limited and Contributors.
  *
  * Project Info:  http://www.jfree.org/jfreechart/index.html
  *
@@ -27,13 +27,11 @@
  * -------------------------
  * TimeSeriesCollection.java
  * -------------------------
- * (C) Copyright 2001-2005, by Object Refinery Limited.
+ * (C) Copyright 2001-2008, by Object Refinery Limited.
  *
  * Original Author:  David Gilbert (for Object Refinery Limited);
  * Contributor(s):   -;
  *
- * $Id: TimeSeriesCollection.java,v 1.10.2.2 2005/12/13 17:55:06 mungady Exp $
- *
  * Changes
  * -------
  * 11-Oct-2001 : Version 1 (DG);
@@ -72,9 +70,14 @@
  * 11-Jan-2005 : Removed deprecated code in preparation for the 1.0.0 
  *               release (DG);
  * 28-Mar-2005 : Fixed bug in getSeries(int) method (1170825) (DG);
- * ------------- JFREECHART 1.0.0 ---------------------------------------------
+ * ------------- JFREECHART 1.0.x ---------------------------------------------
  * 13-Dec-2005 : Deprecated the 'domainIsPointsInTime' flag as it is 
  *               redundant.  Fixes bug 1243050 (DG);
+ * 04-May-2007 : Override getDomainOrder() to indicate that items are sorted
+ *               by x-value (ascending) (DG);
+ * 08-May-2007 : Added indexOf(TimeSeries) method (DG);
+ * 18-Jan-2008 : Changed getSeries(String) to getSeries(Comparable) (DG);
+ *
  */
 
 package org.jfree.data.time;
@@ -88,6 +91,7 @@ import java.util.List;
 import java.util.TimeZone;
 
 import org.jfree.data.DomainInfo;
+import org.jfree.data.DomainOrder;
 import org.jfree.data.Range;
 import org.jfree.data.general.DatasetChangeEvent;
 import org.jfree.data.xy.AbstractIntervalXYDataset;
@@ -214,6 +218,15 @@ public class TimeSeriesCollection extends AbstractIntervalXYDataset
         notifyListeners(new DatasetChangeEvent(this, this));    
     }
     
+    /**
+     * Returns the order of the domain values in this dataset.
+     *
+     * @return {@link DomainOrder#ASCENDING}
+     */
+    public DomainOrder getDomainOrder() {
+        return DomainOrder.ASCENDING;
+    }
+    
     /**
      * Returns the position within each time period that is used for the X 
      * value when the collection is used as an 
@@ -258,6 +271,23 @@ public class TimeSeriesCollection extends AbstractIntervalXYDataset
         return this.data.size();
     }
 
+    /**
+     * Returns the index of the specified series, or -1 if that series is not
+     * present in the dataset.
+     * 
+     * @param series  the series (<code>null</code> not permitted).
+     * 
+     * @return The series index.
+     * 
+     * @since 1.0.6
+     */
+    public int indexOf(TimeSeries series) {
+        if (series == null) {
+            throw new IllegalArgumentException("Null 'series' argument.");
+        }
+        return this.data.indexOf(series);
+    }
+
     /**
      * Returns a series.
      *
@@ -281,7 +311,7 @@ public class TimeSeriesCollection extends AbstractIntervalXYDataset
      * 
      * @return The series with the given key.
      */
-    public TimeSeries getSeries(String key) {
+    public TimeSeries getSeries(Comparable key) {
         TimeSeries result = null;
         Iterator iterator = this.data.iterator();
         while (iterator.hasNext()) {
@@ -413,7 +443,7 @@ public class TimeSeriesCollection extends AbstractIntervalXYDataset
     /**
      * Returns the x-value for a time period.
      *
-     * @param period  the time period.
+     * @param period  the time period (<code>null</code> not permitted).
      *
      * @return The x-value.
      */
