@@ -2,7 +2,7 @@
  * JFreeChart : a free chart library for the Java(tm) platform
  * ===========================================================
  *
- * (C) Copyright 2000-2007, by Object Refinery Limited and Contributors.
+ * (C) Copyright 2000-2008, by Object Refinery Limited and Contributors.
  *
  * Project Info:  http://www.jfree.org/jfreechart/index.html
  *
@@ -27,7 +27,7 @@
  * ------------------
  * SWTGraphics2D.java
  * ------------------
- * (C) Copyright 2006, 2007, by Henry Proudhon and Contributors.
+ * (C) Copyright 2006-2008, by Henry Proudhon and Contributors.
  *
  * Original Author:  Henry Proudhon (henry.proudhon AT ensmp.fr);
  * Contributor(s):   Cedric Chabanois (cchabanois AT no-log.org);
@@ -122,7 +122,10 @@ public class SWTGraphics2D extends Graphics2D {
     /** A HashMap to store the Swt font resources. */
     private Map fontsPool = new HashMap();
 
-    /** A HashMap to store the Swt color resources. */
+    /** A HashMap to store the Swt transform resources. */
+    private Map transformsPool = new HashMap();
+
+    /** A List to store the Swt resources. */
     private List resourcePool = new ArrayList();
 
     /**
@@ -280,7 +283,7 @@ public class SWTGraphics2D extends Graphics2D {
             AlphaComposite acomp = (AlphaComposite) this.composite;
             switch (acomp.getRule()) {
             case AlphaComposite.SRC_OVER:
-                this.gc.setAlpha((int) (color.getAlpha()*acomp.getAlpha()));
+                this.gc.setAlpha((int) (color.getAlpha() * acomp.getAlpha()));
                 break;
             default:
                 this.gc.setAlpha(color.getAlpha());
@@ -293,11 +296,8 @@ public class SWTGraphics2D extends Graphics2D {
      * @see java.awt.Graphics2D#setBackground(java.awt.Color)
      */
     public void setBackground(Color color) {
-        this.gc.getBackground().dispose();
-        org.eclipse.swt.graphics.Color swtColor = SWTUtils.toSwtColor(
-                this.gc.getDevice(), color);
+        org.eclipse.swt.graphics.Color swtColor = getSwtColorFromPool(color);
         this.gc.setBackground(swtColor);
-        swtColor.dispose(); 
     }
 
     /* (non-Javadoc)
@@ -342,7 +342,7 @@ public class SWTGraphics2D extends Graphics2D {
         this.composite = comp;
         if (comp instanceof AlphaComposite) {
             AlphaComposite acomp = (AlphaComposite) comp; 
-            int alpha = (int) (acomp.getAlpha()*0xFF);
+            int alpha = (int) (acomp.getAlpha() * 0xFF);
             this.gc.setAlpha(alpha);
         } 
         else {
@@ -487,7 +487,8 @@ public class SWTGraphics2D extends Graphics2D {
      * @see java.awt.Graphics2D#setTransform(java.awt.geom.AffineTransform)
      */
     public void setTransform(AffineTransform Tx) {
-        this.gc.setTransform(toSwtTransform(Tx));
+        Transform transform = getSwtTransformFromPool(Tx);
+        this.gc.setTransform(transform);
     }
 
     /* (non-Javadoc)
@@ -496,10 +497,8 @@ public class SWTGraphics2D extends Graphics2D {
     public void transform(AffineTransform Tx) {
         Transform swtTransform = new Transform(this.gc.getDevice()); 
         this.gc.getTransform(swtTransform);
-        Transform swtMatrix = toSwtTransform(Tx);
-        swtTransform.multiply(swtMatrix);
+        swtTransform.multiply(getSwtTransformFromPool(Tx));
         this.gc.setTransform(swtTransform);
-        swtMatrix.dispose();
         swtTransform.dispose();
     }
 
@@ -527,7 +526,7 @@ public class SWTGraphics2D extends Graphics2D {
     public void rotate(double theta) {
         Transform swtTransform = new Transform(this.gc.getDevice()); 
         this.gc.getTransform(swtTransform);
-        swtTransform.rotate( (float) (theta * 180 / Math.PI));
+        swtTransform.rotate((float) (theta * 180 / Math.PI));
         this.gc.setTransform(swtTransform);
         swtTransform.dispose();
     }
@@ -607,7 +606,7 @@ public class SWTGraphics2D extends Graphics2D {
     public void drawPolygon(int [] xPoints, int [] yPoints, int npoints) {
         drawPolyline(xPoints, yPoints, npoints);
         if (npoints > 1) {
-            this.gc.drawLine(xPoints[npoints-1], yPoints[npoints-1], 
+            this.gc.drawLine(xPoints[npoints - 1], yPoints[npoints - 1], 
                     xPoints[0], yPoints[0]);            
         }
     }
@@ -872,7 +871,7 @@ public class SWTGraphics2D extends Graphics2D {
      */
     public void drawString(String text, float x, float y) {
         float fm = this.gc.getFontMetrics().getAscent();
-        this.gc.drawString(text, (int) x, (int) ( y - fm ), true);
+        this.gc.drawString(text, (int) x, (int) (y - fm), true);
     }
 
     /* (non-Javadoc)
@@ -1100,8 +1099,9 @@ public class SWTGraphics2D extends Graphics2D {
             Resource resource = (Resource) it.next();
             resource.dispose();
         }
-        this.resourcePool.clear();
+        this.fontsPool.clear();
         this.colorsPool.clear();
+        this.transformsPool.clear();
         this.resourcePool.clear();
     }
 
@@ -1154,6 +1154,31 @@ public class SWTGraphics2D extends Graphics2D {
         return swtColor;
     }
 
+    /**
+     * Internal method to convert a AWT transform object into 
+     * a SWT transform resource. If a corresponding SWT transform
+     * instance is already in the pool, it will be used 
+     * instead of creating a new one. This is used in 
+     * {@link #setTransform()} for instance. 
+     * 
+     * @param awtTransform The AWT transform to convert.
+     * @return A SWT transform instance.
+     */
+    private Transform getSwtTransformFromPool(AffineTransform awtTransform) {
+    	Transform t = (Transform) this.transformsPool.get(awtTransform);
+    	if (t == null) {
+    		t = new Transform(this.gc.getDevice());
+            double[] matrix = new double[6];
+            awtTransform.getMatrix(matrix);
+            t.setElements((float) matrix[0], (float) matrix[1],
+                    (float) matrix[2], (float) matrix[3],
+                    (float) matrix[4], (float) matrix[5]); 
+    		addToResourcePool(t);
+    		this.transformsPool.put(awtTransform, t);
+    	}
+    	return t;
+    }
+    
     /**
      * Perform a switch between foreground and background 
      * color of gc. This is needed for consistency with 
@@ -1206,23 +1231,6 @@ public class SWTGraphics2D extends Graphics2D {
         return path;
     }
 
-    /**
-     * Converts an AWT transform into the equivalent SWT transform.
-     * 
-     * @param awtTransform  the AWT transform.
-     * 
-     * @return The SWT transform.
-     */
-    private Transform toSwtTransform(AffineTransform awtTransform) {
-        Transform t = new Transform(this.gc.getDevice());
-        double[] matrix = new double[6];
-        awtTransform.getMatrix(matrix);
-        t.setElements((float) matrix[0], (float) matrix[1],
-                (float) matrix[2], (float) matrix[3],
-                (float) matrix[4], (float) matrix[5]); 
-        return t;
-    }
-
     /**
      * Converts an SWT transform into the equivalent AWT transform.
      * 
@@ -1236,5 +1244,5 @@ public class SWTGraphics2D extends Graphics2D {
         AffineTransform awtTransform = new AffineTransform(elements);
         return awtTransform;
     }
-    
+
 }
