@@ -2,7 +2,7 @@
  * JFreeChart : a free chart library for the Java(tm) platform
  * ===========================================================
  *
- * (C) Copyright 2000-2007, by Object Refinery Limited and Contributors.
+ * (C) Copyright 2000-2008, by Object Refinery Limited and Contributors.
  *
  * Project Info:  http://www.jfree.org/jfreechart/index.html
  *
@@ -27,7 +27,7 @@
  * --------------------------
  * DatasetUtilitiesTests.java
  * --------------------------
- * (C) Copyright 2003-2007, by Object Refinery Limited and Contributors.
+ * (C) Copyright 2003-2008, by Object Refinery Limited and Contributors.
  *
  * Original Author:  David Gilbert (for Object Refinery Limited);
  * Contributor(s):   -;
@@ -40,6 +40,7 @@
  * 07-Jan-2005 : Updated for method name changes (DG);
  * 03-Feb-2005 : Added testFindStackedRangeBounds2() method (DG);
  * 26-Sep-2007 : Added testIsEmptyOrNullXYDataset() method (DG);
+ * 28-Mar-2008 : Added and renamed various tests (DG);
  * 
  */
 
@@ -53,14 +54,20 @@ import org.jfree.data.KeyToGroupMap;
 import org.jfree.data.Range;
 import org.jfree.data.category.CategoryDataset;
 import org.jfree.data.category.DefaultCategoryDataset;
+import org.jfree.data.category.DefaultIntervalCategoryDataset;
+import org.jfree.data.function.Function2D;
+import org.jfree.data.function.LineFunction2D;
 import org.jfree.data.general.DatasetUtilities;
 import org.jfree.data.general.DefaultPieDataset;
 import org.jfree.data.general.PieDataset;
 import org.jfree.data.xy.DefaultTableXYDataset;
+import org.jfree.data.xy.IntervalXYDataset;
 import org.jfree.data.xy.TableXYDataset;
 import org.jfree.data.xy.XYDataset;
 import org.jfree.data.xy.XYSeries;
 import org.jfree.data.xy.XYSeriesCollection;
+import org.jfree.data.xy.YIntervalSeries;
+import org.jfree.data.xy.YIntervalSeriesCollection;
 
 /**
  * Tests for the {@link DatasetUtilities} class.
@@ -151,21 +158,125 @@ public class DatasetUtilitiesTests extends TestCase {
     }
     
     /**
-     * Some tests for the iterateCategoryRangeBounds() method.
+     * A test for the findRangeBounds(XYDataset) method using
+     * an IntervalXYDataset.
      */
-    public void testIterateCategoryRangeBounds() {
+    public void testFindRangeBounds3() {
+        YIntervalSeriesCollection dataset = new YIntervalSeriesCollection();
+        Range r = DatasetUtilities.findRangeBounds(dataset);
+        assertNull(r);
+        YIntervalSeries s1 = new YIntervalSeries("S1");
+        dataset.addSeries(s1);
+        r = DatasetUtilities.findRangeBounds(dataset);
+        assertNull(r);
+        
+        // try a single item
+        s1.add(1.0, 2.0, 1.5, 2.5);
+        r = DatasetUtilities.findRangeBounds(dataset);
+        assertEquals(1.5, r.getLowerBound(), EPSILON);
+        assertEquals(2.5, r.getUpperBound(), EPSILON);
+        
+        // another item
+        s1.add(2.0, 2.0, 1.4, 2.1);
+        r = DatasetUtilities.findRangeBounds(dataset);
+        assertEquals(1.4, r.getLowerBound(), EPSILON);
+        assertEquals(2.5, r.getUpperBound(), EPSILON);
+        
+        // another empty series
+        YIntervalSeries s2 = new YIntervalSeries("S2");
+        dataset.addSeries(s2);
+        r = DatasetUtilities.findRangeBounds(dataset);
+        assertEquals(1.4, r.getLowerBound(), EPSILON);
+        assertEquals(2.5, r.getUpperBound(), EPSILON);
+        
+        // an item in series 2
+        s2.add(1.0, 2.0, 1.9, 2.6);
+        r = DatasetUtilities.findRangeBounds(dataset);
+        assertEquals(1.4, r.getLowerBound(), EPSILON);
+        assertEquals(2.6, r.getUpperBound(), EPSILON);
+        
+        // what if we don't want the interval?
+        r = DatasetUtilities.findRangeBounds(dataset, false);
+        assertEquals(2.0, r.getLowerBound(), EPSILON);
+        assertEquals(2.0, r.getUpperBound(), EPSILON);
+        
+    }
+    
+    /**
+     * Some tests for the iterateRangeBounds() method.
+     */
+    public void testIterateRangeBounds_CategoryDataset() {
         CategoryDataset dataset = createCategoryDataset1();
-        Range r = DatasetUtilities.iterateCategoryRangeBounds(dataset, false);
+        Range r = DatasetUtilities.iterateRangeBounds(dataset, false);
         assertEquals(1.0, r.getLowerBound(), EPSILON);
         assertEquals(6.0, r.getUpperBound(), EPSILON);           
     }
+    
+    /**
+     * Some checks for the iterateRangeBounds() method.
+     */
+    public void testIterateRangeBounds2_CategoryDataset() {
+        // an empty dataset should return a null range
+        DefaultCategoryDataset dataset = new DefaultCategoryDataset();
+        Range r = DatasetUtilities.iterateRangeBounds(dataset, false);
+        assertNull(r);
+        
+        // a dataset with a single value
+        dataset.addValue(1.23, "R1", "C1");
+        r = DatasetUtilities.iterateRangeBounds(dataset, false);
+        assertEquals(1.23, r.getLowerBound(), EPSILON);
+        assertEquals(1.23, r.getUpperBound(), EPSILON);
+        
+        // null is ignored
+        dataset.addValue(null, "R2", "C1");
+        r = DatasetUtilities.iterateRangeBounds(dataset, false);
+        assertEquals(1.23, r.getLowerBound(), EPSILON);
+        assertEquals(1.23, r.getUpperBound(), EPSILON);
+        
+        // a Double.NaN messes things up
+        dataset.addValue(Double.NaN, "R2", "C1");
+        r = DatasetUtilities.iterateRangeBounds(dataset, false);
+        assertTrue(Double.isNaN(r.getLowerBound()));
+        assertTrue(Double.isNaN(r.getUpperBound()));
+    }
+    
+    /**
+     * Some checks for the iterateRangeBounds() method using an
+     * IntervalCategoryDataset.
+     */
+    public void testIterateRangeBounds3_CategoryDataset() {
+        Number[][] starts = new Double[2][3];
+        Number[][] ends = new Double[2][3];
+        starts[0][0] = new Double(1.0);
+        starts[0][1] = new Double(2.0);
+        starts[0][2] = new Double(3.0);
+        starts[1][0] = new Double(11.0);
+        starts[1][1] = new Double(12.0);
+        starts[1][2] = new Double(13.0);
+        ends[0][0] = new Double(4.0);
+        ends[0][1] = new Double(5.0);
+        ends[0][2] = new Double(6.0);
+        ends[1][0] = new Double(16.0);
+        ends[1][1] = new Double(15.0);
+        ends[1][2] = new Double(14.0);
+        
+        DefaultIntervalCategoryDataset d = new DefaultIntervalCategoryDataset(
+                starts, ends);
+        Range r = DatasetUtilities.iterateRangeBounds(d, false);
+        assertEquals(4.0, r.getLowerBound(), EPSILON);
+        assertEquals(16.0, r.getUpperBound(), EPSILON);
+        r = DatasetUtilities.iterateRangeBounds(d, true);
+        assertEquals(1.0, r.getLowerBound(), EPSILON);
+        assertEquals(16.0, r.getUpperBound(), EPSILON);
+
+    }
 
     /**
-     * Some tests for the iterateXYRangeBounds() method.
+     * Some tests for the iterateRangeBounds() method.
      */
-    public void testIterateXYRangeBounds() {
+    public void testIterateRangeBounds() {
         XYDataset dataset = createXYDataset1();
-        Range r = DatasetUtilities.iterateXYRangeBounds(dataset);
+        Range r = DatasetUtilities.iterateRangeBounds(dataset);
         assertEquals(100.0, r.getLowerBound(), EPSILON);
         assertEquals(105.0, r.getUpperBound(), EPSILON);           
     }
@@ -173,17 +284,88 @@ public class DatasetUtilitiesTests extends TestCase {
     /**
      * Check the range returned when a series contains a null value.
      */
-    public void testIterateXYRangeBounds2() {
+    public void testIterateRangeBounds2() {
         XYSeries s1 = new XYSeries("S1");
         s1.add(1.0, 1.1);
         s1.add(2.0, null);
         s1.add(3.0, 3.3);
         XYSeriesCollection dataset = new XYSeriesCollection(s1);
-        Range r = DatasetUtilities.iterateXYRangeBounds(dataset);
+        Range r = DatasetUtilities.iterateRangeBounds(dataset);
         assertEquals(1.1, r.getLowerBound(), EPSILON);
         assertEquals(3.3, r.getUpperBound(), EPSILON);
     }
     
+    /**
+     * Some checks for the iterateRangeBounds() method.
+     */
+    public void testIterateRangeBounds3() {
+        // an empty dataset should return a null range
+        XYSeriesCollection dataset = new XYSeriesCollection();
+        Range r = DatasetUtilities.iterateRangeBounds(dataset);
+        assertNull(r);
+        XYSeries s1 = new XYSeries("S1");
+        dataset.addSeries(s1);
+        r = DatasetUtilities.iterateRangeBounds(dataset);
+        assertNull(r);
+        
+        // a dataset with a single value
+        s1.add(1.0, 1.23);
+        r = DatasetUtilities.iterateRangeBounds(dataset);
+        assertEquals(1.23, r.getLowerBound(), EPSILON);
+        assertEquals(1.23, r.getUpperBound(), EPSILON);
+        
+        // null is ignored
+        s1.add(2.0, null);
+        r = DatasetUtilities.iterateRangeBounds(dataset);
+        assertEquals(1.23, r.getLowerBound(), EPSILON);
+        assertEquals(1.23, r.getUpperBound(), EPSILON);
+        
+        // Double.NaN DOESN'T mess things up
+        s1.add(3.0, Double.NaN);
+        r = DatasetUtilities.iterateRangeBounds(dataset);
+        assertEquals(1.23, r.getLowerBound(), EPSILON);
+        assertEquals(1.23, r.getUpperBound(), EPSILON);
+    }
+    
+    /**
+     * Some checks for the range bounds of a dataset that implements the
+     * {@link IntervalXYDataset} interface.
+     */
+    public void testIterateRangeBounds4() {
+        YIntervalSeriesCollection dataset = new YIntervalSeriesCollection();
+        Range r = DatasetUtilities.iterateRangeBounds(dataset);
+        assertNull(r);
+        YIntervalSeries s1 = new YIntervalSeries("S1");
+        dataset.addSeries(s1);
+        r = DatasetUtilities.iterateRangeBounds(dataset);
+        assertNull(r);
+        
+        // try a single item
+        s1.add(1.0, 2.0, 1.5, 2.5);
+        r = DatasetUtilities.iterateRangeBounds(dataset);
+        assertEquals(1.5, r.getLowerBound(), EPSILON);
+        assertEquals(2.5, r.getUpperBound(), EPSILON);
+        
+        // another item
+        s1.add(2.0, 2.0, 1.4, 2.1);
+        r = DatasetUtilities.iterateRangeBounds(dataset);
+        assertEquals(1.4, r.getLowerBound(), EPSILON);
+        assertEquals(2.5, r.getUpperBound(), EPSILON);
+        
+        // another empty series
+        YIntervalSeries s2 = new YIntervalSeries("S2");
+        dataset.addSeries(s2);
+        r = DatasetUtilities.iterateRangeBounds(dataset);
+        assertEquals(1.4, r.getLowerBound(), EPSILON);
+        assertEquals(2.5, r.getUpperBound(), EPSILON);
+        
+        // an item in series 2
+        s2.add(1.0, 2.0, 1.9, 2.6);
+        r = DatasetUtilities.iterateRangeBounds(dataset);
+        assertEquals(1.4, r.getLowerBound(), EPSILON);
+        assertEquals(2.6, r.getUpperBound(), EPSILON);
+    }
+
     /**
      * Some tests for the findMinimumDomainValue() method.
      */
@@ -287,6 +469,21 @@ public class DatasetUtilitiesTests extends TestCase {
         assertEquals(23.39, range.getUpperBound(), 0.00000001);
     }
     
+    /**
+     * A further test for the cumulative range calculation.
+     */
+    public void testCumulativeRange3() {
+        DefaultCategoryDataset dataset = new DefaultCategoryDataset();
+        dataset.addValue(15.76, "Product 1", "Labour");
+        dataset.addValue(8.66, "Product 1", "Administration");
+        dataset.addValue(4.71, "Product 1", "Marketing");
+        dataset.addValue(3.51, "Product 1", "Distribution");
+        dataset.addValue(32.64, "Product 1", "Total Expense");        
+        Range range = DatasetUtilities.findCumulativeRangeBounds(dataset);
+        assertEquals(0.0, range.getLowerBound(), 0.00000001);
+        assertEquals(65.28, range.getUpperBound(), 0.00000001);
+    }
+    
     /**
      * Test the creation of a dataset from an array.
      */
@@ -457,6 +654,22 @@ public class DatasetUtilitiesTests extends TestCase {
         
     }
     
+    /**
+     * Some checks for the sampleFunction2D() method.
+     */
+    public void testSampleFunction2D() {
+        Function2D f = new LineFunction2D(0, 1);
+        XYDataset dataset = DatasetUtilities.sampleFunction2D(f, 0.0, 1.0, 2, 
+                "S1");
+        assertEquals(1, dataset.getSeriesCount());
+        assertEquals("S1", dataset.getSeriesKey(0));
+        assertEquals(2, dataset.getItemCount(0));
+        assertEquals(0.0, dataset.getXValue(0, 0), EPSILON);
+        assertEquals(0.0, dataset.getYValue(0, 0), EPSILON);
+        assertEquals(1.0, dataset.getXValue(0, 1), EPSILON);
+        assertEquals(1.0, dataset.getYValue(0, 1), EPSILON);
+    }
+    
     /**
      * Creates a dataset for testing. 
      * 
